#include "stdafx.h"
#if !defined( _WIN32_WCE )
#include <process.h>	// _beginthread, _endthread
#endif
#include <commctrl.h>
#include <stdio.h>
#include <stdlib.h>
#include <TCHAR.H>
#include "wincemem.h"
#include "MyTime.h"
#include "resource.h"
#include "MySock.h"
#include "MyString.h"
#include "MyHashFile.h"
#include "str.h"

#define MAX_LOADSTRING 100

#define CONFIG_FILE	L"iloggerconf.txt"

#define CONF_LOGFILENAME		"logfilename"
#define CONF_MAX_LOGSIZE		"logsize"
#define CONF_MAX_DISP_LINE		"displine"
#define CONF_LISTEN_PORT		"port"
#define CONF_ALLOWED_SENDER		"allowfrom"
#define CONF_CASHSIZE			"cashsize"

#define LOGSEVERITY_EMERG	0
#define LOGSEVERITY_ALERT	1
#define LOGSEVERITY_CRIT	2
#define LOGSEVERITY_ERR		3
#define LOGSEVERITY_WARNING	4
#define LOGSEVERITY_NOTICE	5
#define LOGSEVERITY_INFO	6
#define LOGSEVERITY_DEBUG	7
#define LOGSEVERITY_LAST	7

const char* pszSeverity[9] =
{
	"EMG:", "ART:", "CRT:", "ERR:", "WRN:", "NTC:", "INF:", "DBG:", ""
};

HINSTANCE			g_hInst;
HWND				g_hwndCB = NULL;
HWND				g_hwndOutput = NULL;
HMENU				g_hMenu = NULL;
HANDLE				g_hMutex;
int					g_bQuit = FALSE;
u_short				g_unPort = 514;	// syslog port
unsigned int		g_dwSeverity = LOGSEVERITY_INFO;
MyHashFile			g_conf;
WCHAR				g_wszLogFile[MAX_PATH+1];

static ATOM				MyRegisterClass	(HINSTANCE, LPTSTR);
static BOOL				InitInstance	(HINSTANCE, int);
static LRESULT CALLBACK	WndProc			(HWND, UINT, WPARAM, LPARAM);
static LRESULT CALLBACK	About			(HWND, UINT, WPARAM, LPARAM);
static LRESULT CALLBACK	SettingDlg		(HWND, UINT, WPARAM, LPARAM);
static unsigned int ATOUI( const char* src );

void MessageW( HWND hwndText, const WCHAR* pwszMsg );
void MessageA( HWND hwndText, const char* pszMsg );
static void thMessage(void* pvParam);
static void LogFile(HANDLE* hFile, const char* msg, int size);
static int LoadConfig(MyHashFile* phfConfig);


#if !defined(_WIN32_WCE)
int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow )
{
	if( NULL != lpCmdLine && *lpCmdLine != '\0' )
	{
		g_unPort = atoi( lpCmdLine );
	}

#else
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	if( NULL != lpCmdLine && *lpCmdLine != '\0' )
	{
		g_unPort = (u_short)_wtoi( lpCmdLine );
	}

#endif
	MSG msg;
	HACCEL hAccelTable;
	TCHAR	szWindowClass[MAX_LOADSTRING];

	g_hInst = hInstance;
	LoadString(hInstance, IDC_ILOGGER, szWindowClass, MAX_LOADSTRING);

	g_hMutex = CreateMutex(NULL,FALSE,szWindowClass);
	if( GetLastError() == ERROR_ALREADY_EXISTS )
	{
		HWND hWnd = FindWindow( szWindowClass, NULL );
		if( hWnd )
		{
			ShowWindow(hWnd, SW_SHOW);
			SetForegroundWindow(hWnd);
		}
		return FALSE;
	}


	if (!InitInstance (hInstance, SW_SHOW/*nCmdShow*/)) 
	{
		return FALSE;
	}

	hAccelTable = LoadAccelerators(hInstance, (LPCTSTR)IDC_ILOGGER);

	while (GetMessage(&msg, NULL, 0, 0)) 
	{
		if (!TranslateAccelerator(msg.hwnd, hAccelTable, &msg)) 
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	return msg.wParam;
}

// ============================================================================
// MyRegisterClass
// ============================================================================
static ATOM MyRegisterClass(HINSTANCE hInstance, LPTSTR szWindowClass)
{
	WNDCLASS	wc;

	wc.style			= CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc		= (WNDPROC) WndProc;
	wc.cbClsExtra		= 0;
	wc.cbWndExtra		= 0;
	wc.hInstance		= hInstance;
	wc.hIcon			= LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ILOGGER));
	wc.hCursor			= 0;
	wc.hbrBackground	= (HBRUSH) GetStockObject(WHITE_BRUSH);
#if defined(_WIN32_WCE)
	wc.lpszMenuName		= 0;
#else
	wc.lpszMenuName		= MAKEINTRESOURCE(IDC_ILOGGER);
#endif
	wc.lpszClassName	= szWindowClass;

	return RegisterClass(&wc);
}

// ============================================================================
// InitInstance
// ============================================================================
static BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
	HWND	hWnd;
	TCHAR	szTitle[MAX_LOADSTRING];
	TCHAR	szWindowClass[MAX_LOADSTRING];

	g_hInst = hInstance;
	LoadString(hInstance, IDC_ILOGGER, szWindowClass, MAX_LOADSTRING);
	MyRegisterClass(hInstance, szWindowClass);
	LoadString(hInstance, IDC_ILOGGER, szTitle, MAX_LOADSTRING);
#if defined(_WIN32_WCE)
	hWnd = CreateWindow(szWindowClass, szTitle, WS_VISIBLE, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, hInstance, NULL);
#else
	hWnd = CreateWindow(szWindowClass, szTitle, WS_VISIBLE | WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, hInstance, NULL);
#endif
	if (!hWnd)
	{	
		return FALSE;
	}

	ShowWindow(hWnd, nCmdShow);
	UpdateWindow(hWnd);
#if defined(_WIN32_WCE)
	if (g_hwndCB)
		CommandBar_Show(g_hwndCB, TRUE);
#endif
	return TRUE;
}

// ============================================================================
// WndProc
// ============================================================================
static LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int wmId, wmEvent;
	MENUITEMINFO mii;
	mii.fMask = MIIM_STATE;
	int ii;

	switch (message) 
	{
		case WM_COMMAND:

			wmId    = LOWORD(wParam);
			wmEvent = HIWORD(wParam);

			switch (wmId)
			{
				case IDM_HELP_ABOUT:
					DialogBox(g_hInst, (LPCTSTR)IDD_ABOUTBOX, hWnd, (DLGPROC)About);
					return TRUE;

				case IDM_FILE_EXIT:
					DestroyWindow(hWnd);
					return TRUE;
				case ID_FILE_CLEAR:
					SetWindowText( g_hwndOutput, L"" );
					return TRUE;
				case  ID_SEV_DEBUG:
				case  ID_SEV_INFO:
				case  ID_SEV_NOTICE:
				case  ID_SEV_WARNING:
				case  ID_SEV_ERR:
				case  ID_SEV_CRIT:
				case  ID_SEV_ALERT:
				case  ID_SEV_EMERG:
					for( ii = 0; ii <= LOGSEVERITY_LAST; ii++ )
					{
						/*
						mii.cbSize = sizeof( MENUITEMINFO );
						mii.fMask  = MIIM_STATE;
						mii.fState = MFS_UNCHECKED;
						SetMenuItemInfo(g_hMenu, ID_SEV_EMERG+ii, FALSE, &mii);
						*/
						CheckMenuItem(g_hMenu,ID_SEV_EMERG+ii,MF_UNCHECKED);
					}
					/*
					mii.fState = MFS_CHECKED;
					SetMenuItemInfo(g_hMenu , wmId , FALSE , &mii);
					*/
					CheckMenuItem(g_hMenu,wmId,MF_CHECKED);
					g_dwSeverity = wmId - ID_SEV_EMERG;
					return TRUE;

				default:
					break;
			}
			break;
		case WM_CTLCOLORSTATIC:
		{
			HDC  hdcStatic = (HDC) wParam;
			HWND hwndStatic = (HWND) lParam;
			SetBkMode(hdcStatic,OPAQUE);
			return (long)GetStockObject(WHITE_BRUSH);
		}
		case WM_SIZE:
		{
			RECT rcWnd = {0};
			RECT rcMenu = {0};
#if defined(_WIN32_WCE)
			GetClientRect( g_hwndCB, &rcMenu );
#endif
			GetClientRect( hWnd, &rcWnd );

			MoveWindow( g_hwndOutput, 0, rcMenu.bottom, rcWnd.right, rcWnd.bottom - rcMenu.bottom, TRUE );

			return 0;
		}	

		case WM_CREATE:
			LoadConfig( &g_conf );
			g_unPort = (unsigned short)ATOUI( g_conf.GetValueI(CONF_LISTEN_PORT) );

			if( 0 == MultiByteToWideChar( CP_ACP, 0, g_conf.GetValueI(CONF_LOGFILENAME), STRLEN(g_conf.GetValueI(CONF_LOGFILENAME)), g_wszLogFile, MAX_PATH ) )
			{
				wcscpy(g_wszLogFile, L"ilog.txt");
			}
			
#if defined(_WIN32_WCE)
			g_hwndCB = CommandBar_Create(g_hInst, hWnd, 1);			
			CommandBar_InsertMenubar( g_hwndCB, g_hInst, IDC_ILOGGER, 0 );
			CommandBar_AddAdornments( g_hwndCB, 0, 0 );
			g_hMenu = CommandBar_GetMenu(g_hwndCB, 0);
#else
			g_hMenu = GetMenu(hWnd);
#endif

			mii.cbSize = sizeof( MENUITEMINFO );
			mii.fMask  = MIIM_STATE;
			mii.fState = MFS_CHECKED;
			SetMenuItemInfo(g_hMenu, g_dwSeverity + ID_SEV_EMERG, FALSE, &mii);
			
			// make a edit control
			g_hwndOutput = CreateWindow(L"EDIT", NULL, 
				WS_BORDER | WS_VISIBLE | WS_CHILD | ES_READONLY | ES_MULTILINE | ES_NOHIDESEL | WS_VSCROLL, 0, 0, 0, 0, hWnd, NULL, g_hInst, NULL);
			ShowWindow(g_hwndOutput, SW_SHOW);

			// start listen thread
			g_bQuit = FALSE;
			DWORD dwThreadID;
			CreateThread( NULL, 0, (LPTHREAD_START_ROUTINE)thMessage, NULL, 0, &dwThreadID );

			break;

		case WM_DESTROY:

#if defined(_WIN32_WCE)
			CommandBar_Destroy(g_hwndCB);
#endif
			g_bQuit = TRUE;
			do
			{
				Sleep(100);
			}while( TRUE == g_bQuit );

			PostQuitMessage(0);
			CloseHandle( g_hMutex );
			break;

		default:

			return DefWindowProc(hWnd, message, wParam, lParam);
   }
   return 0;
}

// ============================================================================
// About box
// ============================================================================
static LRESULT CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	RECT rt, rt1;
	int DlgWidth, DlgHeight;
	int NewPosX, NewPosY;
	u_short tmpPort;

	switch (message)
	{
	case WM_INITDIALOG:
		if (GetWindowRect(hDlg, &rt1))
		{
			GetClientRect(GetParent(hDlg), &rt);
			DlgWidth	= rt1.right - rt1.left;
			DlgHeight	= rt1.bottom - rt1.top ;
			NewPosX		= (rt.right - rt.left - DlgWidth)/2;
			NewPosY		= (rt.bottom - rt.top - DlgHeight)/2;
			
			if (NewPosX < 0) NewPosX = 0;
			if (NewPosY < 0) NewPosY = 0;
			SetWindowPos(hDlg, 0, NewPosX, NewPosY,
				0, 0, SWP_NOZORDER | SWP_NOSIZE);
		}
		SetDlgItemInt( hDlg, IDC_EDIT_PORT, g_unPort, TRUE );
		return TRUE;
	case WM_COMMAND:
		switch( LOWORD(wParam) )
		{
		case IDOK:
			switch( LOWORD(wParam) )
			{
			case IDOK:
				tmpPort = (u_short)GetDlgItemInt( hDlg, IDC_EDIT_PORT, NULL, TRUE );

				if( g_unPort != tmpPort )
				{
					g_unPort = tmpPort;
					g_bQuit = TRUE;
					do
					{
						Sleep(100);
					}while( TRUE == g_bQuit );

					DWORD dwThreadID;
					CreateThread( NULL, 0, (LPTHREAD_START_ROUTINE)thMessage, NULL, 0, &dwThreadID );
				}
				EndDialog(hDlg, LOWORD(wParam));
				return TRUE;

			case IDCANCEL:
				EndDialog(hDlg, LOWORD(wParam));
				return TRUE;

			}
			break;
		}

	default:
		break;
	}
		
    return FALSE;
}

// ============================================================================
// Message recv thread
// ============================================================================
static void thMessage(void* pvParam)
{
	char buf[1024+2];
	MySock s(SOCK_DGRAM);
	unsigned int sev;
	int nRecvLen;
	const char* pszSender = g_conf.GetValueI( CONF_ALLOWED_SENDER );
	int nSender = STRLEN( CONF_ALLOWED_SENDER );
	
	// ------------------------------------------------------------------------
	// create file
	// ------------------------------------------------------------------------
	HANDLE hFile;
	hFile = CreateFile( g_wszLogFile, GENERIC_WRITE, FILE_SHARE_READ, NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
	if( hFile == INVALID_HANDLE_VALUE )
	{
		return;
	}
	
	if( SOCKET_ERROR == s.Bind( g_unPort ) )
	{
		MessageA( g_hwndOutput, "Bind error.\r\n" );
	}
	else
	{
		sprintf( buf, "Listen at %d\r\n", g_unPort );
		MessageA( g_hwndOutput, buf );
	}
	
	s.SetTimeOut( 1 );
	MyVarMem vmCash;

	while( FALSE == g_bQuit )
	{
		memset( buf, 0, 1024+2 );
		sockaddr_in saddr;
		int saddrsize = sizeof(sockaddr);

		nRecvLen = s.RecvFrom( buf, 1024, 0, (sockaddr*)&saddr, &saddrsize );

		if( 0 == nRecvLen ) // timeout every 3 sec
		{
			char* pszCash = (char*)vmCash.GetBuffer();
			size_t dwLength = vmCash.GetLength();

			if( 0 == dwLength || NULL == pszCash )
				continue;

			LogFile( &hFile, pszCash, dwLength );

			char* p = pszCash;
			while( '\0' != *p )
			{
				if( (*p > 126 || *p < 32) && *p != '\r' && *p != '\n')
					*p = '?';
				p++;
			}
			MessageA( g_hwndOutput, pszCash );
			vmCash.Clear();
		}

		if( pszSender[0] != '*' )
		{
			char* pszRemoteHost = inet_ntoa(saddr.sin_addr);
			if( NULL == pszRemoteHost )
				continue;

			if( 0 != STRNCMP( pszRemoteHost, pszSender, nSender ) )
				continue;
		}

		if( 0 < nRecvLen )
		{
			if( '<' == buf[0] && ('>' == buf[2]||'>' == buf[3]||'>' == buf[4]) )
			{
				sev = ATOUI( &buf[1] );
				sev = sev & 7;
			}

			if( sev <= g_dwSeverity )
			{
				STRCAT( buf, "\r\n" );
				vmCash.Add( pszSeverity[sev] );
				vmCash.Add( buf );
			}
		}
	}

	s.Close();

	// ------------------------------------------------------------------------
	// close file
	// ------------------------------------------------------------------------
	CloseHandle( hFile );
	hFile = INVALID_HANDLE_VALUE;
	
	g_bQuit = FALSE;
}

// ============================================================================
// logging to file
// ============================================================================
static void LogFile(HANDLE* phFile, const char* msg, int size)
{
	if( INVALID_HANDLE_VALUE == *phFile )
	{
		*phFile = CreateFile( g_wszLogFile, GENERIC_WRITE, FILE_SHARE_READ, NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);

		if( *phFile == INVALID_HANDLE_VALUE )
		{
			MessageW(g_hwndOutput, L"Create log file failed!!\r\n" );
			return;
		}
	}
	DWORD dwWritten;
	WriteFile( *phFile, msg, size, &dwWritten, NULL );
	DWORD dwFileSize = GetFileSize( *phFile, NULL ); 

	// change backup if necessary
	// original file size reached max size
	if( dwFileSize > (DWORD)ATOUI(g_conf.GetValueI(CONF_MAX_LOGSIZE) ) )
	{
		CloseHandle( *phFile );
		*phFile = INVALID_HANDLE_VALUE;

		WCHAR	wszBackUp[MAX_PATH+1] = {0};
		wcsncpy( wszBackUp, g_wszLogFile, MAX_PATH );
		*wszBackUp = '-';						// last word of original file is changed to '_'
		DeleteFile( wszBackUp );				// delete backup file
		MoveFile( g_wszLogFile, wszBackUp );	// rename original file to backup file
		DeleteFile( g_wszLogFile );				// delete backup file
	}

	// ------------------------------------------------------------------------
	return;
}
// ============================================================================
// MessageWidechar
// ============================================================================
void MessageW( HWND hwndText, const WCHAR* pwszMsg )
{
	DWORD len;
	DWORD line;
	DWORD max_line = ATOUI( g_conf.GetValueI( CONF_MAX_DISP_LINE ) );
	if(max_line < 8) max_line = 8;

	if( !IsWindow( hwndText ) )
		return;

	line = SendMessage( hwndText, EM_GETLINECOUNT, 0, 0 );
	
	if( line > max_line )
	{
		// delete 1/8 line
		len = SendMessage( hwndText, EM_LINEINDEX, (max_line/8), 0 );
		SendMessage( hwndText, EM_SETSEL, 0, len );
		SendMessage( hwndText, EM_REPLACESEL, 0, (LPARAM)(LPCTSTR)L"" );
	}

	len = GetWindowTextLength( hwndText );
	SendMessage( hwndText, EM_SETSEL, len, len );
	SendMessage( hwndText, EM_REPLACESEL, 0, (LPARAM)(LPCTSTR)pwszMsg );

}

// ============================================================================
// MessageAscii
// ============================================================================
void MessageA( HWND hwndText, const char* pszMsg )
{
	int len = 0;
	WCHAR* pwszBuf;

	len = MultiByteToWideChar( CP_ACP, 0, pszMsg, -1, NULL, 0);

	pwszBuf = (WCHAR*)MALLOC(len * sizeof(WCHAR));
	
	if( NULL == pwszBuf )
		return;

	if( 0 == MultiByteToWideChar(CP_ACP,0,pszMsg,-1,pwszBuf,len) )
	{
		FREEIF( pwszBuf );
		return;
	}

	MessageW( hwndText, pwszBuf );
	FREEIF( pwszBuf );
}

static unsigned int ATOUI( const char* src )
{
	unsigned int ret = 0;

	if( NULL == src )
		return 0;

	while( (*src > '9' && '0' > *src) && *src != '\0' )  src++;

	while( '0' <= *src && *src <= '9' )
	{
		ret *= 10;
		ret += (*src - '0');
		src++;
	}

	return ret;
}

static int LoadConfig(MyHashFile* phfConfig)
{
	g_conf.Set( CONF_LOGFILENAME, "maillog.txt" );
	g_conf.Set( CONF_MAX_LOGSIZE, "10240" );
	g_conf.Set( CONF_MAX_DISP_LINE, "200" );
	g_conf.Set( CONF_LISTEN_PORT, "514" );
	g_conf.Set( CONF_ALLOWED_SENDER, "127.0.0.1" );
	g_conf.Set( CONF_CASHSIZE, "4096" );

	// ----------------------------------------------------
	// make full path
	// ----------------------------------------------------
	WCHAR wszFileName[MAX_PATH+1] = {0};
	WCHAR* pFN = &wszFileName[0];
	WCHAR* pN = CONFIG_FILE;
	GetModuleFileName( NULL, wszFileName, MAX_PATH );

	while( *pFN != '\0' ) {pFN++;}
	while( *pFN != '\\' && pFN != &wszFileName[0] ) { *pFN-- = '\0';}
	
	while( *pN != '\0' ) *pFN++ = *pN++;

	if( 0 == phfConfig->LoadFile( CONFIG_FILE ) )
		return 0;

	return 1;
}

