/* =========================================================================
	This library is part of IshiSoft application.
	------------------------------------------------------------------------
	author: Yuichiro Ishitani
			http://www001.upp.so-net.ne.jp/ishi/index.html
  ========================================================================== */
#include "stdafx.h"
#include "wincemem.h"
#include "MXRecord.h"
#include <Winsock.h>

#if !defined(_WIN32_WCE)
#pragma comment( lib , "ws2_32.lib" )
#else
#pragma comment( lib , "winsock.lib" )
#endif

// local define
#define DNS_PORTNO					(53)		// DNS well known port
#define DNS_PACKET_SIZE				(512)		// Max dns system packet size
#define DNS_NAME_SIZE				(256)		// Max domain name length

#define DNS_RCODE_NOERROR			0
#define DNS_RCODE_FORMATERROR		1
#define DNS_RCODE_SERVERERROR		2
#define DNS_RCODE_NAMERROR			3
#define DNS_RCODE_NOTIMPLEMENTED	4	
#define DNS_RCODE_REFUSED			5

/* RFC 1035 -------------------------------------------------------------------
4.1.1. Header section format
 The header contains the following fields:
                                    1  1  1  1  1  1
      0  1  2  3  4  5  6  7  8  9  0  1  2  3  4  5
    +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
    |                      ID                       |
    +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
    |QR|   Opcode  |AA|TC|RD|RA|   Z    |   RCODE   |
    +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
    |                    QDCOUNT                    |
    +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
    |                    ANCOUNT                    |
    +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
    |                    NSCOUNT                    |
    +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
    |                    ARCOUNT                    |
    +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
ID              A 16 bit identifier assigned by the program that
                generates any kind of query.  This identifier is copied
                the corresponding reply and can be used by the requester
                to match up replies to outstanding queries.

QR              A one bit field that specifies whether this message is a
                query (0), or a response (1).

OPCODE          A four bit field that specifies kind of query in this
                message.  This value is set by the originator of a query
                and copied into the response.  The values are:
                0               a standard query (QUERY)
                1               an inverse query (IQUERY)
                2               a server status request (STATUS)
                3-15            reserved for future use

AA              Authoritative Answer - this bit is valid in responses,
                and specifies that the responding name server is an
                authority for the domain name in question section.

                Note that the contents of the answer section may have
                multiple owner names because of aliases.  The AA bit
                corresponds to the name which matches the query name, or
                the first owner name in the answer section.

TC              TrunCation - specifies that this message was truncated
                due to length greater than that permitted on the
                transmission channel.

RD              Recursion Desired - this bit may be set in a query and
                is copied into the response.  If RD is set, it directs
                the name server to pursue the query recursively.
                Recursive query support is optional.

RA              Recursion Available - this be is set or cleared in a
                response, and denotes whether recursive query support is
                available in the name server.

Z               Reserved for future use.  Must be zero in all queries
                and responses.

RCODE           Response code - this 4 bit field is set as part of
                responses.  The values have the following
                interpretation:

                0               No error condition
                1               Format error - The name server was
                                unable to interpret the query.
                2               Server failure - The name server was
                                unable to process this query due to a
                                problem with the name server.
                3               Name Error - Meaningful only for
                                responses from an authoritative name
                                server, this code signifies that the
                                domain name referenced in the query does
                                not exist.
                4               Not Implemented - The name server does
                                not support the requested kind of query.
                5               Refused - The name server refuses to
                                perform the specified operation for
                                policy reasons.  For example, a name
                                server may not wish to provide the
                                information to the particular requester,
                                or a name server may not wish to perform
                                a particular operation (e.g., zone
                                transfer) for particular data.
                6-15            Reserved for future use.
-----------------------------------------------------------------------------*/
#define	MASK_QR				0x8000	// 1000 0000 0000 0000
#define	MASK_OPCODE			0x7800	// 0111 1000 0000 0000
#define	MASK_AA				0x0400	// 0000 0100 0000 0000
#define	MASK_TC				0x0200	// 0000 0010 0000 0000
#define	MASK_RD				0x0100	// 0000 0001 0000 0000
#define	MASK_RA				0x0080	// 0000 0000 1000 0000
#define	MASK_Z				0x0070	// 0000 0000 0111 0000
#define	MASK_RCODE			0x000F	// 0000 0000 0000 1111

typedef struct _DNS_HEADER
{
	unsigned short	id;
	unsigned short	info;
	unsigned short	QDcount;
	unsigned short	ANcount;
	unsigned short	NScount;
	unsigned short	ARcount;
} DNS_HEADER;

/* RFC 1035 -------------------------------------------------------------------
3.2. RR definitions
3.2.1. Format
All RRs have the same top level format shown below:

                                    1  1  1  1  1  1
      0  1  2  3  4  5  6  7  8  9  0  1  2  3  4  5
    +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
    |                                               |
    /                      NAME                     /
    |                                               |
    +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
    |                      TYPE                     |
    +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
    |                     CLASS                     |
    +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
    |                      TTL                      |
    |                                               |
    +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
    |                   RDLENGTH                    |
    +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--|
    /                     RDATA                     /
    /                                               /
    +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
-----------------------------------------------------------------------------*/
typedef struct _DNS_RRecord
{
	// NAME						// NAME -> variable length
	unsigned short	usTYPE;
	unsigned short	usCLASS;
	unsigned long	ulTTL;
	unsigned short	usRDLENGTH;
	char			sRDATA;
	// RDATA					// RDATA -> variable length
} DNS_RRecord;

// ----------------------------------------------------------------------------
// local function
// ----------------------------------------------------------------------------
static char* GetDNSLabel( const char* pRoot, const char* pLabel, char* pDst, int nDstLen );
static int MakeDNSLabel( const char* pSrc, char* pDst, int nDstLen );
static void SortMXRecord(MX* pMX);
static int CheckMXRecord(MX* pMX);

static int GetUDPPacket(const char* pszServer, int nPort, char* sendbuf, int nsendbuflen, char* recvbuf, int nrecvbuflen, unsigned int dwtimeout);

static unsigned short GetUShort( const char* p );
static unsigned long GetULong( const char* p );
static char* SetUShort( char* p, unsigned short us );
static char* SetULong( char* p, unsigned long ul );

static char *MYSTRDUP( const char *src );
static size_t MYSTRLEN( const char *string );
static int MYSTRICMP( const char *str1, const char *str2 );
static int IsDNSString( const char* string );

MXRecord::MXRecord()
{
	int ii;
	for( ii = 0; ii < MXR_MAX_MX_RECORDS+1; ii++ )
	{
		m_MX[ii].usPreference = 0;
		m_MX[ii].pszDName = NULL;
	}
}

MXRecord::~MXRecord()
{
	int ii;
	for( ii = 0; ii < MXR_MAX_MX_RECORDS; ii++ )
	{
		m_MX[ii].usPreference = 0;
		if( m_MX[ii].pszDName )	FREEIF( m_MX[ii].pszDName );
	}
}

//-----------------------------------------------------------------------------
//	@detail
//		Query DNS
//	@param
//		pszDNS		DNS server name that connect to
//		pszQuestion	domain name for query
//	@return
//		SUCCESS		1 or more
//		FAIL		ERROR_CODE
//					MXRECORD_ERR_BADDNSHOSTNAME
//					MXRECORD_ERR_CONNECTFAILED
//					MXRECORD_ERR_SOCKETERROR
//					MXRECORD_ERR_DNSSERVERERROR
//					MXRECORD_ERR_DNSBADREPLY
//	@note
//-----------------------------------------------------------------------------
int MXRecord::QueryDNS( const char* pszDNS, const char* pszQuestion, unsigned int dwTimeOut = 30 )
{
	int ii;
	for( ii = 0; ii < MXR_MAX_MX_RECORDS; ii++ )
	{
		m_MX[ii].usPreference = 0;
		if( m_MX[ii].pszDName )	FREEIF( m_MX[ii].pszDName );
	}

	if( NULL == pszDNS || strlen( pszDNS ) >= 0xFF || NULL == pszQuestion )
		return MXRECORD_ERR_BADDNSHOSTNAME;

	if( 0 == IsDNSString( pszQuestion ) )
		return MXRECORD_ERR_BADDNSHOSTNAME;

	
	
	char Packet[DNS_PACKET_SIZE] = {0};
	char szDName[DNS_NAME_SIZE] = {0};
	char* pQuery = Packet;
	DNS_HEADER* pDH = (DNS_HEADER*)Packet;
	unsigned int nQueryLen;
	unsigned int nDNSLabelLen;

	// ------------------------------------------------------------------------
	// Make Question
	// ------------------------------------------------------------------------
	pDH->id      = (unsigned short)GetTickCount();
	pDH->info    = htons(0x100);//1 0000 0000
	pDH->QDcount = htons(1);
	pDH->ANcount = 0;
	pDH->NScount = 0;
	pDH->ARcount = 0;

	pQuery += sizeof( DNS_HEADER );
	nDNSLabelLen = MakeDNSLabel( pszQuestion, pQuery, DNS_PACKET_SIZE - sizeof( DNS_HEADER ) );
	if( nDNSLabelLen == 0 )
	{
		return MXRECORD_ERR_BADDNSHOSTNAME;
	}
	pQuery += nDNSLabelLen;

	pQuery = SetUShort( pQuery, DNS_TYPE_MX );
	pQuery = SetUShort( pQuery, DNS_CLASS_IN );

	nQueryLen = pQuery - &Packet[0];

	int nRetSocket = GetUDPPacket(pszDNS, DNS_PORTNO, Packet, nQueryLen, Packet, DNS_PACKET_SIZE, 30);
	if( nRetSocket != MXRECORD_SUCCESS )
		return nRetSocket;

	char* pResponse = Packet;
	unsigned short info = GetUShort((const char*)&pDH->info);
	if( !( info & MASK_QR) )
		return MXRECORD_ERR_DNSSERVERERROR;	// This message is query

	switch( info & MASK_RCODE )
	{
	case DNS_RCODE_FORMATERROR:
	case DNS_RCODE_NAMERROR:
		return MXRECORD_ERR_BADDNSHOSTNAME;
	case DNS_RCODE_SERVERERROR:
	case DNS_RCODE_NOTIMPLEMENTED:
	case DNS_RCODE_REFUSED:
		return MXRECORD_ERR_DNSSERVERERROR;
	case DNS_RCODE_NOERROR:
	default:
		break;
	}

	// ------------------------------------------------------------------------
	// Read response
	// ------------------------------------------------------------------------
	pResponse += sizeof( DNS_HEADER );

	// ------------------------------------------
	// Read question section
	// ------------------------------------------
	unsigned short QDcount = GetUShort((const char*)&pDH->QDcount);
	for( ii = 0; ii < QDcount; ii++ )
	{
		// read QNAME
		pResponse = GetDNSLabel( &Packet[0], pResponse, szDName, DNS_NAME_SIZE );
		pResponse += sizeof( DNS_QTYPE );
		pResponse += sizeof( DNS_QCLASS );
	}

	// ------------------------------------------
	// Read answer section
	// ------------------------------------------
	unsigned short ANcount = GetUShort((const char*)&pDH->ANcount);
	for( ii = 0; ii < ANcount && ii < MXR_MAX_MX_RECORDS; ii++ )
	{
		if( NULL == (pResponse = GetDNSLabel( &Packet[0], pResponse, szDName, DNS_NAME_SIZE ))) {
			return MXRECORD_ERR_DNSBADREPLY;
		}

		DNS_RRecord* pRR = (DNS_RRecord*)pResponse;
		unsigned short usCLASS = GetUShort((const char*)&pRR->usCLASS);
		unsigned short usTYPE = GetUShort((const char*)&pRR->usTYPE);

		if( usCLASS != DNS_CLASS_IN || usTYPE != DNS_TYPE_MX ) {
			return MXRECORD_ERR_DNSBADREPLY;
		}

		// Read and make record
		m_MX[ii].usPreference = GetUShort( (const char*)&pRR->sRDATA );
		
		if( NULL == (pResponse = GetDNSLabel( &Packet[0], &pRR->sRDATA + sizeof(DNS_TYPE), szDName, DNS_NAME_SIZE ))) {
			return MXRECORD_ERR_DNSBADREPLY;
		}

		if( NULL == (m_MX[ii].pszDName = MYSTRDUP( szDName ))) {
			return MXRECORD_ERR_NOMEMORY;
		}
		
	}
	// sort by prefernce.
	if( !CheckMXRecord( &m_MX[0] ) )
		return MXRECORD_ERR_DNSBADREPLY;
	SortMXRecord( &m_MX[0] );

	// ------------------------------------------
	// Read authority records
	// ------------------------------------------
	// ... future

	// ------------------------------------------
	// Read additional section
	// ------------------------------------------
	// ... future

	return MXRECORD_SUCCESS;
}

//-----------------------------------------------------------------------------
//	@detail
//		Get first domain name
//	@param
//	@return
//		first domain name
//	@note
//-----------------------------------------------------------------------------
const char* MXRecord::GetFirstMXRecord()
{
	return m_MX[0].pszDName;
}

//-----------------------------------------------------------------------------
//	@detail
//		Get domain name nexted to pszCurrent
//	@param
//		pszCurrent	root key
//	@return
//		next from pszCurrent domain name pointer
//	@note
//-----------------------------------------------------------------------------
const char* MXRecord::GetNetxtMXRecord(const char* pszCurrent)
{
	MX* pmx = &m_MX[0];
	
	while( NULL != pmx->pszDName )
	{
		if( 0 == MYSTRICMP( pszCurrent, pmx->pszDName ) )
		{
			pmx++;
			return pmx->pszDName;
		}
		pmx++;
	}
	return NULL;
}

//-----------------------------------------------------------------------------
//	@detail
//		Check MX records
//	@param
//		pMX			Head position of MX record array.
//	@return
//		GOOD		not 0
//		BAD			0
//	@note
//-----------------------------------------------------------------------------
static int CheckMXRecord(MX* pMX)
{
	MX* pmxA;
	MX* pmxB;

	pmxA = pMX;

	while( NULL != pmxA->pszDName )
	{
		pmxB = pmxA+1;

		while( NULL != pmxB->pszDName )
		{
			if( 0 == MYSTRICMP( pmxA->pszDName, pmxB->pszDName ) ) return 0;
			pmxB++;
		}
		pmxA++;
	}
	return 1;
}

//-----------------------------------------------------------------------------
//	@detail
//		Sort MX records by MX prefernce
//	@param
//		pMX			Head position of MX record array.
//	@return
//	@note
//-----------------------------------------------------------------------------
static void SortMXRecord(MX* pMX)
{
	MX* pmxA;
	MX* pmxB;
	MX mxSwap;

	pmxA = pMX;

	while( pmxA->usPreference != 0 )
	{
		pmxB = pmxA+1;

		while( pmxB->usPreference != 0 )
		{
			if( pmxA->usPreference > pmxB->usPreference )
			{
				// swap << A
				mxSwap.usPreference = pmxA->usPreference;
				mxSwap.pszDName = pmxA->pszDName;
				// A << B				
				pmxA->usPreference = pmxB->usPreference;
				pmxA->pszDName = pmxB->pszDName;
				// B << swap
				pmxB->usPreference = mxSwap.usPreference;
				pmxB->pszDName = mxSwap.pszDName;
			}
			pmxB++;
		}
		pmxA++;
	}
}

//-----------------------------------------------------------------------------
//	@detail
//		Get DNS label from packet
//	@param
//		pRoot		Head position of Packet
//		pLabel		Head position of DNS label
//		pDst		The buffer of destination that will be written DNS label
//		nDstLen		Length of the buffer
//	@return
//		NULL		Fail, don't proceed process.
//		not NULL	The next pointer of DNS label.
//	@note
//		Packet buffer (pointed by pRoot) must have DNS_PACKET_SIZE.
//-----------------------------------------------------------------------------
static char* GetDNSLabel( const char* pRoot, const char* pLabel, char* pDst, int nDstLen )
{
	const char* p = pLabel;
	const char* pRet = pLabel;
	int bCompression = 0;

	if( NULL == pRoot || NULL == pLabel || NULL == pDst || nDstLen < 1 )
		return NULL;

	while( *p != '\0' && nDstLen > 0 )
	{
		if( (*p & 0xC0) != 0 ) // RFC 1035 4.1.4 Message compression
		{

			unsigned short usJump = GetUShort(p);
			//memcpy( (void*)&usJump, p, sizeof(short) );
			usJump = (usJump & 0x3FFF);
			
			//unsigned short* pus = (unsigned short*)p;
			//unsigned short usJump = (ntohs(*pus) & 0x3FFF);
			if( usJump > DNS_PACKET_SIZE -1 )
				return NULL;
			p = pRoot + usJump;	// Jump to the point that indicated.
												// 0x3FFF = 0011 1111 1111 1111
			bCompression = 1;					// if jumped, return pointer is not the place that jumped to.
			pRet += sizeof( unsigned short );
		}

		int nLoop = (int)*p++;
		// *pDst = '.';
		for( int ii = 0; ii < nLoop; ii++ ) 
		{
			*pDst++ = *p++;
			nDstLen--;
		}
		if( !bCompression ) pRet = p;

		if( *p != '\0' )
		{
			*pDst++ = '.';
		}
		else
		{
			*pDst++ = '\0';
			if( !bCompression ) pRet++;
			break;
		}
	}

	return (char*)pRet;
}

static int MakeDNSLabel( const char* pSrc, char* pDst, int nDstLen )
{
	const char* p = pSrc;
	int nCount = 0;

	if( NULL == pSrc || NULL == pDst )
		return 0;

	if( MYSTRLEN( pSrc ) > 0xFF )	// Length of domain name is 255 or less(8bit)
		return 0;

	//for( ii = 0; ii < nDstLen; ii++ )	pDst[ii] = '\0';

	while( NULL != *p )
	{
		unsigned char cc;
		const char* pOct;
		for( cc = 0, pOct = p; *pOct != '.' && *pOct != '\0'; pOct++, cc++ )
		{
			if( cc & 0xC0 ) return 0;	// Length of part of domain name is 63 or less(6bit)
		}

		*pDst = cc;

		strncpy( pDst+1, p, cc );
		
		nCount += (cc+1);
		pDst += (cc+1);

		if( *pOct == '\0' )
			break;

		p = ++pOct;

	}
	*pDst = '\0';
	nCount++;
	return nCount;
}

static unsigned short GetUShort( const char* p )
{
	unsigned short ret;
	memcpy( (void*)&ret, p, sizeof( short ) );
	//ret = (u_short)(((char)*(p+1))*0x100+((char)*(p+0)));
	return ntohs( ret );
}

static unsigned long GetULong( const char* p )
{
	unsigned long ret;
	ret = ((char)*(p+3))*0x10000+((char)*(p+2))*0x1000+((char)*(p+1))*0x100+((char)*(p+0));
	return ntohl( ret );
}

static char* SetUShort( char* p, unsigned short us )
{
	unsigned short set;
	set = htons( us );
	
	char* c = (char*)&set;
	*(p+0) = *(c+0);
	*(p+1) = *(c+1);
	return p+2;
}

static char* SetULong( char* p, unsigned long ul )
{
	unsigned long set;
	set = htonl( ul );

	char* c = (char*)&set;
	*(p+0) = *(c+0);
	*(p+1) = *(c+1);
	*(p+2) = *(c+2);
	*(p+3) = *(c+3);
	return p+4;
}

static size_t MYSTRLEN( const char *string )
{
	size_t ii = 0;

	while( *string++ != '\0' )
		ii++;
	
	return ii;
}

static char *MYSTRDUP( const char *src )
{
	size_t len = 0;
	char *mem = NULL;
	const char *s = src;
	char *d = NULL;

	if( NULL == src )
		return NULL;

	while( *s++ )
		len++;

	mem = (char*)MALLOC( len + 1 );

	if( NULL == mem )
		return NULL;

	s = src;
	d = mem;
	do
	{
		*d++ = *s++;
	}while( *s != '\0' );

	*d = '\0';

	return mem;
}

static int MYSTRICMP( const char *str1, const char *str2 )
{
	int a, b;

	do
	{
		a = (unsigned char)*str1;
		b = (unsigned char)*str2;

		if( (a >= 'A') && (a <= 'Z') )
			a += 'a' - 'A';

		if( (b >= 'A') && (b <= 'Z') )
			b += 'a' - 'A';
		str1++;
		str2++;
	}while( a && (a == b) );
	
	return a - b;
}


/* RFC 1035 -------------------------------------------------------------------
2.3.1. Preferred name syntax

The DNS specifications attempt to be as general as possible in the rules
for constructing domain names.  The idea is that the name of any
existing object can be expressed as a domain name with minimal changes.

However, when assigning a domain name for an object, the prudent user
will select a name which satisfies both the rules of the domain system
and any existing rules for the object, whether these rules are published
or implied by existing programs.

For example, when naming a mail domain, the user should satisfy both the
rules of this memo and those in RFC-822.  When creating a new host name,
the old rules for HOSTS.TXT should be followed.  This avoids problems
when old software is converted to use domain names.

The following syntax will result in fewer problems with many
applications that use domain names (e.g., mail, TELNET).

<domain> ::= <subdomain> | " "

<subdomain> ::= <label> | <subdomain> "." <label>

<label> ::= <letter> [ [ <ldh-str> ] <let-dig> ]

<ldh-str> ::= <let-dig-hyp> | <let-dig-hyp> <ldh-str>

<let-dig-hyp> ::= <let-dig> | "-"

<let-dig> ::= <letter> | <digit>

<letter> ::= any one of the 52 alphabetic characters A through Z in
             upper case and a through z in lower case

<digit> ::= any one of the ten digits 0 through 9

Note that while upper and lower case letters are allowed in domain
names, no significance is attached to the case.  That is, two names with
the same spelling but different case are to be treated as if identical.

The labels must follow the rules for ARPANET host names.  They must
start with a letter, end with a letter or digit, and have as interior
characters only letters, digits, and hyphen.  There are also some
restrictions on the length.  Labels must be 63 characters or less.

For example, the following strings identify hosts in the Internet:

A.ISI.EDU XX.LCS.MIT.EDU SRI-NIC.ARPA

*/
static int IsDNSString( const char* string )
{
/*
They must start with a letter, 
end with a letter or digit, 
and have as interior characters only letters, digits, and hyphen. 
*/

	if( NULL == string )
		return 0;	//error

	const char* p = string;
	char lastchar;

	do
	{
		// last word must not be '.'
		if( *p == '\0' )
			return 0;

		// 1st char is must be alphabet
		if( ((*p < 'A')) || (*p > 'Z') && ((*p < 'a') || (*p > 'z')) )
			return 0;
		
		p++;
		
		// following char must be aplhabet or '-'
		while( '\0' != *p && '.' != *p )
		{
			if( ((*p < '0') || (*p > '9')) && 
				((*p < 'A') || (*p > 'Z')) && 
				((*p < 'a') || (*p > 'z')) && 
				('-' != *p) )
				return 0;

			lastchar = *p;
			p++;
		}

		if( lastchar == '-' )
			return 0;

		if( *p == '.' )
		{
			p++;
			if( '\0' == *p )
				return 0;
		}

	}while( '\0' != *p );


	return 1;
}


int MXRecord::InitDll(int nInit)
{
	if( nInit )
	{
		WSADATA	wsaData;
		WORD winsockversion = MAKEWORD(2,0);
		int winsockstartup;
		
		winsockstartup = WSAStartup(winsockversion,&wsaData);

		if (winsockstartup != 0 || (wsaData.iMaxSockets < 1) )
		{
			WSACleanup();
			return 0;
		}
	}
	else
	{
		WSACleanup();
	}
	return 1;
}

//-----------------------------------------------------------------------------
//	@detail
//		Check MX records
//	@param
//		const char*		pszServer
//		int				nPort
//		const char*		buf
//		int				nbuflen
//	@return
//		MXRECORD_ERR_SOCKETERROR
//		MXRECORD_ERR_DNSTIMEOUT
//		MXRECORD_SUCCESS
//	@note
//-----------------------------------------------------------------------------
static int GetUDPPacket(const char* pszServer, int nPort, char* sendbuf, int nsendbuflen, char* recvbuf, int nrecvbuflen, unsigned int dwtimeout)
{
	struct sockaddr_in server = {0};
	int saddr_len = sizeof(sockaddr);

	server.sin_family = AF_INET;
	server.sin_addr.s_addr = inet_addr(pszServer);
	server.sin_port = htons(nPort);

	SOCKET sock = socket(AF_INET, SOCK_DGRAM, 0);

	// set option 'non blocking io'
	timeval	timeout = {0};
	int ioctl_opt =1;
	int i = 0;

	if (sock == INVALID_SOCKET)
	{
		return MXRECORD_ERR_SOCKETERROR;
	}

	if (::ioctlsocket(sock,FIONBIO,(unsigned long*)&ioctl_opt) == SOCKET_ERROR)
	{
		closesocket( sock );
		return MXRECORD_ERR_SOCKETERROR;
	}

	if( 1 > sendto( sock, sendbuf, nsendbuflen, 0, (sockaddr*)&server, saddr_len ) )
	{
		closesocket( sock );
		return MXRECORD_ERR_SOCKETERROR;
	}

	memset( (void*)recvbuf, 0, nrecvbuflen );
	fd_set	readfds;
	FD_ZERO(&readfds);
	FD_SET(sock,&readfds);
	timeout.tv_sec = dwtimeout;
	timeout.tv_usec = 0;
	i = select(0, &readfds, NULL, NULL, &timeout);

	if( i == SOCKET_ERROR )
	{
		closesocket( sock );
		return MXRECORD_ERR_SOCKETERROR;
	}
	else if( i == 0 )
	{
		closesocket( sock );
		return MXRECORD_ERR_DNSTIMEOUT;
	}

	if( 1 > recvfrom( sock, recvbuf, nrecvbuflen, 0, (sockaddr*)&server, &saddr_len ) )
	{
		closesocket( sock );
		return MXRECORD_ERR_SOCKETERROR;
	}

	closesocket( sock );
	return MXRECORD_SUCCESS;

}
