/* =========================================================================
	This library is part of IshiSoft application.
	------------------------------------------------------------------------
	author: Yuichiro Ishitani
			http://www001.upp.so-net.ne.jp/ishi/index.html
  ========================================================================== */
#include "stdafx.h"
#include "MyConsole.h"
#include "str.h"
#include "wincemem.h"

#include <stdio.h>

const char* gstr_CRLF = "\r\n";

MyConsole::MyConsole( int nMaxLineSize )
{
	m_BufferSize = nMaxLineSize;
	m_pBuffer = NULL;
}

MyConsole::~MyConsole()
{
	FREEIF( m_pBuffer );
}

// please override this function.
int MyConsole::Dialogue()
{
	char szCmd[64];

	while( 0 <= GetLine( szCmd, m_BufferSize ) )
	{	
		// cmd get.
		break;
	}
	Close();

	return 1;
}

int MyConsole::GetLine(char *pLine, int nSize, int* pbStop /* = NULL*/, int nIdleTimeout/* = 30*/)
{
	int nCmdLen;
	
	if( NULL == pLine )
		return -1;

	memset( pLine, 0, nSize );

	// -- if there is no buffer memory...
	if( NULL == m_pBuffer )
	{
		m_pBuffer = (char*)MALLOC( m_BufferSize );
		
		if( NULL == m_pBuffer )
			return -1;
	}

	int nbStop = 0;
	if( NULL == pbStop )
		pbStop = &nbStop;

	if( *pbStop )
		return -1;

	// while CRLF is not in the buffer, recv the data from client.
	char* pNextLine = STRSTR( m_pBuffer, gstr_CRLF );
	timeval tm;
	
	// get default timeout
	GetTimeOut( &tm );
	SetTimeOut( 1 );

	if( NULL == pNextLine )
	{
		int nCashSize;
		int nRecvLen;
		char* pEnd;
		int nCopySize;
		int tmpTimeout = nIdleTimeout;

		while( NULL == (pNextLine = STRSTR( m_pBuffer, gstr_CRLF )) )
		{
			
			// Cash
			pEnd = STRCHR( m_pBuffer, '\0' );

			nCashSize = pEnd - m_pBuffer;

			// Buffer overflow
			if( nCashSize >= m_BufferSize - 1 )
			{
				memset( m_pBuffer, 0, m_BufferSize );
				goto GETLINE_ERROR_END;
			}

			// Recv
			nRecvLen = RecvData( pEnd, m_BufferSize - 1 - nCashSize );
			if( 0 > nRecvLen )
			{
				goto GETLINE_ERROR_END;
			}
			else if( 0 == nRecvLen )
				tmpTimeout--;
			else
				tmpTimeout = nIdleTimeout;

			if( tmpTimeout < 0 || *pbStop )
				goto GETLINE_ERROR_END;

			// strip strings ---> START
			nCopySize = nRecvLen;

			for( char* p = pEnd; *p != '\0'; )
			{
				if( *p == 0 || *p > 0x7E )			// US-ASCII
				{
					STRNCPY( p, p+1, nCopySize );
					nCopySize--;
					nRecvLen--;
				}
				else
				{
					nCopySize--;
					p++;
				}
			}
			// strip strings ---> END

			// --- Echo back ---
			// SendData( pEnd );

		}

		// restore timeout
		SetTimeOut( tm.tv_sec, tm.tv_usec );
	}

	// Pickup line
	*pNextLine++ = '\0';	// \r
	*pNextLine++ = '\0';	// \n
		
	nCmdLen = STRLEN( m_pBuffer );

	// copy to buffer
	STRNCPY( pLine, m_pBuffer, nSize );

	// Shift data
	memcpy( m_pBuffer, pNextLine, m_BufferSize - (nCmdLen + 2) );
	memset( m_pBuffer + m_BufferSize - (nCmdLen + 2), 0, (nCmdLen + 2) );

	return nCmdLen;

GETLINE_ERROR_END:
	SetTimeOut( tm.tv_sec, tm.tv_usec );
	return -1;
}

void MyConsole::Close()
{
	FREEIF( m_pBuffer );
	MySock::Close();
}

int MyConsole::GetArg(const char *pszCmdLine, char *pszArg, unsigned int nArgLen, unsigned int nArgNumber)
{
	nArgNumber++;

	unsigned int ii;
	for( ii = 0; ii < nArgNumber; ii++ )
	{
		while( ' ' != *pszCmdLine && '\0' != *pszCmdLine ) pszCmdLine++;

		if( '\0' == *pszCmdLine )
			return 0;

		while( ' ' == *pszCmdLine ) pszCmdLine++;
	}

	int flag;
	for( ii = 0, flag = 0; ii < nArgLen; ii++ )
	{
		if( *pszCmdLine == '\0' || *pszCmdLine == ' ' || *pszCmdLine == ' ' )
		{
			*pszArg = '\0';
			return 1;
		}
			
		*pszArg++ = *pszCmdLine++;
	}

	if( '\0' != *pszCmdLine )
		return 0;

	return 1;

}
