/* =========================================================================
	This library is part of IshiSoft application.
	------------------------------------------------------------------------
	author: Yuichiro Ishitani
			http://www001.upp.so-net.ne.jp/ishi/index.html
  ========================================================================== */
#include "stdafx.h"
#include "MyHashFile.h"

//#include "wincemem.h"
#include "str.h"

#define TMPBUFFER_SIZE 512

MyHashFile::MyHashFile()
{
}

MyHashFile::~MyHashFile()
{

}

//-----------------------------------------------------------------------------
//	@detail
//		Load file
//	@param
//		WCHAR*	ptszFile;	file name	(UTF-2)
//	@return
//		0 indicate an error
//-----------------------------------------------------------------------------
int MyHashFile::LoadFile(const WCHAR *pwszSrcFile)
{
	if( NULL == pwszSrcFile )
	{
		return 0;
	}

	WCHAR wszFileName[MAX_PATH+1] = {0};
	WCHAR* pFN = NULL;
	const WCHAR* pN = NULL;

	// test does it have \ mark.
	char f = 0;
	pN = pwszSrcFile;
	while( 0 != *pN )
	{
		if( '\\' == *pN )
		{
			f= 1;
			break;
		}
		pN++;
	}

	if( 0 == f )
	{
		// ----------------------------------------------------
		// make full path
		// ----------------------------------------------------
		GetModuleFileName( NULL, wszFileName, MAX_PATH );
		pFN = &wszFileName[0];
		pN = pwszSrcFile;

		while( *pFN != '\0' ) {pFN++;}
		while( *pFN != '\\' && pFN != &wszFileName[0] ) { *pFN-- = '\0';}
		pFN++;
		while( *pN != '\0' ) *pFN++ = *pN++;
	}
	else
	{
		pFN = &wszFileName[0];
		pN = pwszSrcFile;
		while( *pN != '\0' ) *pFN++ = *pN++;
		*pFN = *pN;
	}

	// --------------------------------------------------------------
	// Read all the contents of a file.
	// --------------------------------------------------------------
	HANDLE hFile = CreateFile(wszFileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
	MyVarMem	vm;

	if( hFile == INVALID_HANDLE_VALUE )
	{
		return 0;
	}
	else
	{
		char	szBuf[TMPBUFFER_SIZE] = {0};
		BOOL	bResult;
		DWORD	dwReadBytes;
		do
		{
			bResult = ReadFile( hFile, szBuf, TMPBUFFER_SIZE, &dwReadBytes, NULL );
			
			if(dwReadBytes == 0)
				break;

			// ----------------------------------------------------
			// Error no memory.
			// ----------------------------------------------------
			if( 0 == vm.Add( szBuf, dwReadBytes ) )
				return 0;

		}while( bResult );

		CloseHandle(hFile);
	}

	if( vm.GetLength() == 0 )
	{
		return 0;
	}
	// ..File read end

	// --------------------------------------------------------------
	// Parse file by cr/lf.
	// --------------------------------------------------------------
	char*	pCurrentLine = NULL;
	char*	pNextLine = NULL;

	pCurrentLine = pNextLine = (char*)vm.GetBuffer();

	do
	{
		while( *pNextLine == '\r' ||  *pNextLine == '\n' )
			pNextLine++;

		if( *pNextLine == '\0' )
			break;

		pCurrentLine = pNextLine;

		// seek to next line
		while( *pNextLine != '\r' &&  *pNextLine != '\n' && *pNextLine != '\0' )
		{
			pNextLine++;
		}

		if( *pNextLine != '\0' )
			*pNextLine++ = '\0';
		
		// ignore the comment symbole
		if( *pCurrentLine == '#' || *pCurrentLine == ' ' || *pCurrentLine == '\t')
		{
			continue;
		}
		else
		{
			char* pKey = NULL;
			char* pValue = NULL;

			pKey = pCurrentLine;
			pValue = pCurrentLine;

			while( *pValue != ' ' && *pValue != '\t' && *pValue != '\0' )
			{
				pValue++;
			}

			// there is no value string;
			if( *pValue == '\0' )
				continue;

			*pValue++ = '\0';

			while( *pValue == ' ' || *pValue == '\t' )
			{
				*pValue++ = '\0';
			}

			Set(pKey, pValue);

		}

		// next line
	}while( *pNextLine != '\0' );

	return 1;
}

int MyHashFile::LoadFile(const char *pszFile)
{
	WCHAR wszTmpFile[MAX_PATH+1];
	if( 0 == MultiByteToWideChar( CP_ACP, 0, pszFile, -1, wszTmpFile, MAX_PATH ) )
		return 0;

	return LoadFile( wszTmpFile );
}

//-----------------------------------------------------------------------------
//	@detail
//		Save file
//	@param
//		WCHAR*	ptszFile;	file name	(UTF-2)
//	@return
//		0 indicate an error
//-----------------------------------------------------------------------------
int MyHashFile::SaveFile(const WCHAR *pwszFile)
{
	WCHAR wszTmpFile[MAX_PATH+1] = {0};

	wsprintf( wszTmpFile, L"%s_tmp_", pwszFile );

	HANDLE hFile = CreateFile(wszTmpFile, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);

	if( hFile == INVALID_HANDLE_VALUE )
	{
		return 0;
	}

	SEEKKEY p = SeekToFirst();
	char* pBuf = NULL;
	DWORD dwWritten = 0;
	DWORD dwKeyLen = 0;
	DWORD dwValLen = 0;
	const char* pValue = NULL;

	while( p )
	{

		pValue = GetValueBySeekKey( p );
		if( NULL == pValue )
		{
			continue;
		}

		dwKeyLen = (DWORD)STRLEN( p );
		dwValLen = (DWORD)STRLEN( pValue );

		DWORD dwTotal = dwKeyLen + 1 + dwValLen + 1;
		pBuf = (char*)MALLOC( dwTotal + 1 );
		if( NULL == pBuf )
		{
			CloseHandle( hFile );
			DeleteFile( wszTmpFile );
			return 0;
		}

		STRCPY( pBuf, p );
		if( dwValLen > 0 )
		{
			STRCAT( pBuf, " " );
			STRCAT( pBuf, pValue );
		}
		STRCAT( pBuf, "\r\n" );
		if( 0 == WriteFile( hFile, pBuf, dwTotal, &dwWritten, NULL ) )
		{
			CloseHandle( hFile );
			DeleteFile( wszTmpFile );
			return 0;
		}

		FREEIF( pBuf );

		p = SeekNext( p );
	}

	CloseHandle( hFile );

	BOOL b = MoveFile( wszTmpFile, pwszFile );
	//DeleteFile( wszTmpFile );

	return (b?1:0);
}

unsigned int MyHashFile::GetUINTValueI(const char *pKey)
{
	const char* p = MyHashEx::GetValueI( (char*)pKey );
	if( p )
		return ATOUI( p );
	else
		return 0;

}
