/* =========================================================================
	This library is part of IshiSoft application.
	------------------------------------------------------------------------
	author: Yuichiro Ishitani
			http://www001.upp.so-net.ne.jp/ishi/index.html
  ========================================================================== */
#include <windows.h>
#include <stdio.h>
#include "MyTime.h"

char* MyTime::m_pszMonth[12] =
{
	"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
};

char* MyTime::m_pszWkday[7] =
{
	"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
};

char* MyTime::m_pszWeekday[7] = 
{
	"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"
};

static int DayOfWeek(int y, int m, int d);

// ----------------------------------------------------------------------------
MyTime::MyTime()
{
	/* ---
	This function retrieves the current system date and time.
	The system time is expressed in Coordinated Universal Time (UTC).
	--- */
	SYSTEMTIME		st;
	GetSystemTime(&st);
	SystemTimeToFileTime(&st, &m_ft);
}

// ----------------------------------------------------------------------------
void MyTime::Set(const SYSTEMTIME& pSysTime)
{
	SystemTimeToFileTime(&pSysTime, &m_ft);
}

// ----------------------------------------------------------------------------
void MyTime::Set(const FILETIME& FileTime)
{
	// Filetime is as UTC
	SYSTEMTIME	st;

	m_ft = FileTime;

	FileTimeToSystemTime(&FileTime, &st);
	st.wMilliseconds = 0;
	SystemTimeToFileTime(&st, &m_ft);
}

// ----------------------------------------------------------------------------
MyTime::~MyTime()
{
}

// ----------------------------------------------------------------------------
void MyTime::SetUTCTime()
{
	SYSTEMTIME		st;
	GetSystemTime(&st);
	SystemTimeToFileTime(&st, &m_ft);

}

// ----------------------------------------------------------------------------
void MyTime::SetLocalTime()
{
	SYSTEMTIME		st;
	GetLocalTime(&st);
	SystemTimeToFileTime(&st, &m_ft);
}

// ----------------------------------------------------------------------------
void MyTime::SetRfc1123Time(const char* pszMyTime)
{
//  0         1         2         3
//  0123456789012345678901234567890
//	Sun, 06 Nov 1994 08:49:37 GMT  ; RFC 822, updated by RFC 1123
	char szTime[32];
	int ii;
	SYSTEMTIME	st;

	strncpy(szTime, pszMyTime, 32);

	szTime[07] = '\0';
	szTime[11] = '\0';
	szTime[16] = '\0';
	szTime[19] = '\0';
	szTime[22] = '\0';
	szTime[25] = '\0';

	// month to int
	for( ii = 0 ; ii < 12 ; ii ++ )
	{
		if( strcmp(m_pszMonth[ii], &szTime[8]) == 0 )
		{
			ii++;
			break;
		}
	}
	if( ii > 12 )
		ii = 11;	// Bad Request

	st.wDay    = atoi(&szTime[5]);
	st.wMonth  = ii;
	st.wYear   = atoi(&szTime[12]);
	st.wHour   = atoi(&szTime[17]);
	st.wMinute = atoi(&szTime[20]);
	st.wSecond = atoi(&szTime[23]);
	st.wMilliseconds = 0;
	st.wDayOfWeek = DayOfWeek( st.wYear, st.wMonth, st.wDay );

	SystemTimeToFileTime(&st, &m_ft);
}

// ----------------------------------------------------------------------------
void MyTime::SetRfc1036Time(const char* pszMyTime)
{
//  0         1         2         3
//  0123456789012345678901234567890
//	Sunday, 06-Nov-94 08:49:37 GMT ; RFC 850, obsoleted by RFC 1036
//	Recive only GMT time

	char szTime[32];
	int ii;
	SYSTEMTIME		st;

	strncpy(szTime, pszMyTime, 32);

	szTime[10] = '\0';
	szTime[14] = '\0';
	szTime[17] = '\0';
	szTime[20] = '\0';
	szTime[23] = '\0';
	szTime[26] = '\0';


	// month to int
	for( ii = 0 ; ii < 12 ; ii ++ )
	{
		if( strcmp(m_pszMonth[ii], &szTime[11]) == 0 )
		{
			ii++;
			break;
		}
	}
	if( ii > 11 )
		ii = 11;	// Bad Request

	st.wDay    = atoi(&szTime[8]);
	st.wMonth  = ii;
	st.wYear   = atoi(&szTime[15]);
	st.wYear   += ( st.wYear > 80 ) ? 1900 : 2000;
	st.wHour   = atoi(&szTime[18]);
	st.wMinute = atoi(&szTime[21]);
	st.wSecond = atoi(&szTime[24]);
	st.wMilliseconds = 0;
	st.wDayOfWeek = DayOfWeek( st.wYear, st.wMonth, st.wDay );

	SystemTimeToFileTime(&st, &m_ft);
}

// ----------------------------------------------------------------------------
void MyTime::SetAsctime(const char* pszMyTime)
{
//  0         1         2         3
//  0123456789012345678901234567890
//	Sun Nov  6 08:49:37 1994       ; ANSI C's asctime() format
//	asctime-date = wkday SP date3 SP time SP 4DIGIT
//	date3        = month SP ( 2DIGIT | ( SP 1DIGIT ))
//                ; month day (e.g., Jun  2)

//	Recive only GMT time

	char szTime[32];
	int ii;
	SYSTEMTIME		st;

	strncpy(szTime, pszMyTime, 32);

	szTime[07] = '\0';
	szTime[10] = '\0';
	szTime[13] = '\0';
	szTime[16] = '\0';
	szTime[19] = '\0';
	szTime[24] = '\0';

	// month to int
	for( ii = 0 ; ii < 12 ; ii ++ )
	{
		if( strcmp(m_pszMonth[ii], &szTime[11]) == 0 )
		{
			ii++;
			break;
		}
	}

	if( ii > 11 )
		ii = 11;	// Bad Request

	st.wDay    = atoi(&szTime[8]);
	st.wMonth  = ii;
	st.wYear   = atoi(&szTime[20]);
	st.wHour   = atoi(&szTime[11]);
	st.wMinute = atoi(&szTime[14]);
	st.wSecond = atoi(&szTime[17]);
	st.wMilliseconds = 0;
	st.wDayOfWeek = DayOfWeek( st.wYear, st.wMonth, st.wDay );

	SystemTimeToFileTime(&st, &m_ft);
}

// ----------------------------------------------------------------------------
void MyTime::Set(const char* pszMyTime)
{
//  0         1         2         3
//  0123456789012345678901234567890
//	Sun, 06 Nov 1994 08:49:37 GMT  ; RFC 822, updated by RFC 1123
//	Sunday, 06-Nov-94 08:49:37 GMT ; RFC 850, obsoleted by RFC 1036
//	Sun Nov  6 08:49:37 1994       ; ANSI C's asctime() format

	if( pszMyTime[3] == ',' )
		SetRfc1123Time( pszMyTime );
	else if( pszMyTime[3] == ' ' )
		SetAsctime( pszMyTime );
	else
		SetRfc1036Time( pszMyTime );

}

// ----------------------------------------------------------------------------
const char*	MyTime::GetFormatRfc1123() const
{
//	Sun, 06 Nov 1994 08:49:37 GMT  ; RFC 822, updated by RFC 1123
	SYSTEMTIME		st;

	FileTimeToSystemTime(&m_ft, &st);

	sprintf((char*)m_szBuffer, "%s, %02u %s %04u %02u:%02u:%02u GMT",
		m_pszWkday[st.wDayOfWeek],
		st.wDay,
		m_pszMonth[st.wMonth - 1],
		st.wYear,
		st.wHour, st.wMinute, st.wSecond);

	return m_szBuffer;
}

// ----------------------------------------------------------------------------
const char*	MyTime::GetLocalFormatRfc1123() const
{
/* RFC2822 3.3. Date and Time Specification (extracts)
   The zone specifies the offset from Coordinated Universal Time (UTC,
   formerly referred to as "Greenwich Mean Time") that the date and
   time-of-day represent.  The "+" or "-" indicates whether the
   time-of-day is ahead of (i.e., east of) or behind (i.e., west of)
   Universal Time.  The first two digits indicate the number of hours
   difference from Universal Time, and the last two digits indicate the
   number of minutes difference from Universal Time.  (Hence, +hhmm
   means +(hh * 60 + mm) minutes, and -hhmm means -(hh * 60 + mm)
   minutes).  The form "+0000" SHOULD be used to indicate a time zone at
   Universal Time.  Though "-0000" also indicates Universal Time, it is
   used to indicate that the time was generated on a system that may be
   in a local time zone other than Universal Time and therefore
   indicates that the date-time contains no information about the local
   time zone.

   A date-time specification MUST be semantically valid.  That is, the
   day-of-the-week (if included) MUST be the day implied by the date,
   the numeric day-of-month MUST be between 1 and the number of days
   allowed for the specified month (in the specified year), the
   time-of-day MUST be in the range 00:00:00 through 23:59:60 (the
   number of seconds allowing for a leap second; see [STD12]), and the
   zone MUST be within the range -9959 through +9959.

*/

//	Sun, 06 Nov 1994 08:49:37 +9:00; RFC 822, updated by RFC 1123
	SYSTEMTIME		st;
	GetLocalTime( &st );

	TIME_ZONE_INFORMATION tzi = {0};
	GetTimeZoneInformation( &tzi );

	sprintf((char*)m_szBuffer, "%s, %02u %s %04u %02u:%02u:%02u %c%02d%02u",
		m_pszWkday[st.wDayOfWeek],
		st.wDay,
		m_pszMonth[st.wMonth - 1],
		st.wYear,
		st.wHour,
		st.wMinute,
		st.wSecond,

		(tzi.Bias < 0) ? '+': '-',
		(tzi.Bias < 0) ? (tzi.Bias / 60) * -1: (tzi.Bias / 60),
		(tzi.Bias < 0) ? (tzi.Bias % 60) * -1: (tzi.Bias % 60)
		);

	return m_szBuffer;
}

// ----------------------------------------------------------------------------
static int DayOfWeek(int y, int m, int d)
{
	if (m < 3)
	{
		y--;
		m += 12;
	}
	return (y + y/4 - y/100 + y/400 + (26*m+16)/10 + d) % 7;
}

