/* =========================================================================
	This library is part of IshiSoft base class.
	------------------------------------------------------------------------
	author: Yuichiro Ishitani
			http://www001.upp.so-net.ne.jp/ishi/index.html
  ========================================================================== */
#if !defined(AFX_MyTime_H__06701547_4004_4FD0_B808_2F79A0A60673__INCLUDED_)
#define AFX_MyTime_H__06701547_4004_4FD0_B808_2F79A0A60673__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/*
HTTP-date    = rfc1123-date | rfc850-date | asctime-date
 rfc1123-date = wkday "," SP date1 SP time SP "GMT"
 rfc850-date  = weekday "," SP date2 SP time SP "GMT"
 asctime-date = wkday SP date3 SP time SP 4DIGIT
 date1        = 2DIGIT SP month SP 4DIGIT
                ; day month year (e.g., 02 Jun 1982)
 date2        = 2DIGIT "-" month "-" 2DIGIT
                ; day-month-year (e.g., 02-Jun-82)
 date3        = month SP ( 2DIGIT | ( SP 1DIGIT ))
                ; month day (e.g., Jun  2)
 time         = 2DIGIT ":" 2DIGIT ":" 2DIGIT
                ; 00:00:00 - 23:59:59
 wkday        = "Mon" | "Tue" | "Wed"
              | "Thu" | "Fri" | "Sat" | "Sun"
 weekday      = "Monday" | "Tuesday" | "Wednesday"
              | "Thursday" | "Friday" | "Saturday" | "Sunday"
 month        = "Jan" | "Feb" | "Mar" | "Apr"
              | "May" | "Jun" | "Jul" | "Aug"
              | "Sep" | "Oct" | "Nov" | "Dec"

	0123456789+123456789+123456789
	Sun, 06 Nov 1994 08:49:37 GMT  ; RFC 822, updated by RFC 1123
	Sunday, 06-Nov-94 08:49:37 GMT ; RFC 850, obsoleted by RFC 1036
	Sun Nov  6 08:49:37 1994       ; ANSI C's asctime() format
	
	Coordinated Universal Time 
		GetSystemTime returns
		UTC = local time + bias
*/

// The wDayOfWeek member of the SYSTEMTIME structure is ignored.
class MyTime  
{
public:
	MyTime();
	MyTime(const char* pszMyTime)		{Set( pszMyTime );}
	MyTime(const SYSTEMTIME& pSysTime)	{Set( pSysTime  );}
	MyTime(const FILETIME& pFileTime)	{Set( pFileTime );}
	virtual ~MyTime();

	FILETIME	GetAsFileTime() const	{return m_ft;}
//	SYSTEMTIME	GetAsSystemTime() const;

	const char*		GetFormatRfc1123() const;
	const char*		GetLocalFormatRfc1123() const;

	int operator == ( MyTime time ) const {return !CompareFileTime(&m_ft, &time.GetAsFileTime());}
	int operator != ( MyTime time ) const {return CompareFileTime(&m_ft, &time.GetAsFileTime());}
	int operator <  ( MyTime time ) const {return ( CompareFileTime(&m_ft, &time.GetAsFileTime()) < 0 ? 1 : 0 );}
	int operator >  ( MyTime time ) const {return ( CompareFileTime(&m_ft, &time.GetAsFileTime()) > 0 ? 1 : 0 );}
	int operator <= ( MyTime time ) const {return ( CompareFileTime(&m_ft, &time.GetAsFileTime()) <= 0 ? 1 : 0 );}
	int operator >= ( MyTime time ) const {return ( CompareFileTime(&m_ft, &time.GetAsFileTime()) >= 0 ? 1 : 0 );}

	void Set(const SYSTEMTIME& pSysTime);
	void Set(const FILETIME& pFileTime);
	void Set(const char* pszTime);
	void SetUTCTime();
	void SetLocalTime();
	void SetRfc1123Time(const char* pszTime);
	void SetRfc1036Time(const char* pszTime);
	void SetAsctime(const char* pszTime);

private:
	static char*	m_pszMonth[12];
	static char*	m_pszWeekday[7];
	static char*	m_pszWkday[7];
	char			m_szBuffer[32];
	SYSTEMTIME		m_st;
	FILETIME		m_ft;
};


#endif // !defined(AFX_MyTime_H__06701547_4004_4FD0_B808_2F79A0A60673__INCLUDED_)
