/* =========================================================================
	This library is part of IshiSoft application.
	------------------------------------------------------------------------
	author: Yuichiro Ishitani
			http://www001.upp.so-net.ne.jp/ishi/index.html
  ========================================================================== */
#include "stdafx.h"
#include "MyTmpFileMgr.h"
#include "str.h"
#include "MyString.h"

MyTmpFileMgr::MyTmpFileMgr()
{
	m_hFile = INVALID_HANDLE_VALUE;
	m_wszFileName[0] = '\0';
}

MyTmpFileMgr::~MyTmpFileMgr()
{
	this->Close();
	DeleteFile( m_wszFileName );
}

// ----------------------------------------------------------------------------
// CreateFile
// ----------------------------------------------------------------------------
int MyTmpFileMgr::Create( const char* pszTmpDir )
{
	SYSTEMTIME st;
	GetLocalTime( &st );

	if( NULL == pszTmpDir )
		return 0;

	if( m_hFile != INVALID_HANDLE_VALUE )
	{
		Delete();
	}

	MyString s;

	if( 0 == s.Set( pszTmpDir ) )
	{
		return 0;
	}

	// Create MessageID
	wsprintf( m_wszFileName, L"%s%08x-%04d%02d%02d-%02d%02d%02d-%03d.txt"
		, (const WCHAR*)s
		, GetCurrentThreadId()
		, st.wYear, st.wMonth+1, st.wDay, st.wHour, st.wMinute, st.wSecond, st.wMilliseconds );

	m_hFile = CreateFile( m_wszFileName, GENERIC_READ|GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_ALWAYS, 0, NULL );

	if( m_hFile == INVALID_HANDLE_VALUE )
		return 0;

	return 1;
}

//-----------------------------------------------------------------------------
//	@detail
//		Append line with crlf to file
//	@param
//		pszAppend
//	@return
//
//-----------------------------------------------------------------------------
int MyTmpFileMgr::AppendLine( const char* pszAppend )
{
	if( NULL == pszAppend )			return 0;

	if( '\0' == *pszAppend )
	{
		if( 0 == Append( "\r\n" ) )	return 0;
		else						return 2;
	}

	if( 0 == Append( pszAppend ) )	return 0;
	if( 0 == Append( "\r\n" ) )		return 0;

	return STRLEN( pszAppend ) + 2;
}

//-----------------------------------------------------------------------------
//	@detail
//		Append line to file
//	@param
//		pszAppend
//	@return
//
//-----------------------------------------------------------------------------
int MyTmpFileMgr::Append( const char* pszAppend )
{
	DWORD dwWritten = 0;
	DWORD dwTotal = 0;
	DWORD dwRemain = 0;

	if( NULL == pszAppend )
		return 0;

	char* pStr = (char*)pszAppend;

	if( m_hFile == INVALID_HANDLE_VALUE )
		return 0;

	dwTotal = dwRemain = STRLEN( pszAppend );

	while( dwRemain )
	{
		if( 0 == WriteFile( m_hFile, pStr, dwRemain, &dwWritten, NULL ) )
		{
			Close();
			return 0;
		}
		pStr += dwWritten;
		dwRemain -= dwWritten;
	}

	return dwTotal;
}

//-----------------------------------------------------------------------------
//	@detail
//		Close
//	@param
//		nothing
//	@return
//		
//-----------------------------------------------------------------------------
void MyTmpFileMgr::Close()
{
	if( m_hFile != INVALID_HANDLE_VALUE )
	{
		CloseHandle( m_hFile );
		m_hFile = INVALID_HANDLE_VALUE;
	}
}

//-----------------------------------------------------------------------------
//	@detail
//		Get temp filename
//	@param
//		pwszDst
//		nLen
//	@return
//		
//-----------------------------------------------------------------------------
int MyTmpFileMgr::GetFileName( WCHAR* pwszDst, int nLen ) const
{
	WCHAR* p = (WCHAR*)m_wszFileName;

	size_t ii;
	int flag;
	WCHAR* dst = pwszDst;
	const WCHAR* src = m_wszFileName;

	for( ii = 0, flag = 0; ii < (unsigned)nLen; ii++ )
	{
		if( *src == '\0' )
			flag = 1;

		*dst++ = ( flag ? '\0' : *src++ );
	}

	return ii;
}

//-----------------------------------------------------------------------------
//	@detail
//		Get tmp filename
//	@param
//		
//	@return
//		
//-----------------------------------------------------------------------------
int MyTmpFileMgr::GetFileName( char* pszDst, int nLen ) const
{
	if( NULL == pszDst || 0 >= nLen )
		return 0;

	char szSrc[UNIQUE_ID_MAX_LEN];
	WideCharToMultiByte( CP_ACP, 0, m_wszFileName, -1, szSrc, UNIQUE_ID_MAX_LEN, NULL, NULL );

	STRNCPY( pszDst, szSrc, nLen );
	return STRLEN( pszDst );
}

//-----------------------------------------------------------------------------
//	@detail
//		
//	@param
//		
//	@return
//		
//-----------------------------------------------------------------------------
void MyTmpFileMgr::Delete()
{
	this->Close();
	DeleteFile( m_wszFileName );
	m_wszFileName[0] = '\0';
}

//-----------------------------------------------------------------------------
//	@detail
//		
//	@param
//		
//	@return
//		
//-----------------------------------------------------------------------------
DWORD MyTmpFileMgr::GetFileSize() const
{
	BY_HANDLE_FILE_INFORMATION hfi;
	if( INVALID_HANDLE_VALUE == m_hFile )
		return 0;

	GetFileInformationByHandle( m_hFile, &hfi );

	return hfi.nFileSizeLow;
}
