/* =========================================================================
	This library is part of IshiSoft application.
	------------------------------------------------------------------------
	author: Yuichiro Ishitani
			http://www001.upp.so-net.ne.jp/ishi/index.html
  ========================================================================== */
#include <stdlib.h>
#include "MyVarMem.h"

MyVarMem::MyVarMem()
{
	m_pBuffer = NULL;
	m_nBlockSize = 512;
	m_nBufferLen = 0;
}

MyVarMem::~MyVarMem()
{
	FREEIF(m_pBuffer);
}

//-----------------------------------------------------------------------------
//	@detail
//		Insert data to allocated memory.
//	@param
//		const void* pAddDat		Data for insert
//		int nDataLen		
//	@return
//		success	: Length of insert data.
//		fail	: 0
//-----------------------------------------------------------------------------
size_t MyVarMem::AddHead( const void* pAddData, size_t nDataLen)
{
	if(( 0 == nDataLen )||( NULL == pAddData ))
		return 0;
	
	void* pNewBuffer = NULL;

	if( NULL == m_pBuffer )
	{
		// Initioal data set.
		m_pBuffer = MALLOC(m_nBlockSize);
		
		// Insert terminate data.
		if( NULL != m_pBuffer )
		{
			memset((char*)m_pBuffer, 0, sizeof(m_nBlockSize));	// 2001/08/13
			//((char*)m_pBuffer)[0] = '\0';
			// Terminator
			m_nBufferLen = 0;
		}
		else
		{
			return 0;											// 2003/04/12
		}
	}

	// Real size is m_nBufferLen + 1
	size_t nNewTotal = nDataLen + m_nBufferLen + 1;
	// Create new page
	int nNewPage = (int)(nNewTotal / m_nBlockSize)
									+ ((nNewTotal % m_nBlockSize) ? 1 : 0);

	pNewBuffer = MALLOC(nNewPage * m_nBlockSize);
		
	if( NULL == pNewBuffer )
	{
		return 0;
	}

	// Paste insert data first.
	memcpy(pNewBuffer, pAddData, nDataLen);

	// Paste original data second.
	memcpy((char*)pNewBuffer + nDataLen, m_pBuffer, m_nBufferLen);

	FREEIF(m_pBuffer);
	m_pBuffer = pNewBuffer;

	m_nBufferLen = nNewTotal - 1;	// Terminate data '\0' is not counted.
	((char*)m_pBuffer)[nNewTotal] = '\0';

	return nDataLen;
}

//-----------------------------------------------------------------------------
//	@detail
//		Append a string.
//	@param
//		const char* pszAddData
//				: null terminated stirng
//	@return
//		success	: length of a data added
//		fail	: 0
//-----------------------------------------------------------------------------
size_t MyVarMem::Add( const char* pszAddData )
{
	if( pszAddData ==NULL )
		return 0;

	char* p = (char*)pszAddData;
	for(size_t ii = 0; *p; p++, ii++);

	return Add((const void*)pszAddData, ii);

}

//-----------------------------------------------------------------------------
//	@detail
//		add a data
//	@param
//		const void* pAddData		: pointer to the add data
//		int nDataLen				: length of the add data
//	@return
//		success	: length of a data added
//		fail	: 0
//-----------------------------------------------------------------------------
size_t MyVarMem::Add( const void* pAddData, size_t nDataLen)
{
	if(( 0 == nDataLen )||( NULL == pAddData ))
		return 0;

	void* pNewBuffer = NULL;

	if( NULL == m_pBuffer )
	{
		// Create initial data
		m_pBuffer = MALLOC(m_nBlockSize);
		
		if( NULL != m_pBuffer )
		{
			memset((char*)m_pBuffer, 0x00, m_nBlockSize);	// 2001/08/13
			m_nBufferLen = 0;
		}
		else
		{
			return 0;										// 2003/04/12
		}
	}

	// real size of Allocated is m_nBufferLen + 1
	size_t nNewTotal = nDataLen + m_nBufferLen + 1;
	size_t nOldTotal = m_nBufferLen + 1;
	// create new buffer size
	int nNewPage = (int)(nNewTotal / m_nBlockSize)
									+ ((nNewTotal % m_nBlockSize) ? 1 : 0);

	int nOldPage = (int)(nOldTotal / m_nBlockSize)
									+ ((nOldTotal % m_nBlockSize) ? 1 : 0);

	if(nNewPage > nOldPage)
	{
		// The return value is NULL if there is not enough available memory
		// to expand the block to the given size.

		pNewBuffer = REALLOC(m_pBuffer, (nNewPage * m_nBlockSize));
		if( NULL == pNewBuffer )
		{
			return 0;
		}
	
		m_pBuffer = pNewBuffer;
	}

	memcpy((char*)m_pBuffer + m_nBufferLen, pAddData, nDataLen);
	m_nBufferLen = nNewTotal - 1;	// subtracts null terminate.
	((char*)m_pBuffer)[m_nBufferLen] = '\0';

	return nDataLen;
}

//-----------------------------------------------------------------------------
//	@detail
//		Release all data.
//	@param
//		none
//	@return
//		none
//-----------------------------------------------------------------------------
void MyVarMem::Clear()
{
	FREEIF(m_pBuffer);
	m_nBufferLen = 0;
}

