/* =========================================================================
	This library is part of IshiSoft base class.
	------------------------------------------------------------------------
	author: Yuichiro Ishitani
			http://www001.upp.so-net.ne.jp/ishi/index.html
  ========================================================================== */
#if !defined(_MYVARMEM_H_INCLUDED_)
#define _MYVARMEM_H_INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "wincemem.h"

/* ****************************************************************************

 variable memory class

	Expand its allocated memory on the demand.
	The paging size is 512Bytes

	basic structure of a memory

	data		:|<...  data  ...>|\0|	<- add the null terminate automatically.
	data length	:|<-m_nBufferLen->|		<- MyVarMem.GetLength() returns this score.
	real length	:|<-          512*n            ->|


**************************************************************************** */

class MyVarMem 
{
public:
	MyVarMem();
	virtual ~MyVarMem();

private:
	void*	m_pBuffer;		// Pointer to allocated buffer
	size_t	m_nBufferLen;	// Length of the contents
	int		m_nBlockSize;	// Block size

public:
	
	void* GetBuffer()	const	{return m_pBuffer;}
	size_t GetLength()	const	{return m_nBufferLen;}

	size_t AddHead( const void* pAddData, size_t nDataLen);
	size_t Add( const void* pAddData, size_t nDataLen);
	
	size_t AddHead(MyVarMem* pH)	{return AddHead(pH->GetBuffer(), pH->GetLength());}
	size_t Add(MyVarMem* pH)		{return Add(pH->GetBuffer(), pH->GetLength());}
	
	size_t Add(const char* pszStr);

	virtual void Clear();
	virtual int IsEmpty()	{return (m_pBuffer==NULL)?1:0;}

	operator const void*()	{return (const void*)m_pBuffer;}
	operator const char*()	{return (const char*)m_pBuffer;}
};
#endif // !defined(_MYVARMEM_H_INCLUDED_)
