/* =========================================================================
	This library is part of IshiSoft application.
	------------------------------------------------------------------------
	author: Yuichiro Ishitani
			http://www001.upp.so-net.ne.jp/ishi/index.html
  ========================================================================== */
#include "stdafx.h"
#include "Str.h"
#include "commonhelper.h"
#include "MyConf.h"
#include "MyString.h"
#include "Usr.h"
#include "staticvariable.h"
#include "stdio.h"

extern MyConf	g_conf;

Usr::Usr()
{

}

Usr::~Usr()
{

}

//-----------------------------------------------------------------------------
//	@detail
//		Get user home directory
//	@param
//		pszUsrName:			<user name> or <user name>@<localdomain>
//							mast have MAX_PATH length.
//		pszDir:				
//	@return
//		USR_LOCAL_DOMAIN
//		USR_OTHER_DOMAIN
//		USR_ERR_NAME
//		USR_ERR_BADPARAM	--> not error but not local user
//-----------------------------------------------------------------------------
int Usr::GetHomePath(const char *pszUsrName, char* pszDir)
{
	char		szUsrDir[MAX_PATH] = {0};
	char		szTmpUser[MAX_LOCALUSERNAME_LEN64+1] = {0};
	char*		pDomain = NULL;
	const char* pMe = g_conf.GetValueI(CONFIG_ME);
	const char* pszHome = g_conf.GetValueI(CONFIG_HOMEDIR);
	unsigned int	nUserNameLength;

	if( NULL == pszUsrName || NULL == pMe || NULL == pszHome )
		return USR_ERR_BADPARAM;

	STRNCPY( szTmpUser, pszUsrName, MAX_LOCALUSERNAME_LEN64 );

	pDomain = STRRCHR( szTmpUser, '@' );
	
	// chop atmark for get username.
	if( NULL != pDomain )
	{
		*pDomain++ = '\0';
	}

	// User name length must be MAX_USER_NAME(64) or less
	nUserNameLength = STRLEN( szTmpUser );
	if( nUserNameLength > MAX_LOCALUSERNAME_LEN64 || nUserNameLength == 0 )
	{
		return USR_ERR_NAME;
	}

	// Is this my domain?
	if( NULL != pDomain )
	{
		if( 0 != STRICMP( pDomain, pMe ) )
		{
			return USR_OTHER_DOMAIN;
		}
	}

	STRNCPY( szUsrDir, pszHome, MAX_CONFIG_DIR_PATH );
	STRNCAT( szUsrDir, szTmpUser, MAX_LOCALUSERNAME_LEN64 );

	if( NULL != pszDir )
	{
		STRNCPY( pszDir, szUsrDir, MAX_PATH );
	}
	return USR_LOCAL_DOMAIN;
}

//-----------------------------------------------------------------------------
//	@detail
//		Check the user is on this server.
//	@param
//		pszUsrName	user name
//	@return
//		USR_LOCAL_DOMAIN
//		USR_OTHER_DOMAIN
//		USR_ERR_NAME
//		USR_ERR_BADPARAM
//		USR_ERR_MEMORY
//-----------------------------------------------------------------------------
int Usr::IsMyUser( const char* pszUsrName )
{
	return GetHomePath(pszUsrName, NULL);
}

//-----------------------------------------------------------------------------
//	@detail
//		Resolve alias
//	@param
//		const char* pszDstDataLocalFile
//		MyHashEx *phsRcptLocal
//		MyHashFile *phsRcptRemote)
//	@return
//-----------------------------------------------------------------------------
int Usr::ResolveAlias(const char* pszDstDataLocalFile, MyHashEx *phsRcptLocal, MyHashFile *phsRcptRemote)
{
	MyString s;
	if( 0 == s.Set(pszDstDataLocalFile) )
	{
		return 0;
	}

	HANDLE hFile = CreateFile( s, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, NULL );

	if( hFile == INVALID_HANDLE_VALUE )
	{
		return 0;
	}

	char szMailAddress[MAX_MAILADDR_LEN256+2]={0};

	while( NULL != GetFileLine( hFile, szMailAddress, MAX_MAILADDR_LEN256 ) )
	{
		if( !GetMailAddressString( szMailAddress, MAX_MAILADDR_LEN256, szMailAddress, g_conf.GetValueI(CONFIG_ME) ) )
		{
			continue;
		}

		switch( IsMyUser( szMailAddress ) )
		{
		case USR_OTHER_DOMAIN:	//0
			phsRcptRemote->Set( szMailAddress, "" );
			break;

		case USR_LOCAL_DOMAIN:	//1
			phsRcptLocal->Set( szMailAddress, "" );
			break;

		default:
			break;
		}

	}

	CloseHandle( hFile );
	return 1;
}

//-----------------------------------------------------------------------------
//	@detail
//		Deliver mail file
//	@param
//		const char *srcFile
//		MyHashEx *phsRcptLocal
//		MyHashFile *phsRcptRemote
//	@return
//		0 error
//		1 SUCCESS
//-----------------------------------------------------------------------------
int Usr::DeliveryFile(const char *srcFile, MyHashEx *phsRcptLocal, MyHashFile *phsRcptRemote, MyLog* plog)
{
	const char* pszHome = g_conf.GetValueI(CONFIG_HOMEDIR);
	const char* pszQueueDir = g_conf.GetValueI(CONFIG_QUEUEDIR);
	MyString sSrcDataFile;
	
	if( 0 == sSrcDataFile.Set( srcFile ) )
	{
		return 0;
	}

	MyString sQueueFile;
	MyString sDstDataRemoteFile;
	MyString sDstDataLocalFile;

	if( NULL == srcFile || NULL == phsRcptLocal || NULL == phsRcptRemote || NULL == plog
		|| NULL == pszHome || NULL == pszQueueDir || 0 == STRLEN( sSrcDataFile ) )
		return 0;

	// safety check. is exist?
	DWORD dwFileAttr = GetFileAttributesMBCS( srcFile );

	char szDstDataFile[MAX_PATH] = {0};
	char szDstQueueFile[MAX_PATH] = {0};

	if( 0xFFFFFFFF == dwFileAttr )
		return 0;

	// get filename(remove filepath)
	const char* pFileName = srcFile;

	if( NULL == pszQueueDir )
		return 0;

	// move to last
	for( ; *pFileName != '\0'; pFileName++ );
	// move to end of backslash
	for( ; *pFileName != '\\' && pFileName != srcFile && pFileName != srcFile; pFileName-- );

	// ------------------------------------------------------------------------
	// resolve alias
	char szDbgMsg[512] = {0};
	char* pAtmark = NULL;
	char szLocalUserName[MAX_LOCALUSERNAME_LEN64+1] = {0};
	SEEKKEY skUsrName = phsRcptLocal->SeekToFirst();

	int nSafeCounter = 0;
	while( skUsrName )
	{
		// when loop counter over 10000, stop resolving alias.
		if( nSafeCounter++ > 10000 )
		{
			plog->Err( "alias loop found. some recipients dropped from recipient list." );
			break;
		}

		STRNCPY( szLocalUserName, skUsrName, MAX_LOCALUSERNAME_LEN64 );

		pAtmark = STRRCHR( szLocalUserName, '@' );
		
		if( NULL != pAtmark )
		{
			*pAtmark = '\0';
		}

		STRNCPY( szDstDataFile, pszHome, MAX_CONFIG_DIR_PATH );
		STRNCAT( szDstDataFile, szLocalUserName, MAX_LOCALUSERNAME_LEN64 );

		if( 0 == sDstDataLocalFile.Set( szDstDataFile ) )
		{
			skUsrName = phsRcptLocal->SeekNext( skUsrName );
			continue;
		}

		dwFileAttr = GetFileAttributes( sDstDataLocalFile );
		if( !(FILE_ATTRIBUTE_DIRECTORY & dwFileAttr) )
		{
			phsRcptLocal->Del( skUsrName );
			ResolveAlias(sDstDataLocalFile, phsRcptLocal, phsRcptRemote);
			skUsrName = phsRcptLocal->SeekToFirst();
			continue;
		}
		
		skUsrName = phsRcptLocal->SeekNext( skUsrName );
	}
	// ------------------------------------------------------------------------
	// start of remote delivery

	// check remote rcpt
	if( NULL != phsRcptRemote->SeekToFirst() )
	{
		STRNCPY( szDstQueueFile, pszQueueDir, MAX_CONFIG_DIR_PATH );
		STRCAT( szDstQueueFile, "index" );
		STRCAT( szDstQueueFile, pFileName );

		if( 0 == sQueueFile.Set( szDstQueueFile ) )
		{
			return 0;
		}

		if( 0 == phsRcptRemote->SaveFile( sQueueFile ) )
		{
			return 0;
		}

		// tmp file
		STRNCPY( szDstDataFile, pszQueueDir, MAX_CONFIG_DIR_PATH );
		STRCAT( szDstDataFile, pFileName );

		if( 0 == sDstDataRemoteFile.Set( szDstDataFile ) )
		{
			return 0;
		}

		if( 0 == STRLEN( sDstDataRemoteFile ) || 0 == CopyFile( sSrcDataFile, sDstDataRemoteFile, FALSE ) )
		{
			DeleteFile( sQueueFile );
			return 0;
		}
		plog->Dbg( "queing remote delivery file." );
	}
	// end of remote delivery
	// ------------------------------------------------------------------------
	// start of local delivery
	
	skUsrName = phsRcptLocal->SeekToFirst();

	while( skUsrName )
	{
		// make user file path
		STRNCPY( szLocalUserName, skUsrName, MAX_LOCALUSERNAME_LEN64 );

		pAtmark = STRRCHR( szLocalUserName, '@' );
		
		if( NULL != pAtmark )
		{
			*pAtmark = '\0';
		}

		STRNCPY( szDstDataFile, pszHome, MAX_CONFIG_DIR_PATH );
		STRNCAT( szDstDataFile, szLocalUserName, MAX_LOCALUSERNAME_LEN64 );

		STRCAT( szDstDataFile, pFileName );

		if( 0 == sDstDataLocalFile.Set( szDstDataFile ) )
		{
			goto LOCAL_DELIVERY_FAILED;
		}

		if( 0 == STRLEN( sDstDataLocalFile ) || 0 == CopyFile( sSrcDataFile, sDstDataLocalFile, FALSE ) )
		{
			goto LOCAL_DELIVERY_FAILED;
		}
		sprintf( szDbgMsg, "delivery to (%s).", skUsrName );
		plog->Info( szDbgMsg );
		skUsrName = phsRcptLocal->SeekNext( skUsrName );
	}
	return 1;
	// end of local delivery

LOCAL_DELIVERY_FAILED: // roll back

	// delete remote file.
	if( NULL != phsRcptRemote->SeekToFirst() )
	{
		DeleteFile( sQueueFile );
		DeleteFile( sDstDataRemoteFile );
	}

	// delete all local file.
	skUsrName = phsRcptLocal->SeekToFirst();
	while( skUsrName )
	{
		STRNCPY( szLocalUserName, skUsrName, MAX_LOCALUSERNAME_LEN64 );

		pAtmark = STRRCHR( szLocalUserName, '@' );
		
		if( NULL != pAtmark )
		{
			*pAtmark = '\0';
		}

		STRNCPY( szDstDataFile, pszHome, MAX_CONFIG_DIR_PATH );
		STRNCAT( szDstDataFile, szLocalUserName, MAX_LOCALUSERNAME_LEN64 );

		STRCAT( szDstDataFile, pFileName );

		sDstDataLocalFile.Set( szDstDataFile );
		DeleteFile( sDstDataLocalFile );
		sprintf( szDbgMsg, "delivery failed. delete message file for (%s).", skUsrName );
		plog->Info( szDbgMsg );
		skUsrName = phsRcptLocal->SeekNext( skUsrName );
	}

	return 0;
}


