/* =========================================================================
	This library is part of IshiSoft application.
	------------------------------------------------------------------------
	author: Yuichiro Ishitani
			http://www001.upp.so-net.ne.jp/ishi/index.html
  ========================================================================== */
#include "stdafx.h"
#include "commonhelper.h"
#include "str.h"
#include "MyString.h"
#include "wincemem.h"
#include "staticvariable.h"


//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
#define ISDOMAINCHAR( c ) \
			(( 'a' <= c && c <= 'z' ) \
			|| ( 'A' <= c && c <= 'Z' ) \
			|| ( '0' <= c && c <= '9' ) \
			|| ( c == '@' ) \
			|| ( c == '.' ) \
			|| ( c == '-' ) )

#define ISMAILCHAR( c ) \
			((c >= 0x21) && (c <= 0x7E) \
			|| ( c != '<' ) || ( c != '>' ) \
			|| ( c != '(' ) || ( c != ')' ) \
			|| ( c != '[' ) || ( c != ']' ) \
			|| ( c != ';' ) || ( c != ':' ) || ( c != ',' ) || ( c != '"' ) || ( c != '\\' ) )

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
int StartProcess( const char* pszExe, const char* pszFile, int nCmdShow )
{
	int nRet;

	WCHAR wszParm[MAX_PATH+1] = {0};
	if( NULL != pszFile )
	{
		if( 0 == MultiByteToWideChar( CP_ACP,0,pszFile,-1,wszParm,MAX_PATH ) )
		{
			return 0;
		}
	}

	WCHAR wszExe[MAX_PATH+1] = {0};
	if( 0 == MultiByteToWideChar( CP_ACP,0,pszExe,-1,wszExe,MAX_PATH ) )
	{
		return 0;
	}

	WCHAR wszProcess[MAX_PATH+1] = {0};
	WCHAR* pExe = wszExe;
	WCHAR* pProcess = &wszProcess[0];
	if( 0 < GetModuleFileName( NULL, wszProcess, MAX_PATH ) )
	{
		while( *pProcess != '\0' ) pProcess++;
		while( *pProcess != '\\' && pProcess > wszProcess )
		{
			*pProcess-- = '\0';
		}
		++pProcess;
	}
	
	while( '\0' != *pExe )
	{
		*pProcess++ = *pExe++;
	}


	PROCESS_INFORMATION pi = {0};
	STARTUPINFO* psi = NULL;
#if !defined(_WIN32_WCE)
	STARTUPINFO si = {0};
	si.cb = sizeof( STARTUPINFO );
	si.dwFlags = STARTF_USESHOWWINDOW;
	si.wShowWindow = nCmdShow;
	si.hStdInput = NULL;
	si.hStdOutput = NULL;
	si.hStdError = NULL;

	psi = &si;
#endif

	nRet = CreateProcess( wszProcess, wszParm, NULL, NULL, FALSE, 0, NULL, NULL, psi, &pi );
	if( nRet )
	{
		CloseHandle( pi.hProcess );
		CloseHandle( pi.hThread );
	}
	return nRet;
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
int FileTimeCmp(FILETIME *pftA, FILETIME *pftB, unsigned long dwAddToBsec)
{
	unsigned __int64 llA = 0;
	unsigned __int64 llB = 0;

	llA = pftA->dwHighDateTime;
	llA <<= 32;
	llA += pftA->dwLowDateTime;
	
	llA /= 10000000L;

	llB = pftB->dwHighDateTime;
	llB <<= 32;
	llB += pftB->dwLowDateTime;
	
	llB /= 10000000L;
	llB += dwAddToBsec;

	if(llA > llB)
		return 1;
	
	return 0;
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
char* TRIM( char* pszSrc )
{
	char* pHead = pszSrc;
	char* pTail = pszSrc;

	while( ' ' == *pTail ) pTail++;
	
	while( '\0' != *pTail ) *pHead++ = *pTail++;
	*pHead-- = '\0';

	// Tail trim
	while( ' ' == *pHead && pHead >= pszSrc ) *pHead-- = '\0';

	return pszSrc;
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
DWORD GetFileAttributesMBCS( const char* lpFileName )
{
	WCHAR	tszFile[MAX_PATH+1]={0};

	if( 0 == MultiByteToWideChar(CP_ACP,0,lpFileName,-1,tszFile,MAX_PATH) )
	{
		return 0xFFFFFFFF;
	}

	return GetFileAttributes( tszFile );
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
int StrageCheck(const char* pszRoot, unsigned long* pdwMemory)
{
	// no check
	if( 0 == *pdwMemory )
		return 1;

	MyString s;
	if( 0 == s.Set( pszRoot ) )
	{
		return 0;
	}

	ULARGE_INTEGER   uFree = {0};
	ULARGE_INTEGER   uTotal = {0};
	// ULARGE_INTEGER   uTotalFree = {0};

	if( !GetDiskFreeSpaceEx( s, &uFree, &uTotal, NULL ) )
	{
		*pdwMemory = 0;
		return 1;
	}

	if( uFree.HighPart > 0 )
	{
		*pdwMemory = 0xFFFFFFFF;
		return 1;
	}

	if( uFree.LowPart > *pdwMemory )
	{
		*pdwMemory = uFree.LowPart;
		return 1;
	}

	return 0;
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
int MemoryCheck(unsigned long* pdwMemory)
{
	// nocheck
	if( 0 == *pdwMemory )
		return 1;
	MEMORYSTATUS ms;
	ms.dwLength = sizeof(MEMORYSTATUS);

	GlobalMemoryStatus( &ms );

	if( ms.dwAvailPhys > *pdwMemory )
	{
		*pdwMemory = ms.dwAvailPhys;
		return 1;
	}

	return 0;
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
/*
HANDLE FindFirstFileMBCS( const char* lpFileName, LPWIN32_FIND_DATA lpFindFileData )
{
	WCHAR	tszFile[MAX_PATH+1]={0};

	if( 0 == MultiByteToWideChar( CP_ACP,0,lpFileName,-1,tszFile,MAX_PATH ) )
	{
		return INVALID_HANDLE_VALUE;
	}

	return FindFirstFile( tszFile, lpFindFileData );
}
*/
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
/*
BOOL CopyFileMBCS(  const char* lpExistingFileName, const char* lpNewFileName, BOOL bFailIfExists )
{
	MyString sExist( lpExistingFileName );
	MyString sNew( lpNewFileName );
	
	const WCHAR* pExist = (const WCHAR*)sExist;
	const WCHAR* pNew = (const WCHAR*)sNew;

	if( pExist && pNew )
		return CopyFile( pExist, pNew, bFailIfExists );
	else
		return FALSE;
}
*/
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
/*
BOOL DeleteFileMBCS( const char* lpFileName )
{
	WCHAR	tszFile[MAX_PATH+1]={0};

	if( 0 == MultiByteToWideChar(CP_ACP,0,lpFileName,-1,tszFile,MAX_PATH) )
	{
		return 0;
	}

	return DeleteFile( tszFile );
}
*/
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
/*
BOOL MoveFileMBCS(const char* lpExistingFileName, const char* lpNewFileName)
{
	MyString sExist( lpExistingFileName );
	MyString sNew( lpNewFileName );
	
	const WCHAR* pExist = (const WCHAR*)sExist;
	const WCHAR* pNew = (const WCHAR*)sNew;

	if( pExist && pNew )
		return MoveFile( pExist, pNew );
	else
		return FALSE;
}
*/
//-----------------------------------------------------------------------------
// 'user@domain'				-> 'user@domain'
// 'foo <user@domain>			-> 'user@domain'
// '@a,@b:user@domain'			-> 'user@domain'
// 'foo <@a,@b:user@domain>'	-> 'user@domain'
// '<>'							-> ''
// '<@hosta.int,@jkl.org:userc@d.bar.org>
//								-> 'userc@d.bar.org'
//-----------------------------------------------------------------------------
int GetMailAddressString( char* pDst, int nDstLen, const char* pSrc, const char* pszDefaultDomain )
{
	char* pszMailAddHead = NULL;
	char* pszMailAddTail = NULL;
	char* pszAtMark = NULL;
	char* pTmp = NULL;
	char* pL = NULL;
	char* pR = NULL;
	char* pCoron = NULL;

	if( NULL == pSrc || NULL == pDst || 0 == nDstLen )
		return 0;

	// forward strip
	while( (' ' == *pSrc) ) pSrc++;

	if( '\0' == *pSrc )
		return 0;

	char* pDupStr = STRDUP( pSrc );

	if( NULL == pDupStr )
	{
		return 0;
	}

	// reverse strip
	pR = pDupStr;
	while( '\0' != *pR ) pR++;
	pR--;
	while( ' ' == *pR && pR >= pDupStr )	*pR-- = '\0';

	if( MAX_MAILADDR_LEN256 < STRLEN( pDupStr ) )
		goto THIS_ERROR;

	// clear dest buffer
	int ii;
	for( ii = 0; ii < nDstLen; ii++ )	pDst[ii] = '\0';

	pL = STRCHR( pDupStr, '<' );
	pR = STRCHR( pDupStr, '>' );
	
	// ----------------------------------------------------
	// strip mail address/ mail address not surrounded by "<" and ">"
	// ----------------------------------------------------
	if( ( ( NULL != pL && NULL == pR ) || ( NULL == pL && NULL != pR ) )	// e.g. '<yu-ishi@domain'
		|| ( ( NULL != pL && NULL != pR ) && ( pL > pR ) ) )				// e.g. '>yu-ishi@domain<'
		goto THIS_ERROR;

	// no '<' and '>'
	if( NULL == pL && NULL == pR )	// e.g. 'yu-ishi@example.com'
	{
		pCoron = STRRCHR( pDupStr, ':' );
		if( NULL != pCoron )
			pTmp = (pCoron+1);
		else
			pTmp = pDupStr;

		STRNCPY( pDst, pTmp, nDstLen );

		// easy mail char check
		for( pTmp = pDst; *pTmp != '\0'; pTmp++ )
		{
			if( !ISMAILCHAR( *pTmp ) )
			{
				goto THIS_ERROR;
			}
		}

		goto THIS_SUCCESS;
	}

	// ----------------------------------------------------
	// strip mail address/ mail address surrounded by "<" and ">"
	// ----------------------------------------------------
	pszMailAddHead = pDupStr;
	pszMailAddTail = NULL;
	pszAtMark = NULL;

	// ishitani <ishitani@example.com>  -->  <ishitani@example.com>
	while( *pszMailAddHead != '<' && *pszMailAddHead != '\0' ) pszMailAddHead++;

	if( *pszMailAddHead == '\0' )
	{
		goto THIS_ERROR;
	}

	// ishitani@example.com>
	pszMailAddHead++;
	pszMailAddTail = pszMailAddHead;
	
	while( *pszMailAddTail != '>' && *pszMailAddTail != '\0' ) pszMailAddTail++;

	if( *pszMailAddTail == '\0' )
	{
		goto THIS_ERROR;
	}

	// ishitani@example.com
	*pszMailAddTail = '\0';

	pCoron = STRRCHR( pszMailAddHead, ':' );
	if( NULL != pCoron )
		pTmp = (pCoron+1);
	else
		pTmp = pszMailAddHead;

	STRNCPY( pDst, pTmp, nDstLen );

	// easy mail char check
	for( pTmp = pDst; *pTmp != '\0'; pTmp++ )
	{
		if( !ISMAILCHAR( *pTmp ) )
		{
			goto THIS_ERROR;
		}
	}

	goto THIS_SUCCESS;

THIS_SUCCESS:
	if( '\0' != *pDst )		// ex. not return-path: <>
	{
		if( NULL == STRRCHR( pDst, '@' ) )
		{
			STRCAT( pDst, "@" );
			STRCAT( pDst, pszDefaultDomain );
		}
	}
	STRFREE( pDupStr );
	return 1;

THIS_ERROR:
	*pDst = '\0';
	STRFREE( pDupStr );
	return 0;
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
char* GetFileLine(HANDLE hFile, char *szBuf, unsigned int dwBufLen)
{
	long lMoveLen;
	unsigned long ii;
	unsigned long dwReadBytes;
	char* szRet = szBuf;
	char* p = NULL;

	if( dwBufLen < 2 )
		return NULL;

	if( FALSE == ReadFile( hFile, szBuf, dwBufLen, &dwReadBytes, NULL ) )
	{
		return NULL;
	}

	if( 0 == dwReadBytes )
		return NULL;

	if( 1 == dwReadBytes )
	{
		if( '\r' == szBuf[0] || '\n' == szBuf[0] )
		{
			szBuf[0] = '\0';
		}
		szBuf[1] = '\0';
		return szRet;
	}

	for( ii = 0, p = szBuf; ii < dwReadBytes; ii++, p++ )
	{
		if( '\r' == p[0] )
		{
			*p = '\0';

			if( '\n' == p[1] )
				lMoveLen = (ii + 2) - dwReadBytes;
			else
				lMoveLen = (ii + 1) - dwReadBytes;
	
			SetFilePointer( hFile, lMoveLen, NULL, FILE_CURRENT );
			return szRet;
		}
		else if( '\n' == p[0] )
		{
			*p = '\0';
			lMoveLen = (ii + 1) - dwReadBytes;
	
			SetFilePointer( hFile, lMoveLen, NULL, FILE_CURRENT );
			return szRet;
		}
	}

	*p = '\0';
	return szRet;
}

// ------------------------------------------------------------------
// BASE64
// ------------------------------------------------------------------
static const char base64_charset[] ="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
static char B64(char c)
{
	if( c == '=' ) return 0;
	for( char ii = 0; base64_charset[ii] != c && base64_charset[ii] != '\0'; ii++ );
	return ii;
}

char* Base64Decode(char *pInOut)
{
	if( NULL == pInOut )
		return NULL;

    char *p = pInOut;

    int len = STRLEN(pInOut);
    for (int ii=0,jj=0; ii < len; ii+=4, jj+=3)
	{
        p[jj]   =(char)(  (B64(pInOut[ii])           << 2) | ((B64(pInOut[ii+1]) & 0x30) >> 4) );
        p[jj+1] =(char)( ((B64(pInOut[ii+1]) & 0x0f) << 4) | ((B64(pInOut[ii+2]) & 0x3c) >> 2) );
        p[jj+2] =(char)( ((B64(pInOut[ii+2]) & 0x03) << 6) |  (B64(pInOut[ii+3]) & 0x3f)       );
    }
    p[jj] = '\0';
    return (p);
}

char *Base64Encode(const char *pIn)
{
	char *p;
	char *pRet;
	int ii, jj;
	int len = STRLEN(pIn);

	pRet = p = (char*)MALLOC((len * 8) / 6 + 4);

	for (ii= 0; ii < len / 3; ++ii)
	{
		*p++ = base64_charset[*pIn >> 2];
		jj = ((*pIn & 0x03) << 4);
		pIn++;
		*p++ = base64_charset[ jj | (*pIn >> 4) ];
		jj = ((*pIn & 0x0F) << 2);
		pIn++;
		*p++ = base64_charset[ jj | (*pIn >> 6) ];
		*p++ = base64_charset[ *pIn++ & 0x3F ];
	}

	switch (len % 3) {
	case 2:
		*p++ = base64_charset[*pIn >> 2];
		jj =  ((*pIn & 0x03) << 4);
		pIn++;
		*p++ = base64_charset[jj | (*pIn >> 4)];
		*p++ = base64_charset[((*pIn++ & 0x0F) << 2)];
		break;
	case 1:
		*p++ = base64_charset[*pIn >> 2];
		*p++ = base64_charset[(*pIn++ & 0x03) << 4];
		break;
	}
	*p++ = '=';
	*p++ = '=';
	*p = 0;
	return pRet;
}

