/* =========================================================================
	This library is part of IshiSoft application.
	------------------------------------------------------------------------
	author: Yuichiro Ishitani
			http://www001.upp.so-net.ne.jp/ishi/index.html
  ========================================================================== */
#include "wincemem.h"
#include "str.h"

// -------------------------------------------------------------------
unsigned int ATOUI( const char* src )
{
	unsigned int ret = 0;

	if( NULL == src )
		return 0;

	while( (*src > '9' && '0' > *src) && *src != '\0' )  src++;

	while( '0' <= *src && *src <= '9' )
	{
		ret *= 10;
		ret += (*src - '0');
		src++;
	}

	return ret;
}

// -------------------------------------------------------------------
int ATOI(const char* src)
{
	int	minus = 0;
	int nAns = 0;

	if( NULL == src )
		return 0;

	while( (*src > '9' && '0' > *src) && *src != '-' && *src != '\0' )  src++;

	if( *src == '-' )
	{
		minus = 1;
		src++;
	}

	while( '0' <= *src && *src <= '9' )
	{
		nAns *= 10;
		nAns += (*src - '0');
		src++;
	}

	return (minus?nAns*-1:nAns);
}

// -------------------------------------------------------------------
size_t STRLEN( const char *string )
{
	size_t ii = 0;

	while( *string++ != '\0' )
		ii++;
	
	return ii;
}

// -------------------------------------------------------------------
char *STRCPY( char *dst, const char *src )
{
	char* p = dst;
	do
	{
		*dst++ = *src++;
	}while( *src != '\0' );

	*dst = '\0';

	return p;
}

// -------------------------------------------------------------------
char *STRNCPY( char *dst, const char *src, size_t count )
{
	size_t ii;
	int flag;
	char* p = dst;

	for( ii = 0, flag = 0; ii < count; ii++ )
	{
		if( *src == '\0' )
			flag = 1;

		*dst++ = ( flag ? '\0' : *src++ );
	}

	return p;
}

// -------------------------------------------------------------------
char *STRUPR( char *src )
{
	char *p;

	for ( p = src; *p; ++p )
	{
		if( 'a' <= *p && *p <= 'z' )
			*p += 'A' - 'a';
	}

	return src;
}

// -------------------------------------------------------------------
char *STRLWR( char *src )
{
	char *p;

	for ( p = src; *p; ++p )
	{
		if( 'A' <= *p && *p <= 'A' )
			*p += 'a' - 'A';
	}

	return src;
}

// -------------------------------------------------------------------
char *STRCAT( char *dst, const char *src )
{
	char* p = dst;

	while( *dst != '\0' )
	{
		dst++;
	}

	while( *src != '\0' )
	{
		*dst++ = *src++;
	}

	*dst = '\0';

	return p;
}

// -------------------------------------------------------------------
char *STRNCAT( char *dst, const char *src, size_t count )
{
	char* p = dst;
	size_t ii = 0;

	while( *dst != '\0' )
		dst++;

	for( ii = 0; ii < count && *src; ii++ )
	{
		*dst++ = *src++;
	}
	
	*dst = '\0';

	return p;

}

// -------------------------------------------------------------------
char *STRSET( char *src, int c )
{
	char* p;

	for( p = src; *p; *p++ = (char)c )
		;

	return src;
}

// -------------------------------------------------------------------
int STRCMP( const char *str1, const char *str2 )
{
	while( *str1 && (*str1 == *str2) )
	{
		str1++;
		str2++;
	}
	
	return *str1 - *str2;
}

// -------------------------------------------------------------------
int STRICMP( const char *str1, const char *str2 )
{
	int a, b;

	do
	{
		a = (unsigned char)*str1;
		b = (unsigned char)*str2;

		if( (a >= 'A') && (a <= 'Z') )
			a += 'a' - 'A';

		if( (b >= 'A') && (b <= 'Z') )
			b += 'a' - 'A';
		str1++;
		str2++;
	}while( a && (a == b) );
	
	return a - b;
}

// -------------------------------------------------------------------
int STRNCMP( const char *str1, const char *str2, size_t count )
{
	size_t ii = 0;

	while( (++ii < count) && *str1 && (*str1 == *str2) )
	{
		str1++;
		str2++;
	};
	
	return *str1 - *str2;
}

// -------------------------------------------------------------------
int STRNICMP( const char *str1, const char *str2, size_t count )
{
	size_t ii = 0;
	int a, b;

	while( (ii++ < count) )
	{
		a = (unsigned char)*str1;
		b = (unsigned char)*str2;

		if( (a >= 'A') && (a <= 'Z') )
			a += 'a' - 'A';

		if( (b >= 'A') && (b <= 'Z') )
			b += 'a' - 'A';

		if( a == '\0' || (a != b) )
			break;

		str1++;
		str2++;
	};
	
	return a - b;
}

// -------------------------------------------------------------------
char *STRDUP( const char *src )
{
	size_t len = 0;
	char *mem = NULL;
	const char *s = src;
	char *d = NULL;

	if( NULL == src )
		return NULL;

	while( *s++ )
		len++;

	mem = (char*)MALLOC( len + 1 );

	if( NULL == mem )
		return NULL;

	s = src;
	d = mem;
	do
	{
		*d++ = *s++;
	}while( *s != '\0' );

	*d = '\0';

	return mem;
}

// -------------------------------------------------------------------
void STRFREE( const char * p )
{
	if( p ) FREE( (void*)p );
}

// -------------------------------------------------------------------
char *STRREV( char *string )
{
	char *ret = string;
	char *head = string;
	char *tail;
	int len = 0;
	char c = 0;

	len = 0;
	tail = string;
	while( *tail++ != '\0' )
		len++;

	if( len == 0 )
		return ret;

	tail = string + len -1 ;

	do
	{
		c = *head;
		*head = *tail;
		*tail = c;

	}while( ++head < --tail );

	return ret;
}

// -------------------------------------------------------------------
char *STRCHR( const char *string, int c )
{
	do
	{
		if( *string == (char)c )
			return (char*)string;
	}while( *string++ );

	return NULL;
}

// -------------------------------------------------------------------
char *STRRCHR( const char *string, int c )
{
	char *p = (char*)string;

	while( *p++ != '\0' )
		;

	while( p-- != string )
	{
		if( *p == (char)c )
			return p;
	}

	return NULL;
}

// -------------------------------------------------------------------
char *STRSTR( const char *dst, const char *src )
{
	char *p = NULL;
	char *d = NULL;
	char *s = NULL;

	if( *src == '\0' )
		return (char*)dst;

	d = (char*)dst;
	while( *d != '\0' )
	{
		s = (char*)src;
		p = d;

		while( *s++ == *p++ )
		{
			if( *s == '\0' )
				return d;
		};
		
		d++;
	}
	return NULL;
}

// -------------------------------------------------------------------
size_t STRSPN( const char *dst, const char *src )
{
	size_t pos = 0;
	char *s = NULL;
	
	while( *dst)
	{
		s = (char*)src;

		while( *s != '\0' )
		{
			if( *dst == *s )
				break;
			
			s++;
		}

		if( *s == '\0' )
			break;

		dst++;
		pos++;
	}
	
	return pos;

}
// -------------------------------------------------------------------
size_t STRCSPN( const char *dst, const char *src )
{
	char* start;
	size_t pos;

	while( *src != '\0' )
	{
		pos = 0;
		start = (char*)dst;

		while( *start != '\0' )
		{
			if( *start == *src )
				return pos;
			
			pos++;
			start++;
		}
		src++;
	}
	
	return pos;
}

// -------------------------------------------------------------------
char *STRTOK( char *dst, const char *dlmt )
{
	static char* token = NULL;
	char *ret = NULL;
	size_t	head = 0;
	size_t	tail = 0;

	if( NULL != dst )
		token = dst;

	if( *token == '\0' )
		return NULL;

	if( NULL == token )
		return NULL;

	head = STRSPN( token, dlmt );

	tail = STRCSPN( (token + head), dlmt );

	ret = token + head;
	
	*(token + head + tail) = '\0';
	token += head + tail + 1;

	return ret;
}

// --------------------------------------
// Copyright (C) IshiSoft
// http://www001.upp.so-net.ne.jp/ishi/
// bug report > yu-ishi@ya2.so-net.ne.jp 
