#include "stdafx.h"
#include "stdio.h"
#include "MySmtpTalker.h"
#include "MyString.h"
#include "str.h"
#include "commonhelper.h"
#include "MXRecord.h"
#include "MyTmpFileMgr.h"
#include "usr.h"
#include "MyTime.h"

char gszDbgMsg[1024];
MyConf g_conf;

MySmtpTalker::MySmtpTalker()
{
	*m_szMailFrom = '\0';
	*m_szSrcFile = '\0';
	m_hIndexFile = INVALID_HANDLE_VALUE;
	m_hDataFile = INVALID_HANDLE_VALUE;

	m_log.SetFacility(LOGFACILITY_MAIL);
	m_log.SetTag("SSMTP");
	m_log.Dbg( "Pegion started." );
}

MySmtpTalker::~MySmtpTalker()
{
	if( m_hIndexFile != INVALID_HANDLE_VALUE ) CloseHandle( m_hIndexFile );
	if( m_hDataFile  != INVALID_HANDLE_VALUE ) CloseHandle( m_hDataFile );
}

//-----------------------------------------------------------------------------
//	@detail
//		SendMail
//	@param
//		const char* pszSrcName /*= NULL*/
//	@return
//		0 indicate an error
//-----------------------------------------------------------------------------
int MySmtpTalker::SendMail(const char* pszSrcName/*= NULL*/)
{
	if( NULL != pszSrcName )
	{
		SetDataFileName(pszSrcName);
		return ProcessQueue();
	}

	// no source name

	HANDLE hff = INVALID_HANDLE_VALUE;
	WIN32_FIND_DATA wfd = {0};
	const char* pszQueueDir = g_conf.GetValueI(CONFIG_QUEUEDIR);
	if( NULL == pszQueueDir )
	{
		m_log.Err("load 'queue' value failed.");
		return 0;
	}

	char szDir[MAX_PATH] = {0};
	MyString sDir;
	
	if( 0 == sDir.Set(pszQueueDir) )
	{
		m_log.Err( "enum file failed by low memory." );
		return 0;
	}

	STRCPY( szDir, sDir );
	STRCAT( szDir, "*.*" );

	if( 0 == sDir.Set( szDir ) )
	{
		m_log.Err( "enum queue file failed by low memory." );
		return 0;
	}

	hff = FindFirstFile( sDir, &wfd );
	sDir.Clear();

	if( INVALID_HANDLE_VALUE == hff )
	{
		m_log.Dbg("SendMail():nofile to send.");
		return 1;	// no file
	}

	// ----------------------------------------------------
	// send mail loop
	// ----------------------------------------------------

	do
	{
		if( FILE_ATTRIBUTE_DIRECTORY & wfd.dwFileAttributes )
			continue;

		MyString s;
		if( 0 == s.Set( wfd.cFileName ) )
		{
			m_log.Err( "enum queue file failed by low memory." );
			FindClose( hff );
			return 0;
		}

		if( 0 == SetDataFileName( s ) )
		{
			sprintf( gszDbgMsg, "error occured while SetDataFileName(%s).", (const char*)s );
			m_log.Err( gszDbgMsg );
			DeleteIndexAndDataFile(m_szSrcFile);
			continue;
		}

		sprintf( gszDbgMsg, "Processing queue file(%s).", (const char*)s );
		m_log.Dbg( gszDbgMsg );

		if( !ProcessQueue() )
		{
			DeleteIndexAndDataFile(m_szSrcFile);
		}

	}while( FindNextFile( hff, &wfd ) );

	FindClose( hff );
	return 1;
}

//-----------------------------------------------------------------------------
//	@detail
//		Init this
//		clear		:m_hsRcptList, m_hsDomainList, m_hsRetryRcptList
//		open file	:m_hIndexFile, m_hDataFile
//		set			:m_szMailFrom, m_szSrcFile
//		if it needs, delete index and data file.
//	@param
//		const char* pszSrcName	// file name
//	@return
//		0 indicate an error
//-----------------------------------------------------------------------------
int MySmtpTalker::SetDataFileName(const char* pszSrcName)
{
	if( NULL == pszSrcName )
		return 0;

	const char* p = pszSrcName;
	while( '\0' != *p ) p++;
	while( '\\' != *p && pszSrcName != p ) p--;
	if( '\\' == *p )
		p++;

	STRNCPY( m_szSrcFile, p, MAX_PATH );

	// clear list
	m_hsRcptList.Clear();
	m_hsDomainList.Clear();
	m_hsRetryRcptList.Clear();

	// ------------------------------------------
	// open index file
	// ------------------------------------------
	char szTmpFileBuf[MAX_PATH] = {0};
	MyString s;
	
	GetIndexFilePath( m_szSrcFile, szTmpFileBuf, MAX_PATH );

	if( 0 == s.Set( szTmpFileBuf ) )
	{
		sprintf( gszDbgMsg, "open index file failed by low memory.(%s)", szTmpFileBuf );
		m_log.Err( gszDbgMsg );
		return 0;
	}

	m_hIndexFile = CreateFile( s, GENERIC_READ|GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL );
	if( INVALID_HANDLE_VALUE == m_hIndexFile )
	{
		sprintf( gszDbgMsg, "open index file failed.(%s)", szTmpFileBuf );
		m_log.Err( gszDbgMsg );
		return 0;
	}

	// ------------------------------------------
	// open data file
	// ------------------------------------------
	GetDataFilePath( m_szSrcFile, szTmpFileBuf, MAX_PATH );

	if( 0 == s.Set( szTmpFileBuf ) )
	{
		return 0;
	}

	m_hDataFile = CreateFile( s, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, NULL );
	if( INVALID_HANDLE_VALUE == m_hDataFile )
	{

		CloseHandle(m_hIndexFile);
		m_hIndexFile = INVALID_HANDLE_VALUE;
		m_log.Err( "open data file failed(1)." );
		return 0;
	}

	char szLine[MAX_MAILADDR_LEN256 + 32] = {0};
	char szTmpMailAddr[MAX_MAILADDR_LEN256 + 32] = {0};

	// ------------------------------------------
	// first line of data file must be start wiht 'Return-Path:'(12octet)
	// ------------------------------------------
	if( NULL == GetFileLine( m_hDataFile, szLine, MAX_MAILADDR_LEN256+10 ) )
	{
		CloseHandle(m_hIndexFile);
		m_hIndexFile = INVALID_HANDLE_VALUE;
		CloseHandle(m_hDataFile);
		m_hDataFile = INVALID_HANDLE_VALUE;

		m_log.Err( "read data file failed(2)." );
		return 0;
	}

	if( 0 != STRNICMP( szLine, "Return-Path:", 12 ) )
	{
		// broken file
		m_log.Err( "found broken index file. delete it." );
		return 0;
	}

	// ------------------------------------------
	// check return-path address
	// ------------------------------------------
	if( 0 == GetMailAddressString( m_szMailFrom, MAX_MAILADDR_LEN256, &szLine[12], g_conf.GetValueI(CONFIG_ME) ) )
	{
		m_log.Err( "found broken index file(return-path broken). delete it." );
		return 0;
	}

	// ------------------------------------------
	// mail from : address check
	// ------------------------------------------
	if( STRLEN( m_szMailFrom ) > MAX_MAILADDR_LEN256 )
	{
		m_log.Err( "found broken index file(return-path broken). delete it." );
		return 0;
	}

	// ------------------------------------------
	// load 'rcpt to:'
	// ------------------------------------------
	while( NULL != GetFileLine( m_hIndexFile, szLine, MAX_MAILADDR_LEN256+10 ) )
	{
		// --------------------------------------------
		// make domain list
		// --------------------------------------------
		if( 0 != GetMailAddressString( szTmpMailAddr, MAX_MAILADDR_LEN256+1, szLine, g_conf.GetValueI(CONFIG_ME) ) )
		{
			m_hsRcptList.Set( szTmpMailAddr, "" );
			char* pDomain = STRRCHR( szTmpMailAddr, '@' );
			pDomain++;
			// set domain list
			m_hsDomainList.Set( pDomain, "" );
		}
	}

	// ------------------------------------------
	// mail from : address check
	// ------------------------------------------
	if( NULL == m_hsRcptList.SeekToFirst() )
	{
		m_log.Dbg( "no recipients. delete it." );
		return 0;
	}

	// ------------------------------------------------
	// timeout check 'CONFIG_QUEUELIFETIME'
	// ------------------------------------------------
	SYSTEMTIME stCurrent = {0};
	FILETIME ftCurrent = {0};
	FILETIME ftFileTime = {0};
	
	GetSystemTime( &stCurrent );
	SystemTimeToFileTime( &stCurrent, &ftCurrent );

	if( 0 != GetFileTime( m_hIndexFile, &ftFileTime, NULL, NULL ) )
	{
		if( 0 != FileTimeCmp(&ftCurrent, &ftFileTime, g_conf.GetUINTValueI(CONFIG_QUEUELIFETIME)) )
		{
			CreatePermanentFatalMsg( m_szMailFrom, "Try, but timeout.", "", &m_hsRcptList );
			DeleteIndexAndDataFile(m_szSrcFile);
			sprintf( gszDbgMsg, "Timeout; msg given up.(%s)", m_szSrcFile );
			m_log.Info(gszDbgMsg);

			return 0;
		}
	}
	return 1;
}

//-----------------------------------------------------------------------------
//	@detail
//		ProcessQueue
//	@param
//		nothing
//	@return
//		0 indicate an error
//-----------------------------------------------------------------------------
int MySmtpTalker::ProcessQueue()
{
	if(( m_hIndexFile == INVALID_HANDLE_VALUE ) || ( m_hDataFile  == INVALID_HANDLE_VALUE ))
		return 0;

	SEEKKEY skDomain = m_hsDomainList.SeekToFirst();

	if( NULL == skDomain || 0 == m_hsRcptList.GetLength() )
	{
		// nothing to do.
		m_log.Dbg( "process queue, but nothing to do." );
		return 0;
	}

	int nRcptLength = m_hsRcptList.GetLength();

	MyHashEx hsRecipientsOfThisDomain;
	SEEKKEY skRcptList;
	
	while( NULL != skDomain )
	{
		MyConsole conServer;

		// ----------------------------------------------------------
		// connect to smtp server
		// ----------------------------------------------------------
		if( 0 == ConnectServer( &conServer, skDomain ) )
		{
			sprintf( gszDbgMsg, "connect failed %s server.", skDomain );
			m_log.Info( gszDbgMsg );
			skDomain = m_hsDomainList.SeekNext( skDomain );
			continue;
		}

		sprintf( gszDbgMsg, "sending to %s server.", skDomain );
		m_log.Info( gszDbgMsg );

		// ----------------------------------------------------------
		// go sequence.
		// ----------------------------------------------------------

		//ishitani
		skRcptList = m_hsRcptList.SeekToFirst();
		hsRecipientsOfThisDomain.Clear();

		while( NULL != skRcptList )
		{
			char* pszAtMark = STRRCHR( skRcptList, '@' );
			
			if( NULL == pszAtMark )
			{
				m_hsRcptList.Del( skRcptList );
				skRcptList = m_hsRcptList.SeekToFirst();
				continue;
			}

			pszAtMark++;

			// domain check
			if( 0 == STRICMP( pszAtMark, skDomain ) )
			{
				hsRecipientsOfThisDomain.Set( skRcptList, "" );
			}
			skRcptList = m_hsRcptList.SeekNext( skRcptList );
		}
		
		if( 0 == Transact( &conServer, &hsRecipientsOfThisDomain ) )
		{
			skDomain = m_hsDomainList.SeekNext( skDomain );
			continue;
		}
		else
		{
		}
		//else
		// ....


		// ----------------------------------------------------------
		// transaction success.
		// ----------------------------------------------------------
		
		// ----------------------------------------------------------
		// delete rcptlist in this domain
		// ----------------------------------------------------------
		SEEKKEY skRcpt = m_hsRcptList.SeekToFirst();
		while( NULL != skRcpt )
		{
			char* pszAtMark = STRRCHR( skRcpt, '@' );
			pszAtMark++;

			// domain check
			if( 0 != STRICMP( pszAtMark, skDomain ) )
			{
				// not this domain
				skRcpt = m_hsRcptList.SeekNext( skRcpt );
				continue;
			}

			// error rcpt check
			if( NULL != m_hsRetryRcptList.SeekToKey( skRcpt ) )
			{
				// if this address is not in rcpient list, this rcpient not be deleted.
				skRcpt = m_hsRcptList.SeekNext( skRcpt );
			}
			else
			{
				// the domain is match and the error is not occured, it provide that mission complete.
				m_hsRcptList.Del( skRcpt );
				skRcpt = m_hsRcptList.SeekToFirst();
			}
		}
		skDomain = m_hsDomainList.SeekNext( skDomain );
	}
	
	// ----------------------------------------------------
	// update rcptlist file
	// ----------------------------------------------------
	if( 0 < m_hsRcptList.GetLength() && nRcptLength != m_hsRcptList.GetLength() )
	{
		char szLine[MAX_MAILADDR_LEN256 + 10 + 1] = {0};
		DWORD dwWritten;
		
		SetFilePointer( m_hIndexFile, 0, NULL, FILE_BEGIN );
		
		SEEKKEY skRcpt = m_hsRcptList.SeekToFirst();
		while( NULL != skRcpt )
		{
			sprintf( szLine, "%s \r\n", skRcpt );
			if( 0 == WriteFile( m_hIndexFile, szLine, STRLEN( szLine ), &dwWritten, NULL ) )
			{
				CloseHandle( m_hIndexFile );
				m_hIndexFile = INVALID_HANDLE_VALUE;
				m_log.Err("critical error. index file update failed.");
				return 1;	// err but going on.
			}

			if( dwWritten != STRLEN( szLine ) )
			{
				CloseHandle( m_hIndexFile );
				m_hIndexFile = INVALID_HANDLE_VALUE;
				m_log.Err("critical error. index file update failed.");
				return 1;	// err but going on.
			}

			skRcpt = m_hsRcptList.SeekNext( skRcpt );
		}

		// truncate file.
		SetEndOfFile( m_hIndexFile );

		return 1;
	}

	// ----------------------------------------------------
	// delete index and data file.
	// ----------------------------------------------------
	if( 0 == m_hsRcptList.GetLength() )
	{
		sprintf( gszDbgMsg, "complete sending mail(%s).", m_szSrcFile );
		m_log.Info( gszDbgMsg );

		// gracefull success.
		DeleteIndexAndDataFile(m_szSrcFile);
	}
	return 1;
}

//-----------------------------------------------------------------------------
//	@detail
//		DeleteIndexAndDataFile
//	@param
//		nothing
//	@return
//		nothing
//-----------------------------------------------------------------------------
void MySmtpTalker::DeleteIndexAndDataFile(const char* pszSrcFile)
{
	char szTmpFileBuf[MAX_PATH+1] = {0};
	MyString s;

	// ------------------------------------------------
	// delete index file
	// ------------------------------------------------
	if( m_hIndexFile != INVALID_HANDLE_VALUE )
	{
		CloseHandle( m_hIndexFile );
		m_hIndexFile = INVALID_HANDLE_VALUE;
	}

	if( 0 != GetIndexFilePath( pszSrcFile, szTmpFileBuf, MAX_PATH ) )
	{
		if( 0 == s.Set( szTmpFileBuf ) )
		{
			m_log.Err( "low memory, delete index failed." );
		}
		else
		{
			if( 0 == DeleteFile( s ) )
			{
				m_log.Err( "'DeleteFile()', delete index failed." );
			}
		}
	}
	else
	{
		m_log.Err( "delete indexfile failed." );
	}
	
	// ------------------------------------------------
	// delete data file
	// ------------------------------------------------
	if( m_hDataFile != INVALID_HANDLE_VALUE )
	{
		CloseHandle( m_hDataFile );
		m_hDataFile = INVALID_HANDLE_VALUE;
	}

	if( 0 != GetDataFilePath( pszSrcFile, szTmpFileBuf, MAX_PATH ) )
	{
		if( 0 == s.Set( szTmpFileBuf ) )
		{
			m_log.Err( "low memory, delete datafile failed." );
		}
		else
		{
			if( 0 == DeleteFile( s ) )
			{
				m_log.Err( "'DeleteFile()', delete datafile failed." );
			}
		}
	}
	else
	{
		m_log.Err( "delete indexfile failed." );
	}

	m_log.Dbg( "DeleteIndexAndDataFile() complete." );
}

//-----------------------------------------------------------------------------
//	@detail
//		Go SMTP Transaction
//	@param
//		MyConsole* pServer
//		const char *pDomain
//	@return
//		0 indicate an error (should retry all)
//		1 indicate transaction complete.
//-----------------------------------------------------------------------------
int MySmtpTalker::Transact(MyConsole* pServer, MyHashEx* phsRecipientsOfThisDomain )
{
	if( NULL == pServer || NULL == phsRecipientsOfThisDomain )
	{
		m_log.Dbg( "Transact() err. may be bug." );
		return 0;	// all retry
	}

	char szCmdLine[MAX_CMDLINE+4] = {0};
	char szServerRes[256] = {0};
	const char* pszMe = g_conf.GetValueI(CONFIG_ME);
	if( NULL == pszMe )
	{
		m_log.Err("Transact():load 'me' value failed.");
		return 0;	// all retry
	}

	m_log.Dbg("Transact() start.");

	if( 0 == m_hsRcptList.GetLength() )
		return 1;	// if happen, it must be bug.

	// ------------------------------------------
	// waiting for greeting message
	// ------------------------------------------
	pServer->SetTimeOut( ATOUI(g_conf.GetValueI(CONFIG_SMTPGREETINGTIMEOUT)) );

	if( SOCKET_ERROR == pServer->GetLine( szServerRes, 255, NULL, 30 ) )
	{
		m_log.Err("Server connection timed out.");
		return 0;	// all retry
	}

	m_log.Dbg(szServerRes);
	if( '2' != szServerRes[0] )
	{
		// Permanent Negative Completion reply(should be error)
		// all error.
		CreatePermanentFatalMsg( m_szMailFrom, "wait greeting message...", szServerRes, phsRecipientsOfThisDomain );
		return 1;
	}

	// ------------------------------------------
	// HELO <domain>
	// ------------------------------------------
	sprintf( szCmdLine, "HELO %s\r\n", pszMe );
	m_log.Dbg(szCmdLine);

	if( SOCKET_ERROR == pServer->SendData( szCmdLine ) )
	{
		m_log.Err("Server connection lost with sending 'HELO'.");
		
		return 0;	// all retry
	}

	if( SOCKET_ERROR == pServer->GetLine( szServerRes, 255, NULL, 30 ) )
	{
		m_log.Err("Server connection timed out.");

		return 0;	// all retry
	}

	m_log.Dbg(szServerRes);
	if( '2' != szServerRes[0] )
	{
		// Permanent Negative Completion reply(should be error)
		// all error.
		CreatePermanentFatalMsg( m_szMailFrom, szCmdLine, szServerRes, phsRecipientsOfThisDomain );
		return 1;
	}

	// ------------------------------------------
	// MAIL FROM: <return-path>
	// ------------------------------------------
	pServer->SetTimeOut( ATOUI(g_conf.GetValueI(CONFIG_SMTPMAILTIMEOUT)) );

	sprintf( szCmdLine, "MAIL FROM:<%s>\r\n", m_szMailFrom );
	m_log.Dbg(szCmdLine);

	if( SOCKET_ERROR == pServer->SendData( szCmdLine ) )
	{
		// Transient Negative Completion reply(should retry)
		m_log.Err("Server connection lost with sending 'MAIL FROM:'.");
		return 0;	// all retry
	}

	if( SOCKET_ERROR == pServer->GetLine( szServerRes, 255, NULL, 30 ) )
	{
		// Transient Negative Completion reply(should retry)
		m_log.Err("Server connection timed out.");
		return 0;	// all retry
	}

	m_log.Dbg(szServerRes);
	
	if( '4' == szServerRes[0] )
	{
		// Transient Negative Completion reply(should retry)
		return 0;	// all retry
	}
	else if( ( '5' == szServerRes[0] ) || ( '2' != szServerRes[0] ) )
	{
		// Permanent Negative Completion reply(should be error)
		// all error.
		CreatePermanentFatalMsg( m_szMailFrom, szCmdLine, szServerRes, phsRecipientsOfThisDomain );
		return 1;
	}



	// ------------------------------------------
	// RCPT TO: <rcpt address>
	// ------------------------------------------
	pServer->SetTimeOut( ATOUI(g_conf.GetValueI(CONFIG_SMTPRCPTTIMEOUT)) );

	SEEKKEY skRcpt = phsRecipientsOfThisDomain->SeekToFirst();
	MyHashEx hsErrRecipient;
	int nRcptCount = 0;

	while( NULL != skRcpt )
	{
		// target address here.
		sprintf( szCmdLine, "RCPT TO:<%s>\r\n", skRcpt );
		m_log.Dbg(szCmdLine);

		if( SOCKET_ERROR == pServer->SendData( szCmdLine ) )
		{
			m_log.Err("Server connection lost when say RCPT TO:.");
			return 0;	// all retry
		}

		if( SOCKET_ERROR == pServer->GetLine( szServerRes, 255, NULL, 30 ) )
		{
			m_log.Err("Server connection timed out.");
			return 0;	// all retry
		}

		m_log.Dbg(szServerRes);
		
		if( '4' == szServerRes[0] )
		{
			m_hsRetryRcptList.Set( skRcpt, "" );
		}
		else if( '2' == szServerRes[0] )
		{
			nRcptCount++;
		}
		else
		{
			hsErrRecipient.Set( skRcpt, "" );
		}

		skRcpt = m_hsRcptList.SeekNext( skRcpt );
	}

	// rcpt to: <foo> is end

	if( hsErrRecipient.GetLength() > 0 )
	{
		CreatePermanentFatalMsg( m_szMailFrom, szCmdLine, szServerRes, &hsErrRecipient );

		// delete error recipients
		SEEKKEY skErrRcpt = hsErrRecipient.SeekToFirst();
		while( NULL != skErrRcpt )
		{
			phsRecipientsOfThisDomain->Del( skErrRcpt );
			m_hsRcptList.Del( skErrRcpt );

			skErrRcpt = hsErrRecipient.SeekNext( skErrRcpt );
		}
		
		// oyaku gomen
		hsErrRecipient.Clear();
	}

	if( 0 == nRcptCount )
	{
		// all recipient are insert to ErrRecipient list or RetryRecipient list
		return 1;
	}


	// ------------------------------------------
	// DATA
	// ------------------------------------------
	pServer->SetTimeOut( ATOUI(g_conf.GetValueI(CONFIG_SMTPDATASTARTTIMEOUT)) );

	sprintf( szCmdLine, "DATA\r\n" );
	m_log.Dbg(szCmdLine);

	if( SOCKET_ERROR == pServer->SendData( szCmdLine ) )
	{
		m_log.Err("Server connection lost with sending 'DATA'.");
		return 0;	// all retry
	}

	if( SOCKET_ERROR == pServer->GetLine( szServerRes, 255, NULL, 30 ) )
	{
		m_log.Err("Server connection timed out.");
		return 0;	// all retry
	}

	m_log.Dbg(szServerRes);
	if( '4' == szServerRes[0] )
	{
		return 0;	// all retry
	}
	else if( /*( '5' == szServerRes[0] ) ||*/ ( '3' != szServerRes[0] ) )
	{
		// create error mail to all recipients in this domain.
		CreatePermanentFatalMsg( m_szMailFrom, szCmdLine, szServerRes, phsRecipientsOfThisDomain );
		return 1;
	}

	// ------------------------------------------
	// DATA channel.
	// ------------------------------------------
	pServer->SetTimeOut( ATOUI(g_conf.GetValueI(CONFIG_SMTPDATABLOCKTIMEOUT)) );

	// seek to first.
	SetFilePointer( m_hDataFile, 0, NULL, FILE_BEGIN );
	
	// first line is return-path, ignored.
	GetFileLine( m_hDataFile, szCmdLine, MAX_CMDLINE );

	while( NULL != GetFileLine( m_hDataFile, szCmdLine, MAX_CMDLINE ) )
	{
		m_log.Dbg(szCmdLine);
		STRCAT( szCmdLine, "\r\n" );

		if( '.' == szCmdLine[0] )
		{
			char* pTail = szCmdLine;
			char* pHead = NULL;
			while( '\0' != *pTail++ ) ;
			pHead = (pTail-1);
			while( pHead != szCmdLine ) *pTail-- = *pHead--;
			*pTail = '.';
		}

		if( SOCKET_ERROR == pServer->SendData( szCmdLine ) )
		{
			m_log.Err("Server connection lost in data channel.");
			return 0;
		}
	}

	sprintf( szCmdLine, ".\r\n" );
	m_log.Dbg(szCmdLine);
	if( SOCKET_ERROR == pServer->SendData( szCmdLine ) )
	{
		m_log.Err("Server connection lost in data channel.");
		return 0;
	}

	pServer->SetTimeOut( ATOUI(g_conf.GetValueI(CONFIG_SMTPDATACOMPLETETIMEOUT)) );

	if( SOCKET_ERROR == pServer->GetLine( szServerRes, 255, NULL, 30 ) )
	{
		m_log.Err("Server connection timed out.");
		return 0;
	}

	m_log.Dbg(szServerRes);
	if( '4' == szServerRes[0] )
	{
		return 0;	// all retry
	}
	else if( ( '5' == szServerRes[0] ) || ( '2' != szServerRes[0] ) )
	{
		// create error mail to all recipients in this domain.
		CreatePermanentFatalMsg( m_szMailFrom, szCmdLine, szServerRes, phsRecipientsOfThisDomain );
		return 1;
	}

	// ------------------------------------------
	// QUIT
	// ------------------------------------------
	pServer->SetTimeOut( ATOUI(g_conf.GetValueI(CONFIG_SMTPGREETINGTIMEOUT)) );

	sprintf( szCmdLine, "QUIT\r\n" );
	m_log.Dbg(szCmdLine);

	if( SOCKET_ERROR == pServer->SendData( szCmdLine ) )
	{
		m_log.Err("Server connection lost with sending 'QUIT'.");
		return 0;
	}

	if( SOCKET_ERROR == pServer->GetLine( szServerRes, 255, NULL, 30 ) )
	{
		m_log.Err("Server connection timed out.");
		return 0;
	}

	m_log.Dbg(szServerRes);

	return 1;
}

//-----------------------------------------------------------------------------
//	@detail
//		CreatePermanentFatalMsg
//	@param
//		const char* pszTo				// err message to
//		const char* pszSendMsg			// string that send message 
//		const char* pszRecvMsg			// string that recv message from SMTP server
//		const MyHashEx* hsErrRecipient	// err recipient list
//	@return
//		nothing
//-----------------------------------------------------------------------------
void MySmtpTalker::CreatePermanentFatalMsg( 
	const char* pszTo, const char* pszSendMsg, const char* pszRecvMsg, MyHashEx* phsErrRecipient )
{
	m_log.Dbg("CreatePermanentFatalMsg()");

	if( NULL == pszTo || NULL == phsErrRecipient )
	{
		m_log.Dbg("Create err msg failed.");
		return;
	}

	// error message has no return-path
	if( 0 == STRCMP( pszTo, "" ) )
	{
		m_log.Err( "double bounce." );
		return;	// do nothing
	}

	char szLine[MAX_CMDLINE+4];
	MyHashEx hsRcptLocal;
	MyHashFile hsRcptRemote;
	Usr usr;

	switch( usr.IsMyUser( pszTo ) )
	{
	case USR_OTHER_DOMAIN:	//0
		hsRcptRemote.Set( pszTo, "" );
		break;

	case USR_LOCAL_DOMAIN:	//1
		hsRcptLocal.Set( pszTo, "" );
		break;

	case USR_ERR_BADPARAM:	//-3
	case USR_ERR_MEMORY:	//-2
	case USR_ERR_NAME:		//-1
	default:
		return;
	}
	sprintf( gszDbgMsg, "CreatePermanentFatalMsg(%s)", pszTo );
	m_log.Info(gszDbgMsg);
	
	MyTmpFileMgr tmpFile;
	if( 0 == tmpFile.Create( g_conf.GetValueI(CONFIG_TMPDIR) ) )
	{
		return;
	}

	MyTime t;
	sprintf( szLine, 
		"Return-Path: <>\r\n"			// 17 octet
		"Date: %s\r\n"					// 36 octet
		"From: MAILER-DAEMON@%s\r\n"	// 22+64 octet
		"To: %s\r\n"					// 6+256 octet
		"Subject: Failure notice\r\n"	// 24 octet
		"\r\n"							// 2 octet
		
		, t.GetLocalFormatRfc1123()		// "Date: %s\r\n"
		, g_conf.GetValueI(CONFIG_ME)	// "From: MAILER-DAEMON@%s\r\n"
		, pszTo							// "To: %s\r\n"
		);

	tmpFile.Append( szLine );

	tmpFile.Append( "The following addresses had permanent fatal errors.\r\n" );
	
	SEEKKEY skErrRcpt = phsErrRecipient->SeekToFirst();
	while( NULL != skErrRcpt )
	{
		tmpFile.AppendLine(skErrRcpt);
		skErrRcpt = phsErrRecipient->SeekNext( skErrRcpt );
	}

	if( NULL != pszSendMsg )
	{
		STRCPY( szLine, "> " );
		STRNCAT( szLine, pszSendMsg, 128 );
		STRCAT( szLine, "\r\n" );
		tmpFile.Append( szLine );
	}

	if( NULL != pszRecvMsg )
	{
		STRCPY( szLine, "< " );
		STRNCAT( szLine, pszRecvMsg, 128 );
		STRCAT( szLine, "\r\n\r\n" );
		tmpFile.Append( szLine );
	}

	// mail file
	// seek to first.
	tmpFile.AppendLine( "--- Below this line is a copy of the message.\r\n" );

	SetFilePointer( m_hDataFile, 0, NULL, FILE_BEGIN );
	
	while( NULL != GetFileLine( m_hDataFile, szLine, MAX_CMDLINE ) )
	{
		if( '\0' == szLine[0] )
			// only header append, because of low memory.(this program is for PDA)
			break;

		STRCAT( szLine, "\r\n" );
		tmpFile.Append( szLine );
	}

	// error mail create complete.
	// deliver to rcpient
	char szTmpFileName[MAX_PATH] = {0};
	
	tmpFile.GetFileName( szTmpFileName, MAX_PATH );

	if( 0 == usr.DeliveryFile( szTmpFileName, &hsRcptLocal, &hsRcptRemote, &m_log ) )
	{
		// error but do nothing. double bounce.
	}

	tmpFile.Delete();
}

//-----------------------------------------------------------------------------
//	@detail
//		connect mx server
//	@param
//		nothing
//	@return
//		0 indicate an error
//-----------------------------------------------------------------------------
int MySmtpTalker::ConnectServer(MyConsole* pServer, const char *pDomain)
{
	if( NULL == pServer || NULL == pDomain )
		return 0;

	const char* pszMailServer = NULL;
	const char* pszDNSServer = g_conf.GetValueI(CONFIG_1STDNSSERVER);
	unsigned int dwTimeOut = g_conf.GetUINTValueI(CONFIG_DNSQUERYTIMEOUT);

	if( NULL == pszDNSServer )
	{
		m_log.Err("load '1st dnsserver' value failed.");
		return 0;
	}

	MXRecord mx;
	// ----------------------------------------------------
	// 1st try
	// ----------------------------------------------------
	int nDnsErr = mx.QueryDNS( pszDNSServer, pDomain, dwTimeOut );
	if( 0 > nDnsErr )
	{
		goto DNSERROR;
	}
	
	pszMailServer = mx.GetFirstMXRecord();

	while( pszMailServer )
	{
		if( SOCKET_ERROR == pServer->Connect( pszMailServer, "25" ) )
		{
			pszMailServer = mx.GetNetxtMXRecord( pszMailServer );
			continue;
		}
		else
		{
			return 1;
		}
	}

	// ----------------------------------------------------
	// 2nd try
	// ----------------------------------------------------
	pszDNSServer = g_conf.GetValueI(CONFIG_2NDDNSSERVER);

	if( NULL == pszDNSServer )
	{
		return 0;
	}

	nDnsErr = mx.QueryDNS( pszDNSServer, pDomain, dwTimeOut );
	if( 0 > nDnsErr )
	{
		goto DNSERROR;
	}
	
	pszMailServer = mx.GetFirstMXRecord();

	while( pszMailServer )
	{
		if( SOCKET_ERROR == pServer->Connect( pszMailServer, "25" ) )
		{
			pszMailServer = mx.GetNetxtMXRecord( pszMailServer );
			continue;
		}
		else
		{
			return 1;
		}
	}
	return 0;

DNSERROR:
	char *pszDnsErrMsg = NULL;
	switch( nDnsErr )
	{
	case MXRECORD_ERR_NOMEMORY:
		pszDnsErrMsg = "no memory";
		break;
	case MXRECORD_ERR_BADDNSHOSTNAME:
		pszDnsErrMsg = "bad host name";
		break;
	case MXRECORD_ERR_DNSSERVERERROR:
		pszDnsErrMsg = "dns server error";
		break;
	case MXRECORD_ERR_CONNECTFAILED:
		pszDnsErrMsg = "connect failed";
		break;
	case MXRECORD_ERR_SOCKETERROR:
		pszDnsErrMsg = "socket error";
		break;
	case MXRECORD_ERR_DNSTIMEOUT:
		pszDnsErrMsg = "dns timeout";
		break;
	case MXRECORD_ERR_DNSBADREPLY:
		pszDnsErrMsg = "dns bad reply";
		break;
	default:
		pszDnsErrMsg = "unknown";
		break;
	}
	sprintf( gszDbgMsg, "DNS error '%s'", pszDnsErrMsg );
	m_log.Err( gszDbgMsg );
	return 0;
}

// helper
int MySmtpTalker::GetIndexFilePath(const char* pszTargetName, char *pszFilePath,  unsigned int dwSize)
{
	if( NULL == pszTargetName || NULL == pszFilePath )
		return 0;

	const char* pszQueueDir = g_conf.GetValueI(CONFIG_QUEUEDIR);
	if( NULL == pszQueueDir )
	{
		m_log.Err("load 'queue' value failed.");
		return 0;
	}

	if( dwSize >= MAX_CONFIG_DIR_PATH )
	{
		STRNCPY( pszFilePath, pszQueueDir, MAX_CONFIG_DIR_PATH );
		if( dwSize > STRLEN(pszFilePath) )
		{
			STRNCAT( pszFilePath, "index\\", dwSize - STRLEN(pszFilePath) );
			if( dwSize > STRLEN(pszFilePath) )
			{
				STRNCAT( pszFilePath, pszTargetName, dwSize - STRLEN(pszFilePath) );
				return 1;
			}
		}
	}
	return 0;


}

// helper
int MySmtpTalker::GetDataFilePath(const char* pszTargetName, char *pszFilePath, unsigned int dwSize)
{
	if( NULL == pszTargetName || NULL == pszFilePath )
		return 0;

	const char* pszQueueDir = g_conf.GetValueI(CONFIG_QUEUEDIR);
	if( NULL == pszQueueDir )
	{
		m_log.Err("load 'queue' value failed.");
		return 0;
	}

	if( dwSize >= MAX_CONFIG_DIR_PATH )
	{
		STRNCPY( pszFilePath, pszQueueDir, MAX_CONFIG_DIR_PATH );
		if( dwSize > STRLEN(pszFilePath) )
		{
			if( dwSize > STRLEN(pszFilePath) )
			{
				STRNCAT( pszFilePath, pszTargetName, dwSize - STRLEN(pszFilePath) );
				return 1;
			}
		}
	}
	return 0;
}

