/* =========================================================================
	This library is part of IshiSoft application.
	------------------------------------------------------------------------
	author: Yuichiro Ishitani
			http://www001.upp.so-net.ne.jp/ishi/index.html
  ========================================================================== */
#include <stdio.h>
#include "stdafx.h"
#include "MainThread.h"
#include "MySock.h"
#include "MyConf.h"
#include "MyPop3Sock.h"
#include "MySmtpSock.h"
#include "str.h"

#if !defined( _WIN32_WCE )
#endif

extern MyConf		g_conf;
extern MyLog		g_log;
static void thAccept(void *p);

// ============================================================================
// Listen thread
// ============================================================================
void thLiten(void *p)
{
	MySock listen;
	int ii;
	LISTENTHREAD *lTh = (LISTENTHREAD*)p;

	// ------------------------------------------------------------------------
	// Create Accept threads.
	// ------------------------------------------------------------------------
	int nServers = lTh->nServers;

	if( nServers < 1 )
	{
		g_log.Err( "config.txt: no servers?" );
	}

	ACCEPTTHREAD* pAcceptThread = new ACCEPTTHREAD[nServers];

	if( NULL == pAcceptThread )
	{
		g_log.Err("Memory error.");
		ExitThread( 0 );
		return;
	}

	ACCEPTTHREAD* pAT = pAcceptThread;
	for( ii = 0; ii < nServers; ii++, pAT++ )
	{
		pAT->bStop = FALSE;
		pAT->bSuspended = TRUE;
		pAT->sock = INVALID_SOCKET;
		//if( lTh->nPort )
		if( lTh->servertype == SERVERTYPE_POP3 )
		{
			pAT->pAcceptConsole = new MyPop3Sock(pAT);
		}
		else if( lTh->servertype == SERVERTYPE_SMTP )
		{
			pAT->pAcceptConsole = new MySmtpSock(pAT);
		}
		pAT->hThread = CreateThread( NULL, 0, (LPTHREAD_START_ROUTINE)thAccept, (LPVOID)pAT, CREATE_SUSPENDED, &pAT->dwThreadID );
		if( NULL == pAT->pAcceptConsole || NULL == pAT->hThread )
		{
			goto TERMINATE_SERVERS;
		}

		#ifdef _DEBUG
		char work[256];
		sprintf( work, "Child thread %X created.", pAT->hThread );
		g_log.Dbg( work );
		#endif
	}


	// ------------------------------------------------------------------------
	// Listen.
	// ------------------------------------------------------------------------

	if( SOCKET_ERROR == listen.Listen( lTh->nPort ) )
	{
		g_log.Err( "Listen failed." );
		goto TERMINATE_SERVERS;
	}

	if( lTh->servertype == SERVERTYPE_POP3 )
		g_log.Info( "POP3 Listen started." );
	else if( lTh->servertype == SERVERTYPE_SMTP )
		g_log.Info( "SMTP Listen started." );

	while( lTh->bStop == FALSE )
	{
		SOCKET client_socket;
		sockaddr_in saRemoteHost;

		listen.SetTimeOut( 1 );

		if( listen.IsReadable() > 0 )
		{
			g_log.Dbg( "AcceptThread:Readable." );
			// Find free thread.
			for( ii = 0, pAT = pAcceptThread; ii < nServers; ii++, pAT++ )
			{
				Sleep(0);
				if( pAT->bSuspended == TRUE )	// Free thread is found.
				{
					client_socket = listen.Accept( &saRemoteHost );
					
					if( client_socket == INVALID_SOCKET )
					{
						g_log.Err( "Accept failed." );
						ii = nServers;
						break;
					}

					// Resume child thread.
					pAT->sock = client_socket;
					memcpy( &(pAT->saRemoteHost), &saRemoteHost, sizeof(sockaddr_in) );
					ResumeThread(pAT->hThread);
					break;
				}
			}

			// There is no free thread.
			if( ii >= nServers )
			{
				client_socket = listen.Accept( &saRemoteHost );
				
				if( client_socket != INVALID_SOCKET )
				{
					MySock sErrorSock(client_socket);
					sErrorSock.Close();
				}
			}
		}
	}

TERMINATE_SERVERS:
	// ------------------------------------------------------------------------
	// Terminate servers
	// ------------------------------------------------------------------------
	g_log.Dbg( "ListenThread:TerminateServers..." );

	for( ii = 0, pAT = pAcceptThread; ii < nServers; ii++, pAT++ )
	{
		pAT->bStop = TRUE;
		Sleep(1);
		ResumeThread(pAT->hThread);
	}

	for( ii = 0, pAT = pAcceptThread; ii < nServers; ii++, pAT++ )
	{
		if( NULL != pAT->hThread )
		{
			if( WAIT_TIMEOUT == WaitForSingleObject( pAT->hThread, 10000 ) )
			{
				g_log.Dbg( "Accept thread was force terminated." );
				TerminateThread( pAT->hThread, 0 );
			}
			CloseHandle( pAT->hThread );
			pAT->hThread = NULL;
		}
		if( pAT->pAcceptConsole ) delete pAT->pAcceptConsole;

		g_log.Dbg( "Accept thread terminated." );
	}

	delete pAcceptThread;
	g_log.Dbg( "ListenThread:Exit" );
	ExitThread( 1 );
}

// ============================================================================
// accept thread
// ============================================================================
void thAccept(void *p)
{
	if( NULL == p )
	{
		g_log.Err( "console thread: fatal error caused by invalid pointer." );
		ExitThread( 0 );
	}

	ACCEPTTHREAD* pAT = (ACCEPTTHREAD*)p;

	g_log.Dbg( "AcceptThread:START" );
	//MySmtpSock smtp(pAT);
	MyConsole* con = (MyConsole*)pAT->pAcceptConsole;
	
	if( NULL == con )
	{
		g_log.Err( "console thread: fatal error caused by no memory." );
		ExitThread( 0 );
	}

	while( pAT->bStop == FALSE )
	{
		con->SetSocket(pAT->sock);

		g_log.Dbg( "AcceptThread:Dialogue in." );
		con->Dialogue();
		g_log.Dbg( "AcceptThread:Dialogue out" );

		g_log.Dbg( "AcceptThread:SUSPEND..." );
		SetThreadPriority( pAT->hThread, THREAD_PRIORITY_ABOVE_NORMAL );
		pAT->bSuspended = TRUE;
		SuspendThread( pAT->hThread );

		// ... wake up by parent thread ...
		
		g_log.Dbg( "AcceptThread:RESUME..." );
		pAT->bSuspended = FALSE;
		SetThreadPriority( pAT->hThread, THREAD_PRIORITY_NORMAL );
	}

	g_log.Dbg( "AcceptThread:END" );

	ExitThread( 1 );
}

