/* =========================================================================
	This library is part of IshiSoft application.
	------------------------------------------------------------------------
	author: Yuichiro Ishitani
			http://www001.upp.so-net.ne.jp/ishi/index.html
  ========================================================================== */
#include "stdafx.h"
#include "MyHashFile.h"
#include "MyAuth.h"
#include "MyConf.h"
#include "str.h"
#include "md5.h"
#include "commonhelper.h"


extern MyConf	g_conf;

static void hmac_md5( unsigned char* text, int text_len, unsigned char*  key, int key_len, char* digest );

MyAuth::MyAuth()
{

}

MyAuth::~MyAuth()
{

}

//-----------------------------------------------------------------------------
//	@detail
//		Check passwd
//	@param
//		const char *pszUserName
//		const char *pszPasswd
//		const char* pszAPOPKey
//	@return
//		0 error
//		1 OK
//-----------------------------------------------------------------------------
int MyAuth::AuthAPOP(const char* pszUserName, const char* pClientDigest, const char* pszAPOPKey)
{
	if( NULL == pszUserName || NULL == pClientDigest || NULL == pszAPOPKey )
		return 0;

	MD5Context md5;
	unsigned char cDigest[16+2] = {0};

	// ---
	MyHashFile cf;

	if( 0 == cf.LoadFile( g_conf.GetValueI(CONFIG_PASSWDFILE) ) )
		return 0;

	const char* pszRegisterdPasswd = cf.GetValueI( pszUserName );

	if( NULL == pszRegisterdPasswd || NULL == pClientDigest )
		return 0;

	MD5Init(&md5);
	MD5Update(&md5, (const unsigned char*)pszAPOPKey, STRLEN( (const char*)pszAPOPKey ) );
	MD5Update(&md5, (const unsigned char*)pszRegisterdPasswd, STRLEN( (const char*)pszRegisterdPasswd ) );
	MD5Final( cDigest, &md5 );

	int ii;
	unsigned char cByte;
	for( ii = 0; ii < 16; ii++ )
	{
		cByte = 0;
		if( *pClientDigest > 'z' )		{return 0;}
		if( *pClientDigest >= 'a' )		{cByte = (*pClientDigest - 'a'+10) * 0x10;}
		else if( *pClientDigest >= 'A' ){cByte = (*pClientDigest - 'A'+10) * 0x10;}
		else if( *pClientDigest >= '0' ){cByte = (*pClientDigest - '0') * 0x10;}
		pClientDigest++;

		if( *pClientDigest > 'z' )		{return 0;}
		if( *pClientDigest >= 'a' )		{cByte += (*pClientDigest - 'a'+10);}
		else if( *pClientDigest >= 'A' ){cByte += (*pClientDigest - 'A'+10);}
		else if( *pClientDigest >= '0' ){cByte += (*pClientDigest - '0');}
		else							{return 0;}
		pClientDigest++;

		if( cDigest[ii] != cByte )
			return 0;
	}

	return 1;
}

//-----------------------------------------------------------------------------
//	@detail
//		Check passwd
//	@param
//		const char *pszUserName
//		const char *pszPasswd
//	@return
//		0 error
//		1 OK
//-----------------------------------------------------------------------------
int MyAuth::AuthPLAIN(const char *pszUserName, const char *pszPasswd)
{
	MyHashFile cf;

	if( 0 == cf.LoadFile( g_conf.GetValueI(CONFIG_PASSWDFILE) ) )
		return 0;

	const char* pszRegisterdPasswd = cf.GetValueI( pszUserName );

	if( NULL == pszRegisterdPasswd || NULL == pszPasswd )
		return 0;

	if( 0 != STRCMP( pszRegisterdPasswd, pszPasswd ) )
	{
		return 0;
	}

	return 1;
}

//-----------------------------------------------------------------------------
//	@detail
//		Check passwd
//	@param
//		const char *pszPhrase
//		char *pszKey
//	@return
//		0 error
//		1 OK
//-----------------------------------------------------------------------------
int MyAuth::AuthLOGIN(char* pszPhrase, char* pszKey)
{
	return 0;
}

//-----------------------------------------------------------------------------
//	@detail
//		Check passwd
//	@param
//		char *pszPhrase
//		char *pszKey
//		char *pszOutUserName
//		int nOutUserNameLen		// buffer length
//	@return
//		0 error
//		1 OK
//-----------------------------------------------------------------------------
int MyAuth::AuthCRAMMD5(char* pszPhrase, char* pszKey, char* pszOutUserName/*=NULL*/, int nOutUserNameLen/*=MAX_LOCALUSERNAME_LEN64*/)
{
	if( NULL == pszPhrase || NULL == pszKey )
		return 0;

	MyHashFile cf;
	if( 0 == cf.LoadFile( g_conf.GetValueI(CONFIG_PASSWDFILE) ) )
		return 0;

	Base64Decode( pszPhrase );
	Base64Decode( pszKey );
	
	unsigned int nUserNameLen = 0;

	SEEKKEY sk = cf.SeekToFirst();
	while( NULL != sk )
	{
		nUserNameLen = STRLEN(sk);

		// phrase must have length that username + 16(digest)
		if( (nUserNameLen+16) > STRLEN(pszPhrase) )
			return 0;

		if( 0 == STRNICMP(sk, pszPhrase, nUserNameLen) )
			break;
		sk = cf.SeekNext(sk);
	}
	
	if( NULL == sk )
		return 0;

	if( NULL != pszOutUserName && nOutUserNameLen > 0 )
	{
		STRNCPY( pszOutUserName, sk, nOutUserNameLen );
	}

	const char* pszPasswd = cf.GetValueBySeekKey( sk );
	unsigned char cDigest[16+2] = {0};
	char szDigestStr[32+2] = {0};

	hmac_md5( (unsigned char*)pszKey, STRLEN( pszKey ), (unsigned char*)pszPasswd, STRLEN( pszPasswd ), (char*)cDigest );

	sprintf( szDigestStr, "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x"
		,cDigest[0],cDigest[1],cDigest[2],cDigest[3],cDigest[4],cDigest[5],cDigest[6],cDigest[7]
		,cDigest[8],cDigest[9],cDigest[10],cDigest[11],cDigest[12],cDigest[13],cDigest[14],cDigest[15]);

	if( 0 != STRICMP( &pszPhrase[nUserNameLen+1], szDigestStr ) )
		return 0;

	return 1;
}

/*
RFC2104
HMAC: Keyed-Hashing for Message Authentication
text,                pointer to data stream
text_len,            length of data stream
key,                 pointer to authentication key
key_len,             length of authentication key
digest               caller digest to be filled in
*/
static void hmac_md5( unsigned char* text, int text_len, unsigned char*  key, int key_len, char* digest )
{
	MD5_CTX context;
	unsigned char tk[16];
	int i;
	/* if key is longer than 64 bytes reset it to key=MD5(key) */
	if (key_len > 64)
	{
		MD5_CTX      tctx;

		MD5Init(&tctx);
		MD5Update(&tctx, key, key_len);
		MD5Final(tk, &tctx);

		key = tk;
		key_len = 16;
	}

	/*
	 * the HMAC_MD5 transform looks like:
	 *
	 * MD5(K XOR opad, MD5(K XOR ipad, text))
	 *
	 * where K is an n byte key
	 * ipad is the byte 0x36 repeated 64 times

	 * opad is the byte 0x5c repeated 64 times
	 * and text is the data being protected
	 */

	/* start out by storing key in pads */
	unsigned char k_ipad[65] = {0};
	unsigned char k_opad[65] = {0};
	strcpy( (char*)k_ipad, (const char*)key );
	strcpy( (char*)k_opad, (const char*)key );
	//bcopy( key, k_ipad, key_len);
	//bcopy( key, k_opad, key_len);


	/* XOR key with ipad and opad values */
	for (i=0; i<64; i++) {
		k_ipad[i] ^= 0x36;
		k_opad[i] ^= 0x5c;
	}
	/*
	 * perform inner MD5
	 */
	MD5Init(&context);							/* init context for 1st pass */
	MD5Update(&context, k_ipad, 64);			/* start with inner pad */
	MD5Update(&context, text, text_len);		/* then text of datagram */
	MD5Final((unsigned char*)digest, &context);	/* finish up 1st pass */
	/*
	 * perform outer MD5
	 */
	MD5Init(&context);                   /* init context for 2nd
										  * pass */
	MD5Update(&context, k_opad, 64);     /* start with outer pad */
	MD5Update(&context, (unsigned char*)digest, 16);     /* then results of 1st
										  * hash */
	MD5Final((unsigned char*)digest, &context);          /* finish up 2nd pass */
}
