/* =========================================================================
	This library is part of IshiSoft application.
	------------------------------------------------------------------------
	author: Yuichiro Ishitani
			http://www001.upp.so-net.ne.jp/ishi/index.html
  ========================================================================== */
#include "stdafx.h"
#include "MyPop3List.h"
#include "str.h"
#include "MyString.h"

MyPop3List::MyPop3List()
{
	Clear();
}

void MyPop3List::Clear()
{
	MyHashEx::Clear();
	m_hsDeleList.Clear();
	m_dwOctet = 0;
	m_dwCount = 0;
}

MyPop3List::~MyPop3List()
{
}

//-----------------------------------------------------------------------------
//	@detail
//		Create list
//	@param
//		const char* pszDir
//	@return
//		success	not 0
//		fail	0
//	@comment
//		
//-----------------------------------------------------------------------------
int MyPop3List::CreateList(const char *pszDir)
{
	HANDLE hff = INVALID_HANDLE_VALUE;
	WIN32_FIND_DATA wfd = {0};
	char szHomeDir[MAX_PATH+1] = {0};

	Clear();

	if( NULL == pszDir )
		return 0;

	// store original directory
	STRNCPY( m_szHomeDir, pszDir, MAX_PATH );
	
	STRNCPY( szHomeDir, pszDir, MAX_PATH );
	STRCAT( szHomeDir, "\\*.txt" );
	
	MyString sDir;

	if( 0 == sDir.Set(szHomeDir) )
		return 0;

	hff = FindFirstFile( sDir, &wfd );

	if( INVALID_HANDLE_VALUE == hff )
	{
		// nofile
		return 1;
	}

	do
	{
		if( FILE_ATTRIBUTE_DIRECTORY & wfd.dwFileAttributes )
			continue;

		MyString sFileName;
		if( 0 == sFileName.Set(wfd.cFileName) )
		{
			Clear();
			FindClose( hff );
			return 0;
		}
		char szKey[32] = {0};
		char szVal[MAX_PATH + 32 + 1] = {0};

		sprintf( szKey, "%u", m_dwCount+1 );
		sprintf( szVal, "%u %s", wfd.nFileSizeLow, (const char*)sFileName ); 

		if( 0 == Set( szKey, szVal ) )
		{
			Clear();
			FindClose( hff );
			return 0;
		}
		m_dwOctet += wfd.nFileSizeLow;
		m_dwCount++;

	}while( FindNextFile( hff, &wfd ) );
	
	if( ERROR_NO_MORE_FILES != GetLastError() )
	{
		Clear();
		FindClose( hff );
		return 0;
	}
	FindClose( hff );
	return 1;
}

//-----------------------------------------------------------------------------
//	@detail
//		Stat
//	@param
//		unsigned long *dwCount	:	place holder for count
//		unsigned long *dwOctet	:	place holder for octet
//	@return
//		success	not 0
//		fail	0
//	@comment
//		
//-----------------------------------------------------------------------------
int MyPop3List::Stat(unsigned long *dwCount, unsigned long *dwOctet)
{
	*dwCount = m_dwCount;
	*dwOctet = m_dwOctet;
	return 0;
}

//-----------------------------------------------------------------------------
//	@detail
//		Dele
//	@param
//		const char* pszKey	:	key string for delete.
//	@return
//		success	not 0
//		fail	0
//	@comment
//		
//-----------------------------------------------------------------------------
int MyPop3List::Dele(const char* pszKey)
{
	char* pKey;
	const char* pVal;

	pVal = GetValueI( pszKey );

	if( NULL == pVal )
		return 0;

	m_dwOctet -= ATOUI(pVal);
	m_dwCount--;

	// delete mark
	pKey = (char*)SeekToKey( pszKey );
	*pKey = DELE_MARK;

	return 1;
}

//-----------------------------------------------------------------------------
//	@detail
//		Update
//	@param
//		none
//	@return
//		success	not 0
//		fail	0
//	@comment
//		
//-----------------------------------------------------------------------------
int MyPop3List::Update()
{
	const char* pVal;
	SEEKKEY skKey = SeekToFirst();

	while( NULL != skKey )
	{
		if( DELE_MARK == *skKey )
		{
			pVal = GetValueBySeekKey( skKey );
			
			if( NULL != pVal )
			{
				for( ; *pVal != SEP_MARK && *pVal != '\0'; pVal++);

				if( '\0' != *pVal )
				{
					pVal++;

					char szDelFile[MAX_PATH+1] = {0};
					STRCPY( szDelFile, m_szHomeDir );
					STRCAT( szDelFile, "\\" );
					STRCAT( szDelFile, pVal );
					MyString s;

					if( 0 != s.Set(szDelFile) )
					{
						DeleteFile( s );
					}
				}
			}
		}
		skKey = SeekNext( skKey );
	}

	return 1;
}

//-----------------------------------------------------------------------------
//	@detail
//		Rset
//	@param
//		none
//	@return
//		success	not 0
//		fail	0
//	@comment
//		
//-----------------------------------------------------------------------------
int MyPop3List::Rset()
{
	if( '\0' == m_szHomeDir[0] )
		return 0;

	return CreateList( m_szHomeDir );
}

//-----------------------------------------------------------------------------
//	@detail
//		GetUID
//	@param
//		
//	@return
//		success	not 0
//		fail	0
//	@comment
//		
//-----------------------------------------------------------------------------
const char* MyPop3List::GetUID(SEEKKEY pszKey)
{
	const char* pVal;
	pVal = GetValueBySeekKey( pszKey );
	
	if( NULL == pVal )
		return 0;

	for( ; *pVal != SEP_MARK && *pVal != '\0'; pVal++ );

	if( '\0' == *pVal )	// unknown error! format mismatch. (must be bug)
		return NULL;
	
	pVal++;

	return pVal;
}

//-----------------------------------------------------------------------------
//	@detail
//		GetOctec
//	@param
//		SEEKKEY pszKey
//	@return
//		success	not 0
//		fail	0
//	@comment
//		
//-----------------------------------------------------------------------------
unsigned long MyPop3List::GetOctet(SEEKKEY pszKey)
{
	const char* pVal;

	pVal = GetValueBySeekKey( pszKey );

	if( NULL == pVal )
		return 0;

	return ATOUI(pVal);
}

