/* =========================================================================
	This library is part of IshiSoft application.
	------------------------------------------------------------------------
	author: Yuichiro Ishitani
			http://www001.upp.so-net.ne.jp/ishi/index.html
  ========================================================================== */
#include "stdafx.h"
#include "MyPop3Sock.h"
#include "MyConf.h"
#include "usr.h"
#include "str.h"
#include "MyString.h"
#include "staticvariable.h"
#include "commonhelper.h"
#include "MyAuthIPList.h"
#include "MyAuth.h"

extern MyConf	g_conf;

//#define SOCKET_SEND_BUF			1400

enum {
	STAT_AUTH,
	STAT_TRANSACTION,
	STAT_UPDATE
};

#define RES_NG		0
#define RES_OK		1
#define RES_NOMES	2

static int MakeLockFile( const char* pszUserName );
static void DeleteLockFile( const char* pszUserName );

//-----------------------------------------------------------------------------
//	@detail
//		constructor
//	@param
//		pAT:		thread information
//	@return
//		nothing
//	@comment
//		
//-----------------------------------------------------------------------------
MyPop3Sock::MyPop3Sock(ACCEPTTHREAD* pAT)
{
	m_pAT = pAT;
	
	for( int ii = 0; ii < MAX_LOCALUSERNAME_LEN64+1; ii++ )
		m_szUserName[ii] = '\0';

	m_nState = STAT_AUTH;
	
	m_log.SetFacility( LOGFACILITY_MAIL );
	m_log.SetTag("POP3");
}

//-----------------------------------------------------------------------------
//	@detail
//		destructor
//	@param
//		nothing
//	@return
//		nothing
//	@comment
//		
//-----------------------------------------------------------------------------
MyPop3Sock::~MyPop3Sock()
{

}

//-----------------------------------------------------------------------------
//	@detail
//		Dialogue
//	@param
//		nothing
//	@return
//		always 1
//	@comment
//		
//-----------------------------------------------------------------------------
int MyPop3Sock::Dialogue()
{
	char szLine[MAX_CMDLINE+2];
	m_nState = STAT_AUTH;

	sprintf( m_szAuthKey, "<%u.%u@%s>", GetCurrentProcessId(), GetTickCount(), g_conf.GetValueI(CONFIG_ME) );

	Res( RES_OK, m_szAuthKey );
	m_szUserName[0] = '\0';

	while( 0 <= GetLine( szLine, MAX_CMDLINE, &m_pAT->bStop ) )
	{		
		if( m_pAT->bStop )
		{
			Res( RES_NG, "pop3 halt." );
			break;
		}

		m_log.Dbg( szLine );

		if( 0 == Dispatch( szLine ) )
			break;

		memset( szLine, 0, MAX_CMDLINE + 2 );
	}
	// delete lock file
	if( STAT_TRANSACTION == m_nState && '\0' != m_szUserName[0] )
	{
		DeleteLockFile( m_szUserName );
	}

	m_nState = STAT_AUTH;
	m_log.Dbg( "Connection closed." );
	Close();
	
	m_szUserName[0] = '\0';

	return 1;
}

//-----------------------------------------------------------------------------
//	@detail
//		Dispatch
//	@param
//		pszCmdLine:		Received string
//	@return
//		...
//	@comment
//		
//-----------------------------------------------------------------------------
int MyPop3Sock::Dispatch(char *pszCmdLine)
{
	if( 0 == STRNICMP( pszCmdLine, "NOOP", 4 ) )		return cmdNOOP( pszCmdLine );
	else if( 0 == STRNICMP( pszCmdLine, "USER", 4 ) )	return cmdUSER( pszCmdLine );
	else if( 0 == STRNICMP( pszCmdLine, "PASS", 4 ) )	return cmdPASS( pszCmdLine );
	else if( 0 == STRNICMP( pszCmdLine, "STAT", 4 ) )	return cmdSTAT( pszCmdLine );
	else if( 0 == STRNICMP( pszCmdLine, "LIST", 4 ) )	return cmdLIST( pszCmdLine );
	else if( 0 == STRNICMP( pszCmdLine, "RETR", 4 ) )	return cmdRETR( pszCmdLine );
	else if( 0 == STRNICMP( pszCmdLine, "DELE", 4 ) )	return cmdDELE( pszCmdLine );
	else if( 0 == STRNICMP( pszCmdLine, "RSET", 4 ) )	return cmdRSET( pszCmdLine );
	else if( 0 == STRNICMP( pszCmdLine, "QUIT", 4 ) )	return cmdQUIT( pszCmdLine );

	else if( 0 == STRNICMP( pszCmdLine, "APOP", 4 ) )	return cmdAPOP( pszCmdLine );
	else if( 0 == STRNICMP( pszCmdLine, "TOP", 3 ) )	return cmdTOP( pszCmdLine );
	else if( 0 == STRNICMP( pszCmdLine, "UIDL", 3 ) )	return cmdUIDL( pszCmdLine );

	return Res( RES_NG, "unimplemented" );;
}

/* RFC1939-3
   Responses in the POP3 consist of a status indicator and a keyword
   possibly followed by additional information.  All responses are
   terminated by a CRLF pair.  Responses may be up to 512 characters
   long, including the terminating CRLF.  There are currently two status
   indicators: positive ("+OK") and negative ("-ERR").  Servers MUST
   send the "+OK" and "-ERR" in upper case.
*/
//-----------------------------------------------------------------------------
//	@detail
//		Res
//	@param
//		nOK:	+OK or -NG
//		pMes:	additional string
//	@return
//		0:		socket error
//		1:		success
//	@comment
//		
//-----------------------------------------------------------------------------
int MyPop3Sock::Res(int nOK, const char *pMes /*= NULL*/)
{
	char szRes[512] = {0};
	char* pszRes = NULL;

	if( nOK == RES_OK )
		pszRes = "+OK ";
	else if( nOK == RES_NG )
		pszRes = "-ERR ";
	else
		pszRes = "";

	if( NULL == pMes )
		pMes="";
	
	// chop message
	// 512 - ("-ERR "+'CR'+'LF'+'\0') = 7bytes

	STRCPY( szRes, pszRes );
	STRNCAT( szRes, pMes, 512 - 7 );
	m_log.Dbg( szRes );

	STRCAT( szRes, "\r\n" );

	if( SOCKET_ERROR == SendData( szRes ) )
		return 0;

	return 1;
}

/*---------------------------------------------------------------------------
      NOOP

         Arguments: none

         Restrictions:
             may only be given in the TRANSACTION state

         Discussion:
             The POP3 server does nothing, it merely replies with a
             positive response.

         Possible Responses:
             +OK

         Examples:
             C: NOOP
             S: +OK
*/
//-----------------------------------------------------------------------------
//	@detail
//		cmdNOOP
//	@param
//		pszCmdLine
//	@return
//		...
//	@comment
//		
//-----------------------------------------------------------------------------
int MyPop3Sock::cmdNOOP(char *pszCmdLine)
{
	return Res( RES_OK );
}

/*
      USER name

         Arguments:
             a string identifying a mailbox (required), which is of
             significance ONLY to the server

         Restrictions:
             may only be given in the AUTHORIZATION state after the POP3
             greeting or after an unsuccessful USER or PASS command

         Discussion:
             To authenticate using the USER and PASS command
             combination, the client must first issue the USER
             command.  If the POP3 server responds with a positive
             status indicator ("+OK"), then the client may issue
             either the PASS command to complete the authentication,
             or the QUIT command to terminate the POP3 session.  If
             the POP3 server responds with a negative status indicator
             ("-ERR") to the USER command, then the client may either
             issue a new authentication command or may issue the QUIT
             command.

             The server may return a positive response even though no
             such mailbox exists.  The server may return a negative
             response if mailbox exists, but does not permit plaintext

             password authentication.

         Possible Responses:
             +OK name is a valid mailbox
             -ERR never heard of mailbox name

         Examples:
             C: USER frated
             S: -ERR sorry, no mailbox for frated here
                ...
             C: USER mrose
             S: +OK mrose is a real hoopy frood
*/
//-----------------------------------------------------------------------------
//	@detail
//		cmdUSER
//	@param
//		pszCmdLine
//	@return
//		...
//	@comment
//		
//-----------------------------------------------------------------------------
int MyPop3Sock::cmdUSER(char *pszCmdLine)
{
	if( m_nState != STAT_AUTH )
	{
		return Res( RES_NG );
	}

	if( 0 == STRICMP( "Yes", g_conf.GetValueI(CONFIG_USEAPOPONLY) ) )
	{
		return Res( RES_NG, "Use APOP." );
	}

	Usr usr;
	const char* pszUserName = &pszCmdLine[5];

	if( USR_LOCAL_DOMAIN != usr.IsMyUser( pszUserName ) )
	{
		return Res( RES_NG, "User unknown." );
	}

	STRNCPY( m_szUserName, pszUserName, MAX_LOCALUSERNAME_LEN64 );

	return Res( RES_OK, m_szUserName);
}

/*---------------------------------------------------------------------------
      PASS string

         Arguments:
             a server/mailbox-specific password (required)

         Restrictions:
             may only be given in the AUTHORIZATION state immediately
             after a successful USER command

         Discussion:
             When the client issues the PASS command, the POP3 server
             uses the argument pair from the USER and PASS commands to
             determine if the client should be given access to the
             appropriate maildrop.

             Since the PASS command has exactly one argument, a POP3
             server may treat spaces in the argument as part of the
             password, instead of as argument separators.

         Possible Responses:
             +OK maildrop locked and ready
             -ERR invalid password
             -ERR unable to lock maildrop

         Examples:
             C: USER mrose
             S: +OK mrose is a real hoopy frood
             C: PASS secret
             S: -ERR maildrop already locked
               ...
             C: USER mrose
             S: +OK mrose is a real hoopy frood
             C: PASS secret
             S: +OK mrose's maildrop has 2 messages (320 octets)
*/
//-----------------------------------------------------------------------------
//	@detail
//		cmdPASS
//	@param
//		pszCmdLine
//	@return
//		...
//	@comment
//		
//-----------------------------------------------------------------------------
int MyPop3Sock::cmdPASS(char *pszCmdLine)
{
	char* pErr;

	if( m_nState != STAT_AUTH )
	{
		return Res( RES_NG );
	}

	if( 0 == STRICMP( "Yes", g_conf.GetValueI(CONFIG_USEAPOPONLY) ) )
	{
		return Res( RES_NG, "Use APOP." );
	}

	const char* pszInputPasswd = &pszCmdLine[5];
	MyAuth auth;
	if( 0 == auth.AuthPLAIN( m_szUserName, pszInputPasswd ) )
	{
		pErr = "Authentication error.";
		m_log.Warn( m_szUserName );

		m_log.Warn( pErr );
		Res( RES_NG, pErr );
		return 0;
	}

	return Login();
}

//-----------------------------------------------------------------------------
//	@detail
//		cmdAPOP
//	@param
//		pszCmdLine
//	@return
//		...
//	@comment
//		
//-----------------------------------------------------------------------------
int MyPop3Sock::cmdAPOP(char *pszCmdLine)
{
	char szDigest[32+2] = {0};
	char* pErr;

	if( m_nState != STAT_AUTH )
	{
		return Res( RES_NG );
	}

	if( 0 == GetArg( pszCmdLine, m_szUserName, MAX_LOCALUSERNAME_LEN64 )
		|| 0 == GetArg( pszCmdLine, szDigest, 32, 1 ) )
	{
		pErr = "bad username or param.";
		
		m_log.Warn( pErr );
		m_log.Warn( m_szUserName );

		return Res( RES_NG, pErr );
	}

	MyAuth auth;
	if( 0 == auth.AuthAPOP( m_szUserName, szDigest, m_szAuthKey ) )
	{
		pErr = "Authentication error.";

		m_log.Warn( pErr );
		m_log.Warn( m_szUserName );
		
		Res( RES_NG, pErr );
		return 0;
	}
	*m_szAuthKey = '\0';
	return Login();
}

//-----------------------------------------------------------------------------
//	@detail
//		Login
//	@param
//		nothing
//	@return
//		0:	login failed
//		1:	login successed
//	@comment
//		
//-----------------------------------------------------------------------------
int MyPop3Sock::Login()
{
	// make list
	char* pErr;
	Usr	u;
	char szHomeDir[MAX_PATH+1] = {0};
	if( !u.GetHomePath( m_szUserName, szHomeDir ) )
	{
		pErr = "GetHomePath failed.";
		m_log.Err( pErr );
		Res( RES_NG, "GetHomePath failed." );
		return 0;
	}

	if( 0 == m_list.CreateList(szHomeDir) )
	{
		pErr = "CreateList failed.";
		m_log.Err( pErr );
		Res( RES_NG, pErr );
		return 0;
	}

	// lock user
	if( 0 == MakeLockFile(m_szUserName) )
		return Res( RES_NG, "User locked." );

	// transacrion state from here.
	m_nState = STAT_TRANSACTION;
	
	sockaddr_in saddr;
	int saddrsize = sizeof(sockaddr);
	getpeername( m_socket, (sockaddr*)&saddr, &saddrsize );


	char szUserName[MAX_LOCALUSERNAME_LEN64 + 64/*add message +64*/] = {0};
	char *pszRemoteHost = inet_ntoa(saddr.sin_addr);
	if( NULL == pszRemoteHost )
	{
		pszRemoteHost = "unknown";
		sprintf( szUserName, "%s login failed. ipaddress unknown", m_szUserName );
		m_log.Info( szUserName );
		return Res( RES_NG, "ip address unknown." );
	}
	sprintf( szUserName, "%s logged in(%s).", m_szUserName, pszRemoteHost );
	m_log.Info( szUserName );

	MyAuthIPList ip;
	ip.Set( g_conf.GetValueI(CONFIG_POPAUTHDIR), pszRemoteHost );

	return Res( RES_OK, szUserName );

}

/*---------------------------------------------------------------------------
      STAT

         Arguments: none

         Restrictions:
             may only be given in the TRANSACTION state

         Discussion:
             The POP3 server issues a positive response with a line
             containing information for the maildrop.  This line is
             called a "drop listing" for that maildrop.

             In order to simplify parsing, all POP3 servers are
             required to use a certain format for drop listings.  The
             positive response consists of "+OK" followed by a single
             space, the number of messages in the maildrop, a single
             space, and the size of the maildrop in octets.  This memo
             makes no requirement on what follows the maildrop size.
             Minimal implementations should just end that line of the
             response with a CRLF pair.  More advanced implementations
             may include other information.

                NOTE: This memo STRONGLY discourages implementations
                from supplying additional information in the drop
                listing.  Other, optional, facilities are discussed
                later on which permit the client to parse the messages
                in the maildrop.

             Note that messages marked as deleted are not counted in
             either total.

         Possible Responses:
             +OK nn mm

         Examples:
             C: STAT
             S: +OK 2 320
*/
//-----------------------------------------------------------------------------
//	@detail
//		cmdSTAT
//	@param
//		pszCmdLine
//	@return
//		...
//	@comment
//		
//-----------------------------------------------------------------------------
int MyPop3Sock::cmdSTAT(char *pszCmdLine)
{
	unsigned long dwCount = 0;
	unsigned long dwTotalOctet = 0;
	char szBuf[64];

	if( m_nState != STAT_TRANSACTION )
	{
		return Res( RES_NG, "Authorization first." );
	}
	m_list.Stat( &dwCount, &dwTotalOctet );

	sprintf( szBuf, "%u %u", dwCount, dwTotalOctet );
	return Res( RES_OK, szBuf );
}

/*---------------------------------------------------------------------------
         Arguments:
             a message-number (optional), which, if present, may NOT
             refer to a message marked as deleted

         Restrictions:
             may only be given in the TRANSACTION state

         Discussion:
             If an argument was given and the POP3 server issues a
             positive response with a line containing information for
             that message.  This line is called a "scan listing" for
             that message.

             If no argument was given and the POP3 server issues a
             positive response, then the response given is multi-line.
             After the initial +OK, for each message in the maildrop,
             the POP3 server responds with a line containing
             information for that message.  This line is also called a
             "scan listing" for that message.  If there are no
             messages in the maildrop, then the POP3 server responds
             with no scan listings--it issues a positive response
             followed by a line containing a termination octet and a
             CRLF pair.

             In order to simplify parsing, all POP3 servers are
             required to use a certain format for scan listings.  A
             scan listing consists of the message-number of the
             message, followed by a single space and the exact size of
             the message in octets.  Methods for calculating the exact
             size of the message are described in the "Message Format"
             section below.  This memo makes no requirement on what
             follows the message size in the scan listing.  Minimal
             implementations should just end that line of the response
             with a CRLF pair.  More advanced implementations may
             include other information, as parsed from the message.

                NOTE: This memo STRONGLY discourages implementations
                from supplying additional information in the scan
                listing.  Other, optional, facilities are discussed
                later on which permit the client to parse the messages
                in the maildrop.

             Note that messages marked as deleted are not listed.

         Possible Responses:
             +OK scan listing follows
             -ERR no such message

         Examples:
             C: LIST
             S: +OK 2 messages (320 octets)
             S: 1 120

             S: 2 200
             S: .
               ...
             C: LIST 2
             S: +OK 2 200
               ...
             C: LIST 3
             S: -ERR no such message, only 2 messages in maildrop
*/
//-----------------------------------------------------------------------------
//	@detail
//		cmdLIST
//	@param
//		pszCmdLine
//	@return
//		...
//	@comment
//		
//-----------------------------------------------------------------------------
int MyPop3Sock::cmdLIST(char *pszCmdLine)
{
	if( m_nState != STAT_TRANSACTION )
	{
		return Res( RES_NG, "Authorization first." );
	}

	SEEKKEY sk;
	char szMsgNo[31] = {0};
	unsigned long dwOctet;

	// one argument

	if( 0 != GetArg( pszCmdLine, szMsgNo, 127 ) )
	{
		unsigned long dwNumber;
		dwNumber = ATOUI( szMsgNo );

		if( 0 == ATOUI( szMsgNo ) )
		{
			return Res( RES_NG, "bad message number." );
		}

		sk = m_list.SeekToKey( szMsgNo );

		char szBuf[128] = {0};
		if( NULL != sk )
		{
			dwOctet = m_list.GetOctet( sk );
			sprintf( szBuf, "%s %u", sk, dwOctet );
			return Res( RES_OK, szBuf );
		}
		return Res( RES_NG, "no such message." );
	}

	// no arguments listing...

	sk = m_list.SeekToFirst();

	if( 0 != Res( RES_OK ) )
	{
		while( NULL != sk )
		{
			if( DELE_MARK != *sk )
			{
				char szBuf[128] = {0};
				dwOctet = m_list.GetOctet( sk );
				sprintf( szBuf, "%s %u", sk, dwOctet );
				Res( RES_NOMES, szBuf );
			}
			sk = m_list.SeekNext( sk );
		}

		return Res( RES_NOMES, "." );
	}
	
	return 0;
}

/*
      UIDL [msg]

      Arguments:
          a message-number (optional), which, if present, may NOT
          refer to a message marked as deleted

      Restrictions:
          may only be given in the TRANSACTION state.

      Discussion:
          If an argument was given and the POP3 server issues a positive
          response with a line containing information for that message.
          This line is called a "unique-id listing" for that message.

          If no argument was given and the POP3 server issues a positive
          response, then the response given is multi-line.  After the
          initial +OK, for each message in the maildrop, the POP3 server
          responds with a line containing information for that message.
          This line is called a "unique-id listing" for that message.

          In order to simplify parsing, all POP3 servers are required to
          use a certain format for unique-id listings.  A unique-id
          listing consists of the message-number of the message,
          followed by a single space and the unique-id of the message.
          No information follows the unique-id in the unique-id listing.

          The unique-id of a message is an arbitrary server-determined
          string, consisting of one to 70 characters in the range 0x21
          to 0x7E, which uniquely identifies a message within a
          maildrop and which persists across sessions.  This
          persistence is required even if a session ends without
          entering the UPDATE state.  The server should never reuse an
          unique-id in a given maildrop, for as long as the entity
          using the unique-id exists.

          Note that messages marked as deleted are not listed.

          While it is generally preferable for server implementations
          to store arbitrarily assigned unique-ids in the maildrop,

          this specification is intended to permit unique-ids to be
          calculated as a hash of the message.  Clients should be able
          to handle a situation where two identical copies of a
          message in a maildrop have the same unique-id.

      Possible Responses:
          +OK unique-id listing follows
          -ERR no such message

      Examples:
          C: UIDL
          S: +OK
          S: 1 whqtswO00WBw418f9t5JxYwZ
          S: 2 QhdPYR:00WBw1Ph7x7
          S: .
             ...
          C: UIDL 2
          S: +OK 2 QhdPYR:00WBw1Ph7x7
             ...
          C: UIDL 3
          S: -ERR no such message, only 2 messages in maildrop
*/
//-----------------------------------------------------------------------------
//	@detail
//		cmdUIDL
//	@param
//		pszCmdLine
//	@return
//		...
//	@comment
//		
//-----------------------------------------------------------------------------
int MyPop3Sock::cmdUIDL(char *pszCmdLine)
{
	if( m_nState != STAT_TRANSACTION )
	{
		return Res( RES_NG, "Authorization first." );
	}

	SEEKKEY sk;
	char szMsgNo[31] = {0};
	char szBuf[128+UNIQUE_ID_MAX_LEN] = {0};
	const char* pszUniqueID;

	// one argument

	if( 0 != GetArg( pszCmdLine, szMsgNo, 127 ) )
	{
		unsigned long dwNumber;
		dwNumber = ATOUI( szMsgNo );

		if( 0 == ATOUI( szMsgNo ) )
		{
			return Res( RES_NG, "bad message number." );
		}

		sk = m_list.SeekToKey( szMsgNo );

		if( NULL != sk )
		{
			pszUniqueID = m_list.GetUID( sk );
			if( NULL == pszUniqueID )
			{
					char* pErr = "System error, no uid.";
					m_log.Err( pErr );
					Res( RES_NG, pErr );
				return 0;
			}
			sprintf( szBuf, "%u %s", ATOUI(sk), pszUniqueID );
			return Res( RES_OK, szBuf );
		}
		return Res( RES_NG, "no such message." );
	}

	// no arguments, listing...

	sk = m_list.SeekToFirst();

	if( 0 != Res( RES_OK ) )
	{
		while( NULL != sk )
		{
			if( DELE_MARK != *sk )
			{
				pszUniqueID = m_list.GetUID( sk );
				if( NULL == pszUniqueID )
				{
					char* pErr = "System error, no uid.";
					m_log.Err( pErr );
					Res( RES_NG, pErr );
					return 0;
				}
				sprintf( szBuf, "%u %s", ATOUI(sk), pszUniqueID );
				Res( RES_NOMES, szBuf );
			}
			sk = m_list.SeekNext( sk );
		}

		return Res( RES_NOMES, "." );
	}
	
	return 0;
}
/*---------------------------------------------------------------------------
      RETR msg

         Arguments:
             a message-number (required) which may NOT refer to a
             message marked as deleted

         Restrictions:
             may only be given in the TRANSACTION state

         Discussion:
             If the POP3 server issues a positive response, then the
             response given is multi-line.  After the initial +OK, the
             POP3 server sends the message corresponding to the given
             message-number, being careful to byte-stuff the termination
             character (as with all multi-line responses).

         Possible Responses:
             +OK message follows
             -ERR no such message

         Examples:
             C: RETR 1
             S: +OK 120 octets
             S: <the POP3 server sends the entire message here>
             S: .
*/
//-----------------------------------------------------------------------------
//	@detail
//		cmdRETR
//	@param
//		pszCmdLine
//	@return
//		...
//	@comment
//		
//-----------------------------------------------------------------------------
int MyPop3Sock::cmdRETR(char *pszCmdLine)
{
	if( m_nState != STAT_TRANSACTION )
	{
		return Res( RES_NG, "Authorization first." );
	}

	char szMsgNo[32] = {0};

	if( 0 == GetArg( pszCmdLine, szMsgNo, 31 ) )
	{
		return Res( RES_NG, "need message number." );
	}

	if( 0 == ATOUI( szMsgNo ) )
	{
		return Res( RES_NG, "bad message number." );
	}

	SEEKKEY sk = m_list.SeekToFirst();

	while( NULL != sk )
	{
		if( DELE_MARK != *sk )
		{
			if(0 == STRCMP( szMsgNo, sk ) )
				break;
		}
		sk = m_list.SeekNext( sk );
	}
	
	if( NULL == sk )
	{
		return Res( RES_NG, "no such message." );
	}

	char szHomeDir[MAX_PATH+1] = {0};
	Usr	u;
	const char* pFileName = m_list.GetUID( sk );

	if( !u.GetHomePath( m_szUserName, szHomeDir ) )
	{
		Res( RES_NG, "sorry, system error GetHomePath" );
		return 0;
	}

	STRCAT( szHomeDir, "\\" );
	STRCAT( szHomeDir, pFileName );

	MyString sFile;
	if( 0 == sFile.Set( szHomeDir ) )
	{
		return 0;
	}


	HANDLE hFile = CreateFile( sFile, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL );

	if( hFile == INVALID_HANDLE_VALUE )
	{
		return Res( RES_NG, "sorry, system error open file." );
	}

	Res( RES_OK );

	DWORD	dwReadBytes;
	char	szSendBuf[MAX_CMDLINE];
	do
	{
		// the file is divided and transmitted.
		if( ReadFile( hFile, szSendBuf, MAX_CMDLINE, &dwReadBytes, NULL ) )
		{
			if( dwReadBytes == 0 )
			{
				break;
			}

			if(  SendData(szSendBuf, dwReadBytes) < 1 )
			{
				CloseHandle( hFile );
				return 0;
			}
		}
		else
		{
			break;
		}
	}while( dwReadBytes > 0 );

	CloseHandle( hFile );

	return Res( RES_NOMES, "\r\n." );
}

/*
      DELE msg

         Arguments:
             a message-number (required) which may NOT refer to a
             message marked as deleted

         Restrictions:
             may only be given in the TRANSACTION state

         Discussion:
             The POP3 server marks the message as deleted.  Any future
             reference to the message-number associated with the message
             in a POP3 command generates an error.  The POP3 server does
             not actually delete the message until the POP3 session
             enters the UPDATE state.

         Possible Responses:
             +OK message deleted
             -ERR no such message

         Examples:
             C: DELE 1
             S: +OK message 1 deleted
                ...
             C: DELE 2
             S: -ERR message 2 already deleted
*/
//-----------------------------------------------------------------------------
//	@detail
//		cmdDELE
//	@param
//		pszCmdLine
//	@return
//		...
//	@comment
//		
//-----------------------------------------------------------------------------
int MyPop3Sock::cmdDELE(char *pszCmdLine)
{
	if( m_nState != STAT_TRANSACTION )
	{
		return Res( RES_NG, "Authorization first." );
	}

	char pszParm[32] = {0};

	if( GetArg( pszCmdLine, pszParm, 31 ) )
	{
		if( 0 == ATOUI( pszParm ) )
		{
			return Res( RES_NG, "bad message number." );
		}

		if( 0 == m_list.Dele( pszParm ) )
		{
			Res( RES_NG, "sorry, system error at m_list.Dele." );
			return 0;
		}
		return Res( RES_OK, "message deleted." );
	}
	
	return Res( RES_NG, "argument must." );
}

/*---------------------------------------------------------------------------
      RSET

         Arguments: none

         Restrictions:
             may only be given in the TRANSACTION state

         Discussion:
             If any messages have been marked as deleted by the POP3
             server, they are unmarked.  The POP3 server then replies

             with a positive response.

         Possible Responses:
             +OK

         Examples:
             C: RSET
             S: +OK maildrop has 2 messages (320 octets)
*/
//-----------------------------------------------------------------------------
//	@detail
//		cmdRSET
//	@param
//		pszCmdLine
//	@return
//		...
//	@comment
//		
//-----------------------------------------------------------------------------
int MyPop3Sock::cmdRSET(char *pszCmdLine)
{
	if( m_nState != STAT_TRANSACTION )
	{
		return Res( RES_NG, "Authorization first." );
	}

	m_list.Rset();

	return Res( RES_OK, "reset." );
}


/*---------------------------------------------------------------------------
6. The UPDATE State

   When the client issues the QUIT command from the TRANSACTION state,
   the POP3 session enters the UPDATE state.  (Note that if the client
   issues the QUIT command from the AUTHORIZATION state, the POP3
   session terminates but does NOT enter the UPDATE state.)

   If a session terminates for some reason other than a client-issued
   QUIT command, the POP3 session does NOT enter the UPDATE state and
   MUST not remove any messages from the maildrop.

      QUIT

         Arguments: none

         Restrictions: none

         Discussion:
             The POP3 server removes all messages marked as deleted
             from the maildrop and replies as to the status of this
             operation.  If there is an error, such as a resource
             shortage, encountered while removing messages, the
             maildrop may result in having some or none of the messages
             marked as deleted be removed.  In no case may the server
             remove any messages not marked as deleted.

             Whether the removal was successful or not, the server
             then releases any exclusive-access lock on the maildrop
             and closes the TCP connection.

         Possible Responses:
             +OK
             -ERR some deleted messages not removed

         Examples:
             C: QUIT
             S: +OK dewey POP3 server signing off (maildrop empty)
                ...
             C: QUIT

             S: +OK dewey POP3 server signing off (2 messages left)
                ...

*/

//-----------------------------------------------------------------------------
//	@detail
//		cmdQUIT
//	@param
//		pszCmdLine
//	@return
//		always 0
//	@comment
//		
//-----------------------------------------------------------------------------
int MyPop3Sock::cmdQUIT(char*pszCmdLine)
{

	if( m_nState == STAT_TRANSACTION )
	{
		// update
		m_list.Update();
	}

	Res( RES_OK, "bye." );
	return 0;
}

/*
      TOP msg n

         Arguments:
             a message-number (required) which may NOT refer to to a
             message marked as deleted, and a non-negative number
             of lines (required)

         Restrictions:
             may only be given in the TRANSACTION state

         Discussion:
             If the POP3 server issues a positive response, then the
             response given is multi-line.  After the initial +OK, the
             POP3 server sends the headers of the message, the blank
             line separating the headers from the body, and then the
             number of lines of the indicated message's body, being
             careful to byte-stuff the termination character (as with
             all multi-line responses).

             Note that if the number of lines requested by the POP3
             client is greater than than the number of lines in the
             body, then the POP3 server sends the entire message.

         Possible Responses:
             +OK top of message follows
             -ERR no such message

         Examples:
             C: TOP 1 10
             S: +OK

             S: <the POP3 server sends the headers of the
                message, a blank line, and the first 10 lines
                of the body of the message>
             S: .
                ...
             C: TOP 100 3
             S: -ERR no such message

*/
//-----------------------------------------------------------------------------
//	@detail
//		cmdTOP
//	@param
//		pszCmdLine
//	@return
//		...
//	@comment
//		
//-----------------------------------------------------------------------------
int MyPop3Sock::cmdTOP(char *pszCmdLine)
{
	if( m_nState != STAT_TRANSACTION )
	{
		return Res( RES_NG, "Authorization first." );
	}

	char szMsgNo[32] = {0};
	char szLineNo[32] = {0};

	// args 1
	if( 0 == GetArg( pszCmdLine, szMsgNo, 31 ) )
	{
		return Res( RES_NG, "need message number." );
	}

	if( 0 == ATOUI( szMsgNo ) )
	{
		return Res( RES_NG, "bad message number." );
	}

	// args 2
	GetArg( pszCmdLine, szLineNo, 31, 1 );
	
	SEEKKEY sk = m_list.SeekToFirst();

	while( NULL != sk )
	{
		if( DELE_MARK != *sk )
		{
			if(0 == STRCMP( szMsgNo, sk ) )
				break;
		}
		sk = m_list.SeekNext( sk );
	}
	
	if( NULL == sk )
	{
		return Res( RES_NG, "no such message." );
	}

	char szHomeDir[MAX_PATH+1] = {0};
	Usr	u;
	const char* pFileName = m_list.GetUID( sk );

	if( !u.GetHomePath( m_szUserName, szHomeDir ) )
	{
		Res( RES_NG, "sorry, system error GetHomePath" );
		return 0;
	}

	STRCAT( szHomeDir, "\\" );
	STRCAT( szHomeDir, pFileName );

	MyString sFile;
	if( 0 == sFile.Set( szHomeDir ) )
	{
		return 0;
	}


	HANDLE hFile = CreateFile( sFile, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL );

	if( hFile == INVALID_HANDLE_VALUE )
	{
		return Res( RES_NG, "sorry, system error open file." );
	}

	Res( RES_OK );

	char	szSendBuf[MAX_CMDLINE+2]={0};

	// header send
	while( NULL != GetFileLine( hFile, szSendBuf, MAX_CMDLINE ) )
	{
		STRCAT( szSendBuf, "\r\n" );

		if(  SendData(szSendBuf) < 1 )
		{
			CloseHandle( hFile );
			return 0;
		}
		if( '\r' == szSendBuf[0] )
			break;
	}

	for( unsigned int ii = 0; ii < ATOUI(szLineNo); ii++ )
	{
		if( NULL == GetFileLine( hFile, szSendBuf, MAX_CMDLINE ) )
			break;
		// byte stuffing...

		if( '.' == szSendBuf[0] && '\0' == szSendBuf[1] )
		{
			STRCAT( szSendBuf, "." );
		}

		STRCAT( szSendBuf, "\r\n" );

		if(  SendData(szSendBuf) < 1 )
		{
			CloseHandle( hFile );
			return 0;
		}
	}

	CloseHandle( hFile );

	return Res( RES_NOMES, "." );
}

//-----------------------------------------------------------------------------
//	@detail
//		MakeLockFile
//	@param
//		pszUserName
//	@return
//		1:	success
//		0:	failed
//	@comment
//		
//-----------------------------------------------------------------------------
static int MakeLockFile( const char* pszUserName )
{
	char szLockFile[MAX_PATH+1];
	STRCPY( szLockFile, g_conf.GetValueI( CONFIG_TMPDIR ) );
	STRCAT( szLockFile, pszUserName );
	STRCAT( szLockFile, LOCKFILEMARK );

	HANDLE hFile;
	MyString s;
	if( 0 == s.Set( szLockFile ) )
	{
		return 0;
	}

	hFile = CreateFile( s, GENERIC_READ|GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_NEW, 0, NULL );

	if( INVALID_HANDLE_VALUE == hFile )
		return 0;

	CloseHandle( hFile );
	return 1;
}

//-----------------------------------------------------------------------------
//	@detail
//		DeleteLockFile
//	@param
//		pszUserName
//	@return
//		nothing
//	@comment
//		
//-----------------------------------------------------------------------------
static void DeleteLockFile( const char* pszUserName )
{
	char szLockFile[MAX_PATH+1];
	STRCPY( szLockFile, g_conf.GetValueI( CONFIG_TMPDIR ) );
	STRCAT( szLockFile, pszUserName );
	STRCAT( szLockFile, LOCKFILEMARK );

	MyString s;
	if( 0 == s.Set( szLockFile ) )
	{
		return;
	}

	DeleteFile( s );
}
