/* =========================================================================
	This library is part of IshiSoft application.
	------------------------------------------------------------------------
	author: Yuichiro Ishitani
			http://www001.upp.so-net.ne.jp/ishi/index.html
  ========================================================================== */
#include "stdafx.h"
#include <stdio.h>
#include "MySmtpSock.h"
#include "Str.h"
#include "MyConf.h"
#include "commonhelper.h"
#include "usr.h"
#include "smtp_errcode.h"
#include "MyTime.h"
#include "staticvariable.h"
#include "MyAuthIPList.h"
#include "MyAuth.h"

#include "malloc.h"
extern MyConf			g_conf;
extern MyHashFile		g_rcpthosts;

#define CHANNEL_DIALOGUE		0
#define CHANNEL_DATA			1
#define CHANNEL_ERRDATA_552		2
#define CHANNEL_ERRDATA_451		3

#define CHANNEL_AUTH_LOGIN		4
#define CHANNEL_AUTH_PLAIN		5
#define CHANNEL_AUTH_CRAMMD5	6
//#define CHANNEL_AUTH_DIGESTMD5	7


static int StartSenderSMTP(const char* pszFile);
static int SevenBitCheck( const char* pszSrc );

static int SevenBitCheck( const char* pszSrc )
{
	while( '\0' != *pszSrc )
	{
		if(*pszSrc++ & 0x80) return 0;
	}
	return 1;
}

//-----------------------------------------------------------------------------
//	@detail
//		ReverseMatch
//	@param
//		pszDst	: Dest string
//		pszSrc	: check string
//	@return
//		0 no match
//		1 match
//	@comment
//		string compare from tail
//		src="foo.bar.com"
//-----------------------------------------------------------------------------
static int ReverseMatch(const char* pszDst, const char* pszSrc)
{
	int ii;
	int nDstCnt = 0;
	int nSrcCnt = 0;
	char d = 0;
	char s = 0;

	while( '\0' != *pszDst ) { pszDst++; nDstCnt++; }
	while( '\0' != *pszSrc ) { pszSrc++; nSrcCnt++; }

	if( nDstCnt < nSrcCnt )
		return 0;

	for ( ii = 0; ii < nSrcCnt; ii++ )
	{
		d = *--pszDst;
		s = *--pszSrc;
		d = (d >= 'A') && (d <= 'Z') ? (d += 'a' - 'A') : d;
		s = (s >= 'A') && (s <= 'Z') ? (s += 'a' - 'A') : s;
		
		if( d != s )
			return 0;
	}
	return 1;
}

//-----------------------------------------------------------------------------
//	@detail
//		MySmtpSock
//	@param
//		pAT
//	@return
//		nothing
//	@comment
//		construct
//-----------------------------------------------------------------------------
MySmtpSock::MySmtpSock(ACCEPTTHREAD* pAT)
{
	m_pAT = pAT;
	m_pszAuthKey = NULL;

	cmdRSET(NULL);

	m_unMaxMailSize = g_conf.GetUINTValueI( CONFIG_MAXMAILSIZE );
	if( m_unMaxMailSize < 1024*64 )
		m_unMaxMailSize = 1024*64;	// least 64KB need.

	m_log.SetFacility( LOGFACILITY_MAIL );
	m_log.SetTag("RSMTP");
}

MySmtpSock::~MySmtpSock()
{
}

//-----------------------------------------------------------------------------
//	@detail
//		Dialogue
//	@param
//		nothing
//	@return
//		0:	failed
//		1:	success
//	@comment
//		
//-----------------------------------------------------------------------------
int MySmtpSock::Dialogue()
{
/* RFC2822 2.1.1. Line Length Limits
   There are two limits that this standard places on the number of
   characters in a line. Each line of characters MUST be no more than
   998 characters, and SHOULD be no more than 78 characters, excluding
   the CRLF.
*/
	char szLine[MAX_CMDLINE] = {0};

	m_nCannnelMode = CHANNEL_DIALOGUE;
	
	// Rset auth status.
	if( 0 == STRICMP( "Yes", g_conf.GetValueI( CONFIG_USESMTPAUTHPLAIN ) )
		|| 0 == STRICMP( "Yes", g_conf.GetValueI( CONFIG_USESMTPAUTHLOGIN ) )
		|| 0 == STRICMP( "Yes", g_conf.GetValueI( CONFIG_USESMTPAUTHCRAMMD5 ) )
//		|| 0 == STRICMP( "Yes", g_conf.GetValueI( CONFIG_USESMTPAUTHDIGESTMD5 ) )
		)
	{
		m_bAuthorized = 0;
	}
	else
	{
		m_bAuthorized = 1;		// SMTP authentication not use.
	}

	// Greeting
	sprintf( szLine, "%s ESMTP", g_conf.GetValueI(CONFIG_ME) );
	Res( 220, szLine );

	while( 0 <= GetLine( szLine, MAX_CMDLINE, &m_pAT->bStop, g_conf.GetUINTValueI(CONFIG_SMTPDTIMEOUT) ) )
	{
		m_log.Dbg( szLine );
		if( m_pAT->bStop )
		{
			Res( 421 );
			break;
		}

		if( 0 == Dispatch( szLine ) )
			break;
	}
	m_log.Dbg( "Connection closed." );
	m_TmpFile.Delete();
	Close();

	cmdRSET(NULL);

	return 1;
}

// ============================================================================
// RFC 821 4.1    SMTP COMMANDS
//         4.1.1  COMMAND SEMANTICS
// ============================================================================
//-----------------------------------------------------------------------------
//	@detail
//		Dispatch
//	@param
//		char* pszCmdLine
//	@return
//		
//	@comment
//		command dispatch
//-----------------------------------------------------------------------------
int MySmtpSock::Dispatch(char *pszCmdLine)
{
	switch( m_nCannnelMode )
	{
	case CHANNEL_DATA:
		return recvDATA( pszCmdLine );
	case CHANNEL_ERRDATA_451:
	case CHANNEL_ERRDATA_552:
		return recvInvalidData( pszCmdLine );
	case CHANNEL_AUTH_CRAMMD5:
		return recvAuthCRAMMD5( pszCmdLine );
	case CHANNEL_AUTH_LOGIN:
		return recvAuthLOGIN( pszCmdLine );
	default:
		break;
	}

	if( 0 == SevenBitCheck( pszCmdLine ) )
		return Res( 500 );

	if(      0 == STRNICMP( pszCmdLine, "NOOP", 4 ) ){	return cmdNOOP( pszCmdLine );	}
	else if( 0 == STRNICMP( pszCmdLine, "HELP", 4 ) ){	return cmdHELP( pszCmdLine );	}
	else if( 0 == STRNICMP( pszCmdLine, "EHLO", 4 ) ){	return cmdEHLO( pszCmdLine );	}
	else if( 0 == STRNICMP( pszCmdLine, "HELO", 4 ) ){	return cmdHELO( pszCmdLine );	}
	else if( 0 == STRNICMP( pszCmdLine, "AUTH", 4 ) ){	return cmdAUTH( pszCmdLine );	}
	else if( 0 == STRNICMP( pszCmdLine, "MAIL", 4 ) ){	return cmdMAIL( pszCmdLine );	}
	else if( 0 == STRNICMP( pszCmdLine, "RCPT", 4 ) ){	return cmdRCPT( pszCmdLine );	}
	else if( 0 == STRNICMP( pszCmdLine, "DATA", 4 ) ){	return cmdDATA( pszCmdLine );	}
	else if( 0 == STRNICMP( pszCmdLine, "RSET", 4 ) ){	return cmdRSET( pszCmdLine );	}
	else if( 0 == STRNICMP( pszCmdLine, "VRFY", 4 ) ){	return cmdVRFY( pszCmdLine );	}
	else if( 0 == STRNICMP( pszCmdLine, "EXPN", 4 ) ){	return cmdEXPN( pszCmdLine );	}
	else if( 0 == STRNICMP( pszCmdLine, "QUIT", 4 ) ){	return cmdQUIT( pszCmdLine );	}


	SendData( "502 unimplemented\r\n" );
	return 502;
}

//-----------------------------------------------------------------------------
//	@detail
//		cmdHELP
//	@param
//		pszCmdLine
//	@return
//		SUCCESS	214
//	@comment
//		
//-----------------------------------------------------------------------------
int MySmtpSock::cmdHELP(char *pszCmdLine)
{
	return Res( 214, "NOOP,EHLO,HELO,MAIL RCPT,DATA,RSET,VRFY,EXPN,QUIT" );
}

//-----------------------------------------------------------------------------
//	@detail
//		cmdNOOP
//	@param
//		pszCmdLine
//	@return
//		SUCCESS 250 
//		ERROR   500, 421 
//	@comment
//		
//-----------------------------------------------------------------------------
int MySmtpSock::cmdNOOP(char *pszCmdLine)
{
	return Res( 250 );
}

/*
RFC 2821 4.1.1.1  Extended HELLO (EHLO) or HELLO (HELO)

   These commands are used to identify the SMTP client to the SMTP
   server.  The argument field contains the fully-qualified domain name
   of the SMTP client if one is available.  In situations in which the
   SMTP client system does not have a meaningful domain name (e.g., when
   its address is dynamically allocated and no reverse mapping record is
   available), the client SHOULD send an address literal (see section
   4.1.3), optionally followed by information that will help to identify
   the client system.  y The SMTP server identifies itself to the SMTP
   client in the connection greeting reply and in the response to this
   command.

   A client SMTP SHOULD start an SMTP session by issuing the EHLO
   command.  If the SMTP server supports the SMTP service extensions it
   will give a successful response, a failure response, or an error
   response.  If the SMTP server, in violation of this specification,
   does not support any SMTP service extensions it will generate an
   error response.  Older client SMTP systems MAY, as discussed above,
   use HELO (as specified in RFC 821) instead of EHLO, and servers MUST
   support the HELO command and reply properly to it.  In any event, a
   client MUST issue HELO or EHLO before starting a mail transaction.

   These commands, and a "250 OK" reply to one of them, confirm that
   both the SMTP client and the SMTP server are in the initial state,
   that is, there is no transaction in progress and all state tables and
   buffers are cleared.

   Syntax:

      ehlo            = "EHLO" SP Domain CRLF
      helo            = "HELO" SP Domain CRLF

   Normally, the response to EHLO will be a multiline reply.  Each line
   of the response contains a keyword and, optionally, one or more
   parameters.  Following the normal syntax for multiline replies, these
   keyworks follow the code (250) and a hyphen for all but the last
   line, and the code and a space for the last line.  The syntax for a
   positive response, using the ABNF notation and terminal symbols of
   [8], is:

      ehlo-ok-rsp  =    ( "250"    domain [ SP ehlo-greet ] CRLF )
                   / (    "250-"   domain [ SP ehlo-greet ] CRLF
                       *( "250-"   ehlo-line                CRLF )
                          "250"    SP ehlo-line             CRLF  )

      ehlo-greet   = 1*(%d0-9 / %d11-12 / %d14-127)
                   ; string of any characters other than CR or LF

      ehlo-line    = ehlo-keyword *( SP ehlo-param )

      ehlo-keyword = (ALPHA / DIGIT) *(ALPHA / DIGIT / "-")
                   ; additional syntax of ehlo-params depends on
                   ; ehlo-keyword

      ehlo-param   = 1*(%d33-127)
                   ; any CHAR excluding <SP> and all
                   ; control characters (US-ASCII 0-31 inclusive)

   Although EHLO keywords may be specified in upper, lower, or mixed
   case, they MUST always be recognized and processed in a case-
   insensitive manner.  This is simply an extension of practices
   specified in RFC 821 and section 2.4.1.
*/
//-----------------------------------------------------------------------------
//	@detail
//		cmdEHLO
//	@param
//		pszCmdLine
//	@return
//		SUCCESS 250
//		ERROR   500, 501, 504, 421 
//	@comment
//		
//-----------------------------------------------------------------------------
int MySmtpSock::cmdEHLO(char *pszCmdLine)
{
	char szDomain[MAX_DOMAIN_LEN255+1] = {0};
	char szResMsg[512];

	if( GetArg( pszCmdLine, szDomain, MAX_DOMAIN_LEN255, 0 ) )
	{
		if( m_vmHelo.Add( szDomain ) )
		{
			cmdRSET(NULL);

			// greeting message
			sprintf( szResMsg, "250-%s\r\n", g_conf.GetValueI(CONFIG_ME) );
			SendData( szResMsg );

			// auth
			if( 0 == m_bAuthorized )
			{
				sprintf( szResMsg, "250-AUTH" );
				if( 0 == STRICMP( "Yes", g_conf.GetValueI( CONFIG_USESMTPAUTHPLAIN ) ) )
					STRCAT( szResMsg, " PLAIN" );
				if( 0 == STRICMP( "Yes", g_conf.GetValueI( CONFIG_USESMTPAUTHLOGIN ) ) )
					STRCAT( szResMsg, " LOGIN" );
				if( 0 == STRICMP( "Yes", g_conf.GetValueI( CONFIG_USESMTPAUTHCRAMMD5 ) ) )
					STRCAT( szResMsg, " CRAM-MD5" );
				//if( 0 == STRICMP( "Yes", g_conf.GetValueI( CONFIG_USESMTPAUTHDIGESTMD5 ) ) )
				//	STRCAT( szResMsg, "DIGEST-MD5" );

				STRCAT( szResMsg, "\r\n" );

				m_log.Dbg( szResMsg );
				SendData( szResMsg );
			}

			// size
			/*
			unsigned int dwMaxMailSize = g_conf.GetUINTValueI( CONFIG_MAXMAILSIZE );
			if( 0 == dwMaxMailSize )
			{
				sprintf( szResMsg, "250-SIZE %u\r\n",  );
				SendData( szResMsg );
			}
			*/

			// res
			return Res( 250, "HELP" );
		}
		return Res( 554, "critical error, low memory." );
	}

	return Res( 500, "ehlo requires domain address" );
}

//-----------------------------------------------------------------------------
//	@detail
//		cmdHELO
//	@param
//		pszCmdLine
//	@return
//		SUCCESS 250
//		ERROR   500, 501, 504, 421 
//	@comment
//		
//-----------------------------------------------------------------------------
int MySmtpSock::cmdHELO(char *pszCmdLine)
{
	char szDomain[MAX_DOMAIN_LEN255+1] = {0};
	if( GetArg( pszCmdLine, szDomain, MAX_DOMAIN_LEN255, 0 ) )
	{
		cmdRSET(NULL);

		if( m_vmHelo.Add( szDomain ) )
		{
			return Res( 250, g_conf.GetValueI(CONFIG_ME) );
		}
		return Res( 554, "critical error, low memory." );
	}

	return Res( 500, "ehlo requires domain address" );
}

//-----------------------------------------------------------------------------
//	@detail
//		cmdAUTH
//	@param
//		pszCmdLine
//	@return
//		SUCCESS 354
//		ERROR   451, 504
//	@comment
//		
//-----------------------------------------------------------------------------
int MySmtpSock::cmdAUTH(char *pszCmdLine)
{
	if( (CHANNEL_AUTH_LOGIN ==			m_nCannnelMode)
		|| (CHANNEL_AUTH_PLAIN ==		m_nCannnelMode)
		|| (CHANNEL_AUTH_CRAMMD5 ==		m_nCannnelMode)
//		|| (CHANNEL_AUTH_DIGESTMD5 ==	m_nCannnelMode)
		|| (0 != m_bAuthorized )
		)
	{
		return Res(503); // "Bad sequence of commands"
	}
	
	char szArgs[64] = {0};

	FREEIF( m_pszAuthKey );
	m_szUserName[0] = '\0';

	GetArg( pszCmdLine, szArgs, 63, 0 );

	// --------------------------------------------------------------
	// AUTH CRAM-MD5
	// --------------------------------------------------------------
	if( 0 == STRICMP( szArgs, "CRAM-MD5" ) )
	{
		if( 0 != STRICMP( "Yes", g_conf.GetValueI( CONFIG_USESMTPAUTHCRAMMD5 ) ) )
		{
			return Res(504);
		}

		// make key phrase
		char szAuthKey[MAX_DOMAIN_LEN255+32] = {0};

		sprintf( szAuthKey, "<%u.%u@%s>", GetCurrentProcessId(), GetTickCount(), g_conf.GetValueI(CONFIG_ME) );
		m_pszAuthKey = Base64Encode(szAuthKey);

		if( NULL == m_pszAuthKey )
		{
			return Res(451);
		}

		m_nCannnelMode = CHANNEL_AUTH_CRAMMD5;
		return Res( 334, m_pszAuthKey );
	}
	// --------------------------------------------------------------
	// AUTH PLAIN
	// --------------------------------------------------------------
	else if( 0 == STRICMP( szArgs, "PLAIN" ) )
	{
		if( 0 != STRICMP( "Yes", g_conf.GetValueI( CONFIG_USESMTPAUTHPLAIN ) ) )
		{
			return Res(504);
		}

		char szPhrase[256+32] = {0};
		if( 0 == GetArg( pszCmdLine, szPhrase, 256, 1 ) )
		{
			return  Res(535);
		}

		if( 0 == Base64Decode( szPhrase ) )
		{
			return Res(535);
		}

		// authorization identifier (userid) \0 authentication identifer (authid) \0 password \0

		char* pszUserID = szPhrase;
		while( NULL != *pszUserID++ );	// go to authid
		char* pszPasswd = pszUserID;
		while( NULL != *pszPasswd++ );	// go to passowrd
		
		MyAuth auth;
		if( 0 != auth.AuthPLAIN( pszUserID, pszPasswd ) )
		{
			m_bAuthorized = 1;
			return Res(235);
		}
		else
		{
			return Res(535);
		}
		
	}
	// --------------------------------------------------------------
	// AUTH LOGIN
	// --------------------------------------------------------------
	else if( 0 == STRICMP( szArgs, "LOGIN" ) )
	{
		if( 0 != STRICMP( "Yes", g_conf.GetValueI( CONFIG_USESMTPAUTHLOGIN ) ) )
		{
			return Res(504);
		}

		m_szUserName[0] = '\0';
		m_nCannnelMode = CHANNEL_AUTH_LOGIN;
		return Res( 334, "VXNlcm5hbWU6" );
	}

	return Res(504);

}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
int MySmtpSock::recvAuthCRAMMD5(char*pszCmdLine)
{
	m_nCannnelMode = CHANNEL_DIALOGUE;

	if( NULL == m_pszAuthKey )
	{
		return Res(451);
	}

	/* RFC2554 4. The AUTH command
			 The authentication protocol exchange consists of a series of
			 server challenges and client answers that are specific to the
			 authentication mechanism.  A server challenge, otherwise known
			 as a ready response, is a 334 reply with the text part
			 containing a BASE64 encoded string.  The client answer consists
			 of a line containing a BASE64 encoded string.  If the client
			 wishes to cancel an authentication exchange, it issues a line
			 with a single "*".  If the server receives such an answer, it
			 MUST reject the AUTH command by sending a 501 reply.
	*/

	if( 0 == STRCMP( "*", pszCmdLine ) )
	{
		return Res(501);
	}

	MyAuth auth;
	if( 0 == auth.AuthCRAMMD5( pszCmdLine, m_pszAuthKey, m_szUserName, MAX_LOCALUSERNAME_LEN64 ) )
	{
		/* RFC2554 4. The AUTH command
				 After an AUTH command has successfully completed, no more AUTH
				 commands may be issued in the same session.  After a successful
				 AUTH command completes, a server MUST reject any further AUTH
				 commands with a 503 reply.
		*/
		
		// login fail.
		m_szUserName[0] = '\0';
		char szDbgMsg[256];
		sockaddr_in saddr;
		int saddrsize = sizeof(sockaddr);
		getpeername( m_socket, (sockaddr*)&saddr, &saddrsize );
		char* pszRemoteHost = inet_ntoa(saddr.sin_addr);
		if( NULL == pszRemoteHost )
		{
			pszRemoteHost = "unknown";
		}
		sprintf( szDbgMsg, "SMTP-AUTH failed(%s).", pszRemoteHost );
		m_log.Warn( szDbgMsg );
		if( 0 == Base64Decode( pszCmdLine ) )
		{
			return Res(535);
		}

		m_log.Warn( pszCmdLine );

		FREEIF( m_pszAuthKey );
		Res(535);
		return 0;	// connection close.
	}

	FREEIF( m_pszAuthKey );

	char szDbgMsg[MAX_LOCALUSERNAME_LEN64+128] = {0};
	sprintf( szDbgMsg, "%s logged in.", m_szUserName );
	m_log.Info( szDbgMsg );
	
	m_szUserName[0] = '\0';
	m_bAuthorized = 1;
	return Res(235);
	
	return Res(235);
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
int MySmtpSock::recvAuthLOGIN(char*pszCmdLine)
{
	if( '\0' == m_szUserName[0] )
	{
		// user name
		if( 0 == Base64Decode(pszCmdLine) )
		{
			m_nCannnelMode = CHANNEL_DIALOGUE;
			return Res(535);
		}

		STRNCPY( m_szUserName, pszCmdLine, MAX_LOCALUSERNAME_LEN64 );
		return Res(334, "UGFzc3dvcmQ6");
	}
	// password
	m_nCannnelMode = CHANNEL_DIALOGUE;

	if( 0 == Base64Decode(pszCmdLine) )
	{
		m_szUserName[0] = '\0';
		return Res(535);
	}

	if( STRLEN( pszCmdLine ) > MAX_PASSWORD_LEN64 )
	{
		m_szUserName[0] = '\0';
		return Res(535);
	}

	// try auth
	MyAuth auth;
	if( 0 == auth.AuthPLAIN( m_szUserName, pszCmdLine ) )
	{
		m_szUserName[0] = '\0';
		return Res(535);
	}

	char szDbgMsg[MAX_LOCALUSERNAME_LEN64+128] = {0};
	sprintf( szDbgMsg, "%s logged in.", m_szUserName );
	m_log.Info( szDbgMsg );
	
	m_szUserName[0] = '\0';
	m_bAuthorized = 1;
	return Res(235);
}

// ============================================================================
// SUCCESS 250 
// FAILURE 552, 451, 452 
// ERROR   500, 501, 421
/* RFC821 3.1
         The first step in the procedure is the MAIL command.  The
         <reverse-path> contains the source mailbox.

            MAIL <SP> FROM:<reverse-path> <CRLF>

         This command tells the SMTP-receiver that a new mail
         transaction is starting and to reset all its state tables and
         buffers, including any recipients or mail data.  It gives the
         reverse-path which can be used to report errors.  If accepted,
         the receiver-SMTP returns a 250 OK reply.

         The <reverse-path> can contain more than just a mailbox.  The
         <reverse-path> is a reverse source routing list of hosts and
         source mailbox.  The first host in the <reverse-path> should be
         the host sending this command.
*/
/* RFC2821 3.3 Mail Transactions
   The first step in the procedure is the MAIL command.

      MAIL FROM:<reverse-path> [SP <mail-parameters> ] <CRLF>

   This command tells the SMTP-receiver that a new mail transaction is
   starting and to reset all its state tables and buffers, including any
   recipients or mail data.  The <reverse-path> portion of the first or
   only argument contains the source mailbox (between "<" and ">"
   brackets), which can be used to report errors (see section 4.2 for a
   discussion of error reporting).  If accepted, the SMTP server returns
   a 250 OK reply.  If the mailbox specification is not acceptable for
   some reason, the server MUST return a reply indicating whether the
   failure is permanent (i.e., will occur again if the client tries to
   send the same address again) or temporary (i.e., the address might be
   accepted if the client tries again later).  Despite the apparent
   scope of this requirement, there are circumstances in which the
   acceptability of the reverse-path may not be determined until one or
   more forward-paths (in RCPT commands) can be examined.  In those
   cases, the server MAY reasonably accept the reverse-path (with a 250
   reply) and then report problems after the forward-paths are received
   and examined.  Normally, failures produce 550 or 553 replies.

   Historically, the <reverse-path> can contain more than just a
   mailbox, however, contemporary systems SHOULD NOT use source routing
   (see appendix C).

   The optional <mail-parameters> are associated with negotiated SMTP
   service extensions (see section 2.2).
*/
// ============================================================================
//-----------------------------------------------------------------------------
//	@detail
//		cmdMAIL
//	@param
//		pszCmdLine
//	@return
//		SUCCESS 250 
//		FAILURE 552, 451, 452 
//		ERROR   500, 501, 421
//	@comment
//		
//-----------------------------------------------------------------------------
int MySmtpSock::cmdMAIL(char *pszCmdLine)
{
	m_hsRcptRemote.Clear();
	m_hsRcptLocal.Clear();
	m_vmFrom.Clear();

	if( 0 != STRNICMP( pszCmdLine, "MAIL FROM:", 10 ) )
		return Res( 501 );

	// ----------------------------------------------------
	// strage check
	DWORD dwStrage = g_conf.GetUINTValueI(CONFIG_STRAGEWATERMARK);
	if(0 == StrageCheck( g_conf.GetValueI(CONFIG_HOMEDIR), &dwStrage ))
	{
		return Res( 452 );
	}
	// ----------------------------------------------------

	const char* p = &pszCmdLine[10];
	char szMailAddr[MAX_MAILADDR_LEN256+16] = {0};

	// get and check address
	if( !GetMailAddressString( szMailAddr, MAX_MAILADDR_LEN256, p, g_conf.GetValueI(CONFIG_ME) ) )
		return Res( 501 );

	// mail from : address check
	if( STRLEN( szMailAddr ) > (MAX_MAILADDR_LEN256) )
		return Res( 501 );

	// mail from:<>
	if( '\0' == szMailAddr )
		m_vmFrom.Add("<>");
	else
	{
		if( NULL == STRCHR( szMailAddr, '@' ) )
		{
			STRCAT( szMailAddr, "@" );
			STRCAT( szMailAddr, g_conf.GetValueI(CONFIG_ME) );
		}

		m_vmFrom.Add( szMailAddr );
	}

	// hsRcptRemote's first record is "MAILFROM:<local-part>@<domain>"
	//m_hsRcptRemote.Set( &szMailAddr[0], "" );
	return Res( 250 );
}

// ============================================================================
// SUCCESS 250, 251
// FAILURE 550, 551, 552, 553, 450, 451, 452
// ERROR   500, 501, 503, 421
/* RFC821 3.1
         The second step in the procedure is the RCPT command.

            RCPT <SP> TO:<forward-path> <CRLF>

         This command gives a forward-path identifying one recipient.
         If accepted, the receiver-SMTP returns a 250 OK reply, and
         stores the forward-path.  If the recipient is unknown the
         receiver-SMTP returns a 550 Failure reply.  This second step of
         the procedure can be repeated any number of times.
         The <forward-path> can contain more than just a mailbox.  The
         <forward-path> is a source routing list of hosts and the
         destination mailbox.  The first host in the <forward-path>
         should be the host receiving this command.
*/
/* RFC2821 3.3 Mail Transactions
   The second step in the procedure is the RCPT command.

      RCPT TO:<forward-path> [ SP <rcpt-parameters> ] <CRLF>

   The first or only argument to this command includes a forward-path
   (normally a mailbox and domain, always surrounded by "<" and ">"
   brackets) identifying one recipient.  If accepted, the SMTP server
   returns a 250 OK reply and stores the forward-path.  If the recipient
   is known not to be a deliverable address, the SMTP server returns a
   550 reply, typically with a string such as "no such user - " and the
   mailbox name (other circumstances and reply codes are possible).
   This step of the procedure can be repeated any number of times.

   The <forward-path> can contain more than just a mailbox.
   Historically, the <forward-path> can be a source routing list of
   hosts and the destination mailbox, however, contemporary SMTP clients
   SHOULD NOT utilize source routes (see appendix C).  Servers MUST be
   prepared to encounter a list of source routes in the forward path,
   but SHOULD ignore the routes or MAY decline to support the relaying
   they imply.  Similarly, servers MAY decline to accept mail that is
   destined for other hosts or systems.  These restrictions make a
   server useless as a relay for clients that do not support full SMTP
   functionality.  Consequently, restricted-capability clients MUST NOT
   assume that any SMTP server on the Internet can be used as their mail
   processing (relaying) site.  If a RCPT command appears without a
   previous MAIL command, the server MUST return a 503 "Bad sequence of
   commands" response.  The optional <rcpt-parameters> are associated
   with negotiated SMTP service extensions (see section 2.2).
*/
 // ============================================================================
//-----------------------------------------------------------------------------
//	@detail
//		cmdRCPT
//	@param
//		pszCmdLine
//	@return
//		success not 0
//		fail	0
//	@comment
//		SUCCESS 250, 251
//		FAILURE 550, 551, 552, 553, 450, 451, 452
//		ERROR   500, 501, 503, 421
//-----------------------------------------------------------------------------
int MySmtpSock::cmdRCPT(char *pszCmdLine)
{
	char szMailAddress[MAX_MAILADDR_LEN256 + 1] = {0};
	char* pszAtMark = NULL;

	// ----------------------------------------------------
	// strage check
	DWORD dwStrage = g_conf.GetUINTValueI(CONFIG_STRAGEWATERMARK);
	if(0 == StrageCheck( g_conf.GetValueI(CONFIG_HOMEDIR), &dwStrage ))
	{
		return Res( 452 );
	}
	// ----------------------------------------------------
	// 503 MAIL first (#5.5.1)
	if( m_vmFrom.IsEmpty() )
	{
		return Res( 503, "Bad sequence of commands. MAIL first." );
	}

	if( 0 != STRNICMP( pszCmdLine, "RCPT TO:", 8 ) )
	{
		return Res( 501, "Command syntax error. RCPT <SP> TO:<forward-path> <CRLF>" );
	}


	if( !GetMailAddressString( szMailAddress, MAX_MAILADDR_LEN256, &pszCmdLine[8], g_conf.GetValueI(CONFIG_ME) ) )
	{
		return Res( 553 );
	}

	// No domain address seems local address
	if( NULL == ( pszAtMark = STRRCHR( szMailAddress, '@' )) )
	{
		const char* pDomainName = g_conf.GetValueI(CONFIG_ME);
		if( STRLEN( szMailAddress ) + STRLEN( pDomainName ) + 1 > MAX_MAILADDR_LEN256 )
			return Res( 553 );
		
		STRCAT( szMailAddress, "@" );
		STRCAT( szMailAddress, pDomainName );
	}

	/*
	4.5.3.1 Size limits and minimums
	local-part : The maximum total length of a user name or other local-part is 64 characters.
	domain : The maximum total length of a domain name or number is 255 characters.
	path : The maximum total length of a reverse-path or forward-path is 256 characters
	       (including the punctuation and element separators).
	*/
	pszAtMark = STRRCHR( szMailAddress, '@' );

	if( ( pszAtMark - &szMailAddress[0] ) > MAX_LOCALUSERNAME_LEN64
		|| STRLEN( pszAtMark ) > (MAX_DOMAIN_LEN255+1)
		|| STRLEN( szMailAddress ) > MAX_MAILADDR_LEN256 )
	{
		return Res( 553 );
	}

	// ----------------------------------------------------
	// POP before SMTP
	// ----------------------------------------------------
	const char* pszDomain = &pszAtMark[1];
	MyAuthIPList ip;
	sockaddr_in saddr;
	int saddrsize = sizeof(sockaddr);
	
	getpeername( m_socket, (sockaddr*)&saddr, &saddrsize );


	// Is exist in recpthosts?
	// |
	// +->NO: Is my domain?
	// |    |
	// |    +->NO: Pop before smtp is on?
	// |    |    |
	// |    |    +->ON : IP check
	// |    |    |    |
	// |    |    |    +->FAILED: [FAILED]
	// |    |    |
	//[YES][YES][OFF]    
	// |    |    |
	// +----+----+
	//      |
	//  [SUCCESS]


	// Recpthosts check.
	SEEKKEY skRcptHost = g_rcpthosts.SeekToFirst();
	while( NULL != skRcptHost )
	{
		if( 0 != ReverseMatch( szMailAddress, skRcptHost ) )
			break;

		skRcptHost = g_rcpthosts.SeekNext(skRcptHost);
	}

	// to my domain?
	if( NULL == skRcptHost && 0 == ReverseMatch( szMailAddress, g_conf.GetValueI(CONFIG_ME) ))
	{
		// pop before smtp
		if( 0 != STRNICMP( "Off", g_conf.GetValueI(CONFIG_POPBEFORESMTP), 3 ) )
		{
			// pop before smtp on

			if( 0 == ip.Check( g_conf.GetValueI(CONFIG_POPAUTHDIR), inet_ntoa(saddr.sin_addr), g_conf.GetUINTValueI(CONFIG_POPAUTHTIMEOUT) ) )
			{
				// ip check failed

				return Res(550, "POP3 auth first before RCPT.");
			}
		}
		// pop before smtp off
	}
	// ----------------------------------------------------
	// Add mail address to m_hsRcpt
	// ----------------------------------------------------
	Usr usr;
	switch( usr.IsMyUser( szMailAddress ) )
	{
	case USR_OTHER_DOMAIN:	//0
		// Authentication check
		if( 0 == m_bAuthorized )
		{
			return Res( 530, "SMTP auth first before RCPT." );
		}
		m_hsRcptRemote.Set( szMailAddress, "" );
		break;

	case USR_LOCAL_DOMAIN:	//1
		m_hsRcptLocal.Set( szMailAddress, "" );
		break;

	case USR_ERR_BADPARAM:	//-3
	case USR_ERR_MEMORY:	//-2
	case USR_ERR_NAME:		//-1
	default:
		return Res( 550, "Mail address check failed." );
	}
	
	// rcpt list
	return Res( 250, "ok" );
}

/* RFC822 4.3.2.  RECEIVED

        A copy of this field is added by each transport  service  that
        relays the message.  The information in the field can be quite
        useful for tracing transport problems.

        The names of the sending  and  receiving  hosts  and  time-of-
        receipt may be specified.  The "via" parameter may be used, to
        indicate what physical mechanism the message  was  sent  over,
        such  as  Arpanet or Phonenet, and the "with" parameter may be
        used to indicate the mail-,  or  connection-,  level  protocol
        that  was  used, such as the SMTP mail protocol, or X.25 tran-
        sport protocol.

        Note:  Several "with" parameters may  be  included,  to  fully
               specify the set of protocols that were used.

        Some transport services queue mail; the internal message iden-
        tifier that is assigned to the message may be noted, using the
        "id" parameter.  When the  sending  host  uses  a  destination
        address specification that the receiving host reinterprets, by
        expansion or transformation, the receiving host  may  wish  to
        record  the original specification, using the "for" parameter.
        For example, when a copy of mail is sent to the  member  of  a
        distribution  list,  this  parameter may be used to record the
        original address that was used to specify the list.
*/

// ============================================================================
// SUCCESS 354 GO AHEAD
//         250 OK
// FAILURE 552, 554, 451, 452
// FAILURE 451, 554
// ERROR   500, 501, 503, 421
/* RFC821 3.1
         The third step in the procedure is the DATA command.

            DATA <CRLF>

         If accepted, the receiver-SMTP returns a 354 Intermediate reply
         and considers all succeeding lines to be the message text.
         When the end of text is received and stored the SMTP-receiver
         sends a 250 OK reply.

         Since the mail data is sent on the transmission channel the end
         of the mail data must be indicated so that the command and
         reply dialog can be resumed.  SMTP indicates the end of the
         mail data by sending a line containing only a period.  A
         transparency procedure is used to prevent this from interfering
         with the user's text (see Section 4.5.2).

            Please note that the mail data includes the memo header
            items such as Date, Subject, To, Cc, From [2].

         The end of mail data indicator also confirms the mail
         transaction and tells the receiver-SMTP to now process the
         stored recipients and mail data.  If accepted, the
         receiver-SMTP returns a 250 OK reply.  The DATA command should
         fail only if the mail transaction was incomplete (for example,
         no recipients), or if resources are not available.

      The above procedure is an example of a mail transaction.  These
      commands must be used only in the order discussed above.
      Example 1 (below) illustrates the use of these commands in a mail
      transaction.
*/
/* RFC2822 2.3. Body

   The body of a message is simply lines of US-ASCII characters.  The
   only two limitations on the body are as follows:

   - CR and LF MUST only occur together as CRLF; they MUST NOT appear
     independently in the body.

   - Lines of characters in the body MUST be limited to 998 characters,
     and SHOULD be limited to 78 characters, excluding the CRLF.
*/
/* RFC2821 3.3 Mail Transactions
   The third step in the procedure is the DATA command (or some
   alternative specified in a service extension).

      DATA <CRLF>

   If accepted, the SMTP server returns a 354 Intermediate reply and
   considers all succeeding lines up to but not including the end of
   mail data indicator to be the message text.  When the end of text is
   successfully received and stored the SMTP-receiver sends a 250 OK
   reply.

   Since the mail data is sent on the transmission channel, the end of
   mail data must be indicated so that the command and reply dialog can
   be resumed.  SMTP indicates the end of the mail data by sending a
   line containing only a "." (period or full stop).  A transparency
   procedure is used to prevent this from interfering with the user's
   text (see section 4.5.2).

   The end of mail data indicator also confirms the mail transaction and
   tells the SMTP server to now process the stored recipients and mail
   data.  If accepted, the SMTP server returns a 250 OK reply.  The DATA
   command can fail at only two points in the protocol exchange:

   -  If there was no MAIL, or no RCPT, command, or all such commands
      were rejected, the server MAY return a "command out of sequence"
      (503) or "no valid recipients" (554) reply in response to the DATA
      command.  If one of those replies (or any other 5yz reply) is
      received, the client MUST NOT send the message data; more
      generally, message data MUST NOT be sent unless a 354 reply is
      received.

   -  If the verb is initially accepted and the 354 reply issued, the
      DATA command should fail only if the mail transaction was
      incomplete (for example, no recipients), or if resources were
      unavailable (including, of course, the server unexpectedly
      becoming unavailable), or if the server determines that the
      message should be rejected for policy or other reasons.

   However, in practice, some servers do not perform recipient
   verification until after the message text is received.  These servers
   SHOULD treat a failure for one or more recipients as a "subsequent
   failure" and return a mail message as discussed in section 6.  Using
   a "550 mailbox not found" (or equivalent) reply code after the data
   are accepted makes it difficult or impossible for the client to
   determine which recipients failed.

   When RFC 822 format [7] is being used, the mail data include the memo
   header items such as Date, Subject, To, Cc, From [32].  Server SMTP
   systems SHOULD NOT reject messages based on perceived defects in the
   RFC 822 or MIME [12] message header or message body.  In particular,
   they MUST NOT reject messages in which the numbers of Resent-fields
   do not match or Resent-to appears without Resent-from and/or Resent-
   date.
*/
/*RFC2821 4.1.1.4 DATA (DATA)

   The receiver normally sends a 354 response to DATA, and then treats
   the lines (strings ending in <CRLF> sequences, as described in
   section 2.3.7) following the command as mail data from the sender.
   This command causes the mail data to be appended to the mail data
   buffer.  The mail data may contain any of the 128 ASCII character
   codes, although experience has indicated that use of control
   characters other than SP, HT, CR, and LF may cause problems and
   SHOULD be avoided when possible.

   The mail data is terminated by a line containing only a period, that
   is, the character sequence "<CRLF>.<CRLF>" (see section 4.5.2).  This
   is the end of mail data indication.  Note that the first <CRLF> of
   this terminating sequence is also the <CRLF> that ends the final line
   of the data (message text) or, if there was no data, ends the DATA
   command itself.  An extra <CRLF> MUST NOT be added, as that would
   cause an empty line to be added to the message.  The only exception
   to this rule would arise if the message body were passed to the
   originating SMTP-sender with a final "line" that did not end in
   <CRLF>; in that case, the originating SMTP system MUST either reject
   the message as invalid or add <CRLF> in order to have the receiving
   SMTP server recognize the "end of data" condition.

   The custom of accepting lines ending only in <LF>, as a concession to
   non-conforming behavior on the part of some UNIX systems, has proven
   to cause more interoperability problems than it solves, and SMTP
   server systems MUST NOT do this, even in the name of improved
   robustness.  In particular, the sequence "<LF>.<LF>" (bare line
   feeds, without carriage returns) MUST NOT be treated as equivalent to
   <CRLF>.<CRLF> as the end of mail data indication.

   Receipt of the end of mail data indication requires the server to
   process the stored mail transaction information.  This processing
   consumes the information in the reverse-path buffer, the forward-path
   buffer, and the mail data buffer, and on the completion of this
   command these buffers are cleared.  If the processing is successful,
   the receiver MUST send an OK reply.  If the processing fails the
   receiver MUST send a failure reply.  The SMTP model does not allow
   for partial failures at this point: either the message is accepted by
   the server for delivery and a positive response is returned or it is
   not accepted and a failure reply is returned.  In sending a positive
   completion reply to the end of data indication, the receiver takes
   full responsibility for the message (see section 6.1).  Errors that
   are diagnosed subsequently MUST be reported in a mail message, as
   discussed in section 4.4.

   When the SMTP server accepts a message either for relaying or for
   final delivery, it inserts a trace record (also referred to
   interchangeably as a "time stamp line" or "Received" line) at the top
   of the mail data.  This trace record indicates the identity of the
   host that sent the message, the identity of the host that received
   the message (and is inserting this time stamp), and the date and time
   the message was received.  Relayed messages will have multiple time
   stamp lines.  Details for formation of these lines, including their
   syntax, is specified in section 4.4.

   Additional discussion about the operation of the DATA command appears
   in section 3.3.

   Syntax:
      "DATA" CRLF
*/
//-----------------------------------------------------------------------------
//	@detail
//		cmdDATA
//	@param
//		pszCmdLine
//	@return
//		SUCCESS	354
//		ERROR   451 452 503
//	@comment
//		
//-----------------------------------------------------------------------------
int MySmtpSock::cmdDATA(char *pszCmdLine)
{
	m_nReceivedHeaderCount = 0;

	// ----------------------------------------------------
	// strage check
	DWORD dwStrage = g_conf.GetUINTValueI(CONFIG_STRAGEWATERMARK);

	if(0 == StrageCheck( g_conf.GetValueI(CONFIG_HOMEDIR), &dwStrage ))
	{
		return Res( 452 );
	}
	// ----------------------------------------------------

	if( 0 == m_hsRcptRemote.GetLength() && 0 == m_hsRcptLocal.GetLength() )
	{
		return Res( 503 );
	}

	if( 0 != m_TmpFile.Create( g_conf.GetValueI(CONFIG_TMPDIR) ) )
	{
		if( 0 != AddTraceFields() )
		{
			m_nCannnelMode = CHANNEL_DATA;
			return Res( 354 );
		}
	}
	return Res( 451, "Requested action aborted: deliver file create failed." );
}

//-----------------------------------------------------------------------------
//	@detail
//		recvDATA
//	@param
//		pszCmdLine
//	@return
//		SUCCESS	250
//		ERROR   
//	@comment
//		
//-----------------------------------------------------------------------------
int MySmtpSock::recvDATA(char *pszCmdLine)
{
	// single period is end sign.
	if( pszCmdLine[0] == '.' )
	{
		if( pszCmdLine[1] == '\0' )
		{
			m_nCannnelMode = CHANNEL_DIALOGUE;

			// 'data' complete.

			m_log.Info( "message received complete. delivery start." );
			Usr usr;
			char szTmpFileName[MAX_PATH] = {0};
			m_TmpFile.GetFileName( szTmpFileName, MAX_PATH );

			if( 0 == usr.DeliveryFile( szTmpFileName, &m_hsRcptLocal, &m_hsRcptRemote, &m_log ) )
			{
				m_TmpFile.Delete();
				/* the forward-path buffer, and the mail data buffer,
					and on the completion of this command these buffers are cleared.*/
				m_vmFrom.Clear();
				m_hsRcptRemote.Clear();
				m_hsRcptLocal.Clear();
				return Res( 552, "delivery failed." );
			}

			m_TmpFile.Delete();

			if( 0 != m_hsRcptRemote.GetLength() )
			{
				StartProcess(SENDERSMTP, szTmpFileName);
			}

			m_vmFrom.Clear();
			m_hsRcptRemote.Clear();
			m_hsRcptLocal.Clear();

			return Res( 250 );
		}
		else if(pszCmdLine[1] == '.')
		{
			// byte stuff
			pszCmdLine++;
		}
	}
	/* RFC2821 6.2 Loop Detection
	   Simple counting of the number of "Received:" headers in a message has
	   proven to be an effective, although rarely optimal, method of
	   detecting loops in mail systems.  SMTP servers using this technique
	   SHOULD use a large rejection threshold, normally at least 100
	   Received entries.  Whatever mechanisms are used, servers MUST contain
	   provisions for detecting and stopping trivial loops.

	*/
	if( 0 == STRNICMP( pszCmdLine, "received:", 9 ) )
	{
		if( m_nReceivedHeaderCount++ >100 )
		{
			m_TmpFile.Delete();
			m_log.Warn( "Received header over 100 count, may be loop mail." );
			m_nCannnelMode = CHANNEL_ERRDATA_552;
			return 250;	// continue...;
		}
	}

	if( 0 == m_TmpFile.AppendLine( pszCmdLine )	)
	{
		m_log.Err( "::WriteFile failed at recvDATA." );
		m_nCannnelMode = CHANNEL_ERRDATA_451;
		m_TmpFile.Delete();

		return 250;	// continue...;
	}

	// check file(data) size
	if(m_unMaxMailSize < m_TmpFile.GetFileSize() )
	{
		m_TmpFile.Delete();
		m_log.Info( "over size mail recved." );
		m_nCannnelMode = CHANNEL_ERRDATA_552;
		return 250;	// continue...;
	}

	return 250;	// continue...;
}

//-----------------------------------------------------------------------------
//	@detail
//		recvInvalidData
//	@param
//		pszCmdLine
//	@return
//		SUCCESS	250
//	@comment
//		
//-----------------------------------------------------------------------------
int MySmtpSock::recvInvalidData(char *pszCmdLine)
{
	if( pszCmdLine[0] == '.' )
	{
		if( pszCmdLine[1] == '\0' )
		{
			m_vmFrom.Clear();
			m_hsRcptRemote.Clear();
			m_hsRcptLocal.Clear();

			switch( m_nCannnelMode )
			{
			case CHANNEL_ERRDATA_552:	Res( 552 ); break;
			case CHANNEL_ERRDATA_451:	Res( 451 ); break;
			default:					Res( 554 );	break;
			}
			m_nCannnelMode = CHANNEL_DIALOGUE;
		}
	}
	return 250;	// continue...;
}

// ============================================================================
/* RFC822 4.3.2.  RECEIVED
        A copy of this field is added by each transport  service  that
        relays the message.  The information in the field can be quite
        useful for tracing transport problems.

        The names of the sending  and  receiving  hosts  and  time-of-
        receipt may be specified.  The "via" parameter may be used, to
        indicate what physical mechanism the message  was  sent  over,
        such  as  Arpanet or Phonenet, and the "with" parameter may be
        used to indicate the mail-,  or  connection-,  level  protocol
        that  was  used, such as the SMTP mail protocol, or X.25 tran-
        sport protocol.

        Note:  Several "with" parameters may  be  included,  to  fully
               specify the set of protocols that were used.

        Some transport services queue mail; the internal message iden-
        tifier that is assigned to the message may be noted, using the
        "id" parameter.  When the  sending  host  uses  a  destination
        address specification that the receiving host reinterprets, by
        expansion or transformation, the receiving host  may  wish  to
        record  the original specification, using the "for" parameter.
        For example, when a copy of mail is sent to the  member  of  a
        distribution  list,  this  parameter may be used to record the
        original address that was used to specify the list.
*/
/* RFC2822 A.4. Messages with trace fields
   As messages are sent through the transport system as described in
   [RFC2821], trace fields are prepended to the message.  The following
   is an example of what those trace fields might look like.  Note that
   there is some folding white space in the first one since these lines
   can be long.

----
Received: from x.y.test
   by example.net
   via TCP
   with ESMTP
   id ABC12345
   for <mary@example.net>;  21 Nov 1997 10:05:43 -0600
Received: from machine.example by x.y.test; 21 Nov 1997 10:01:22 -0600
From: John Doe <jdoe@machine.example>
To: Mary Smith <mary@example.net>
Subject: Saying Hello
Date: Fri, 21 Nov 1997 09:55:06 -0600
Message-ID: <1234@local.machine.example>

This is a message just to say hello.
So, "Hello".
*/
// ============================================================================
//-----------------------------------------------------------------------------
//	@detail
//		AddTraceFields
//	@param
//		nothing
//	@return
//		1:		sccess
//		0:		failed
//	@comment
//		
//-----------------------------------------------------------------------------
int MySmtpSock::AddTraceFields()
{
	MyTime tm;
	tm.SetLocalTime();

	const char* pszReturnPath = (const char*)m_vmFrom.GetBuffer();
	if( NULL == pszReturnPath )
	{
		return 0;
	}
	else if( 0 >= STRLEN( pszReturnPath ) )
	{
		return 0;
	}

	// return-path is "<>" ?
	if( STRSTR( pszReturnPath, "<>" ) )
		pszReturnPath = "";

	char szIPAddr[32] = {0};
	unsigned char* pIP =(unsigned char*)&m_pAT->saRemoteHost.sin_addr.S_un.S_un_b;
	sprintf( szIPAddr, "%u.%u.%u.%u", pIP[0], pIP[1], pIP[2], pIP[3] );
	char* szRemoteHostName;
	hostent* pHE = gethostbyaddr( (const char*)pIP, sizeof(IN_ADDR), AF_INET );
	if( NULL != pHE && NULL != pHE->h_name )
	{
		szRemoteHostName = pHE->h_name;
	}
	else
	{
		szRemoteHostName = "unknown"; 
	}

	unsigned int nStrLen;
	nStrLen = 0
		+ STRLEN( pszReturnPath )
		+ STRLEN( szIPAddr )
		+ STRLEN( szRemoteHostName )
		+ m_vmHelo.GetLength()
		+ STRLEN( g_conf.GetValueI(CONFIG_ME) )
		+ STRLEN( tm.GetLocalFormatRfc1123() )
		+256;

	char* pszBuf = (char*)MALLOC( nStrLen+1 );
	if( NULL != pszBuf )
	{
		sprintf( pszBuf, 
			"Return-Path: <%s>\r\n"
			"Received: from %s(%s)\r\n"
			"    by %s; %s\r\n"
			, pszReturnPath
			, szRemoteHostName, szIPAddr
			, g_conf.GetValueI(CONFIG_ME), tm.GetLocalFormatRfc1123()
			);

		if( 0 == m_TmpFile.Append( pszBuf )	)
		{
			m_log.Err( "::WriteFile failed at AddTraceFields." );
		}
	}
	else
	{
		FREEIF( pszBuf );
		return 0;
	}

	return 1;
}

// ============================================================================
// SUCCESS 250 
// ERROR   500, 501, 504, 421 
// ============================================================================
//-----------------------------------------------------------------------------
//	@detail
//		cmdRSET
//	@param
//		pszCmdLine
//	@return
//		SUCCESS	250
//	@comment
//		
//-----------------------------------------------------------------------------
int MySmtpSock::cmdRSET(char *pszCmdLine)
{
	m_vmHelo.Clear();
	m_hsRcptRemote.Clear();
	m_hsRcptLocal.Clear();
	m_vmFrom.Clear();
	m_nCannnelMode = CHANNEL_DIALOGUE;
	FREEIF( m_pszAuthKey );

	if( NULL == pszCmdLine )
		return 0;

	return Res( 250 );
}

// ============================================================================
// SUCCESS 252(always)
// ============================================================================
int MySmtpSock::cmdVRFY(char *pszCmdLine)
{
	return Res( 252, "Cannot VRFY user." );
}

// ============================================================================
// SUCCESS 550(always)
// ============================================================================
int MySmtpSock::cmdEXPN(char *pszCmdLine)
{
	return Res( 550, "Access denied to you." );
}

// ============================================================================
// SUCCESS 250 
// ERROR   500 
// ============================================================================
int MySmtpSock::cmdQUIT(char *pszCmdLine)
{
	Res( 221, g_conf.GetValueI(CONFIG_ME) );
	return 0;
}

//-----------------------------------------------------------------------------
//	@detail
//		Res
//	@param
//		nCode
//		pMes
//	@return
//		0:		socket error	
//		1:		...
//	@comment
//		
//-----------------------------------------------------------------------------
int MySmtpSock::Res(int nCode, const char *pMes /*= NULL*/)
{
	if( NULL == pMes )
	{
		switch( nCode )
		{
//		case 211:	pMes=SMTPRES221;		break;
		case 214:	pMes=SMTPRES214;		break;
//		case 220:	pMes=SMTPRES220;		break;
//		case 221:	pMes=SMTPRES221;		break;
		case 235:	pMes=SMTPRES235;		break;
		case 250:	pMes=SMTPRES250;		break;
		case 251:	pMes=SMTPRES251;		break;

		case 354:	pMes=SMTPRES354;		break;

		case 421:	pMes=SMTPRES421;		break;
		case 450:	pMes=SMTPRES450;		break;
		case 451:	pMes=SMTPRES451;		break;
		case 452:	pMes=SMTPRES452;		break;

		case 500:	pMes=SMTPRES500;		break;
		case 501:	pMes=SMTPRES501;		break;
		case 502:	pMes=SMTPRES502;		break;
		case 503:	pMes=SMTPRES503;		break;
		case 504:	pMes=SMTPRES504;		break;
		case 530:	pMes=SMTPRES530;		break;
		case 535:	pMes=SMTPRES535;		break;
		case 550:	pMes=SMTPRES550;		break;
		case 551:	pMes=SMTPRES551;		break;
		case 552:	pMes=SMTPRES552;		break;
		case 553:	pMes=SMTPRES553;		break;
		case 554:	pMes=SMTPRES554;		break;
		default:	pMes="";				break;
		}
	}
	
	char szMessage[81] = {0};	// max message length is 80
	char szCode[96] = {0};		// max send message length is 95
	
	// chop message at 80
	STRNCPY( szMessage, pMes, 80 );

	sprintf( szCode, "%d %s", nCode, szMessage );
	m_log.Dbg( szCode );
	STRCAT( szCode, "\r\n" );

	if( SOCKET_ERROR == SendData( szCode ) )
		return 0;
	return nCode;
}

