/* =========================================================================
	This library is part of IshiSoft application.
	------------------------------------------------------------------------
	author: Yuichiro Ishitani
			http://www001.upp.so-net.ne.jp/ishi/index.html
  ========================================================================== */
#include "stdafx.h"
#include "resource.h"
#include "sparrow_version.h"
#include "threadtype.h"
#include "MyLog.h"
#include "MyConf.h"
#include "str.h"
#include "MainThread.h"
#include "staticvariable.h"
#include "daemon.h"
#include "commonhelper.h"

#include "MyTmpFileMgr.h"
#include "Usr.h"
#include "MyTime.h"


#if !defined(_WIN32_WCE)
#include "shellapi.h"
#include "stdio.h"
#endif

#define MY_NOTIFYICON	(WM_APP + 1000)
#define MAX_LOADSTRING	100

HINSTANCE		g_hInst;
LISTENTHREAD	g_thSMTPListen = {0};
LISTENTHREAD	g_thPOP3Listen = {0};
MyConf			g_conf;
HANDLE			g_Mutex = NULL;
MyLog			g_log;
MyHashFile		g_rcpthosts;

#define	WMTIMER_SPLASH				1
#define WMTIMER_DAEMON_CLEANUP		2
#define WMTIMER_DAEMON_SEND_SMTP	3

static BOOL CALLBACK DlgProc(HWND hDlg, UINT Msg, WPARAM wParam, LPARAM lParam);
static int StartServerThread(LISTENTHREAD* plTh, SERVERTYPE st);
static void StopServerThread(LISTENTHREAD* plTh);
static void MakeStatusMail();

//-----------------------------------------------------------------------------
//	@detail
//		WinMain function.
//	@param
//	@return
//-----------------------------------------------------------------------------
#if !defined(_WIN32_WCE)
int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow )
#else
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
#endif
{
	//------------ Prevent multiple instance  ------------------
	TCHAR	szAppName[MAX_LOADSTRING];
	LoadString(hInstance, IDS_APP_TITLE, szAppName, MAX_LOADSTRING);

	HANDLE g_Mutex = CreateMutex(NULL,FALSE,L"____SPARROW_SMTP_POP3_BY_ISHISOFT___");
	if( GetLastError() == ERROR_ALREADY_EXISTS )
	{
		
		HWND hwndPrev = FindWindow( L"Dialog" , szAppName );
		if( hwndPrev )
		{
			ShowWindow(hwndPrev, SW_SHOWNORMAL);
			SetForegroundWindow(hwndPrev);
		}
		CloseHandle( g_Mutex );
		return 0; 
	}

	g_log.SetFacility(LOGFACILITY_MAIL);
	g_log.SetTag("MAILD");

	// start logger
	StartProcess( LOGGER, NULL, SW_SHOWNORMAL );
	g_log.Info( SPARROW_VERSION );

	// load rcpthosts
	if( 0 == g_rcpthosts.LoadFile(g_conf.GetValueI(CONFIG_RCPTHOSTS)) )
	{
		g_log.Warn("load rcpthosts file failed.");
	}

	MakeStatusMail();

	g_hInst = hInstance;
	DialogBox(hInstance, MAKEINTRESOURCE(IDD_MAINDLG), NULL, DlgProc);
	return 0;

}

//-----------------------------------------------------------------------------
//	@detail
//		Main dialog box
//	@param
//	@return
//-----------------------------------------------------------------------------
static BOOL CALLBACK DlgProc(HWND hDlg, UINT Msg, WPARAM wParam, LPARAM lParam)
{
	static	NOTIFYICONDATA	nIcon;

	switch (Msg) 
	{
	case WM_INITDIALOG:
		TCHAR	szAppName[MAX_LOADSTRING];
		LoadString(g_hInst, IDS_APP_TITLE, szAppName, MAX_LOADSTRING);

		SetWindowText( hDlg, szAppName );
	
		// ------------------------------------------------------
		// Make Tasktray icon
		nIcon.cbSize			= sizeof(nIcon);
		nIcon.hIcon				= (HICON)LoadImage(g_hInst,MAKEINTRESOURCE(IDI_IMAIN),IMAGE_ICON, 16,16,0);
		nIcon.hWnd				= hDlg;
		nIcon.uCallbackMessage	= MY_NOTIFYICON;
		nIcon.uFlags			= NIF_MESSAGE | NIF_ICON;
		nIcon.uID				= 1;

		Shell_NotifyIcon(NIM_ADD, &nIcon);

		// ------------------------------------------------------
		// Position set
		RECT rt, rt1;
		if (GetWindowRect(hDlg, &rt1))
		{
			int CenterPosX, CenterPosY;
			SystemParametersInfo(SPI_GETWORKAREA, 0, &rt, FALSE);
			CenterPosX		= (rt.right - rt.left - (rt1.right - rt1.left))/2;
			CenterPosY		= (rt.bottom - rt.top - (rt1.bottom - rt1.top))/2;
			
			if (CenterPosX < 0) CenterPosX = 0;
			if (CenterPosY < 0) CenterPosY = 0;
			SetWindowPos(hDlg, HWND_NOTOPMOST, CenterPosX, CenterPosY, 0, 0, SWP_NOZORDER | SWP_NOSIZE);
		}
		// hide window
		SetTimer( hDlg, WMTIMER_SPLASH, 3000, NULL );

		// ======================================================
		// Server start
		// ------------------------------------------------------
		if( 0 == StartServerThread(&g_thSMTPListen, SERVERTYPE_SMTP) )
		{
			g_log.Err( "SMTP thread start failed." );
		}
		if( 0 == StartServerThread(&g_thPOP3Listen, SERVERTYPE_POP3) )
		{
			g_log.Err( "SMTP thread start failed." );
		}
		
		// start daemon
		SetTimer( hDlg, WMTIMER_DAEMON_CLEANUP, g_conf.GetUINTValueI(CONFIG_CLEANUPTIMER) * 1000, DaemonCleanUp );
		SetTimer( hDlg, WMTIMER_DAEMON_SEND_SMTP, g_conf.GetUINTValueI(CONFIG_SENDMAILROUNDTIMER) * 1000, DaemonSenderSMTP );

		DaemonCleanUp( hDlg, WM_TIMER, WMTIMER_DAEMON_CLEANUP, GetTickCount() );
		DaemonSenderSMTP( hDlg, WM_TIMER, WMTIMER_DAEMON_SEND_SMTP, GetTickCount() );

		return TRUE;
	
	case WM_TIMER:
		if( wParam == WMTIMER_SPLASH )
		{
			ShowWindow(hDlg, SW_HIDE);
			KillTimer( hDlg, WMTIMER_SPLASH );
			return TRUE;
		}
		break;


	case MY_NOTIFYICON:
		switch(lParam)
		{
		case WM_LBUTTONDOWN:
		{
			if(IsWindowVisible(hDlg) )
			{
				ShowWindow(hDlg, SW_HIDE);
			}
			else
			{
				ShowWindow(hDlg, SW_SHOWNORMAL);
				SetForegroundWindow(hDlg);
			}
			break;
		}
		default:
			break;
		}
		break;

	case WM_COMMAND:
		return FALSE;
	case WM_CLOSE:
		EndDialog( hDlg, 1 );
		return TRUE;

	case WM_DESTROY:
		StopServerThread(&g_thSMTPListen);
		StopServerThread(&g_thPOP3Listen);

		KillTimer( hDlg, WMTIMER_DAEMON_CLEANUP );
		KillTimer( hDlg, WMTIMER_DAEMON_SEND_SMTP );

		if(g_Mutex){
			ReleaseMutex( g_Mutex );
			CloseHandle( g_Mutex );
		}

		Shell_NotifyIcon(NIM_DELETE, &nIcon);

		return 0;
	default:
		break;
	}
	return FALSE;
}

// ----------------------------------------------------------------------------
// StartMainThread
// ----------------------------------------------------------------------------
static int StartServerThread(LISTENTHREAD* plTh, SERVERTYPE st)
{
	if( NULL != plTh->hThread )
		return 0;
	
	plTh->bStop = FALSE;
	plTh->servertype = st;

	if( plTh->servertype == SERVERTYPE_SMTP )
	{
		plTh->nPort = ATOUI(g_conf.GetValueI(CONFIG_SMTPPORT));
		if( plTh->nPort <=0 )
			return 0;

		plTh->nServers = ATOUI(g_conf.GetValueI(CONFIG_SMTPSERVERS));
		if( plTh->nServers <=0 )
		{
			g_log.Info( "number of SMTP server is 0." );
			return 1;
		}
	}
	else if( plTh->servertype == SERVERTYPE_POP3 )
	{
		plTh->nPort = ATOUI(g_conf.GetValueI(CONFIG_POP3PORT));
		if( plTh->nPort <=0 )
			return 0;

		plTh->nServers = ATOUI(g_conf.GetValueI(CONFIG_POP3SERVERS));
		if( plTh->nServers <=0 )
		{
			g_log.Info( "number of POP3 server is 0." );
			return 1;
		}
	}
	else
	{
		return 0;
	}

	plTh->hThread = CreateThread( NULL, 0, (LPTHREAD_START_ROUTINE)thLiten, plTh, 0, &(plTh->dwThreadID) );

	if( NULL == plTh->hThread )
	{
		return 0;
	}

	return 1;
}

// ----------------------------------------------------------------------------
// StopMainThread
// ----------------------------------------------------------------------------
static void StopServerThread(LISTENTHREAD* plTh)
{
	plTh->bStop = TRUE;
	if( WAIT_OBJECT_0 != WaitForSingleObject( plTh->hThread, 30000 ) )
	{
		TerminateThread( plTh->hThread, 0 );
		g_log.Dbg("Listen thread was force terminated.");
	}
	else
	{
		g_log.Dbg("Listen thread was terminated.");
	}

	CloseHandle( plTh->hThread );

	plTh->hThread = NULL;
}

//-----------------------------------------------------------------------------
//	@detail
//		MakeStatusMail
//	@param
//		
//	@return
//		
//	@comment
//		
//-----------------------------------------------------------------------------
static void MakeStatusMail()
{
	MyTmpFileMgr tmpFile;
	if( 0 == tmpFile.Create( g_conf.GetValueI(CONFIG_TMPDIR) ) )
	{
		return;
	}

	char szBuf[1024];

	MyTime t;
	sprintf( szBuf, 
		"Return-Path: <>\r\n"			// 17 octet
		"Date: %s\r\n"					// 36 octet
		"From: MAILER-DAEMON@%s\r\n"	// 22+64 octet
		"To: %s\r\n"					// 6+256 octet
		"Subject: good morning.\r\n"	// 24 octet
		"\r\n"							// 2 octet

		, t.GetLocalFormatRfc1123()		// "Date: %s\r\n"
		, g_conf.GetValueI(CONFIG_ME)	// "From: MAILER-DAEMON@%s\r\n"
		, "Postmaster"					// "To: %s\r\n"
		);

	tmpFile.Append( szBuf );


	// --------------------------------------------------------------
	// Make mail
	// --------------------------------------------------------------
	sprintf( szBuf, 
		"%s\r\n"				// date
		"\r\n"
		"Dear postmaster,\r\n"
		"\r\n"
		"Good morning.\r\n"
		"I am sparrow the MAILER-DAEMON.\r\n"
		"\r\n"
		"Now I'll give you my current config value.\r\n"
		"If all characters are capital letters, It might be default value.\r\n"
		"Please check it.\r\n"
		"\r\n"
		"Yours truly,\r\n"
		"\r\n"
		"Sparrow MAILER-DAEMON@%s (build at %s)\r\n"
		"\r\n"
		
		, t.GetLocalFormatRfc1123()
		, g_conf.GetValueI(CONFIG_ME), __DATE__
		);

	tmpFile.Append( szBuf );
	
	tmpFile.Append( "CONFIG STATUS--------------------------\r\n" );
	SEEKKEY sk = g_conf.SeekToFirst();
	int ii = 0;

	while( NULL != sk )
	{
		const char* pszValue = g_conf.GetValueBySeekKey( sk );
		if( NULL == pszValue )
		{
			pszValue = "(null)";
		}

		if( 254 < STRLEN( sk ) )
			sprintf( szBuf, " %02d:key too long.\r\n", ii );
		else if( 254 < STRLEN( pszValue ) )
			sprintf( szBuf, " %02d:value too long.\r\n", ii );
		else
			sprintf( szBuf, " %02d:%s: %s\r\n", ii++, sk, g_conf.GetValueBySeekKey( sk ) );
		tmpFile.Append( szBuf );
		ii++;
		sk = g_conf.SeekNext( sk );
	}

	// Memory status.
	tmpFile.Append( "MEMORY STATUS--------------------------\r\n" );

	DWORD dwStrageWaterMark = g_conf.GetUINTValueI(CONFIG_STRAGEWATERMARK);
	DWORD dwStrage = dwStrageWaterMark;

	DWORD dwMemoryWaterMark = g_conf.GetUINTValueI(CONFIG_MEMORYWATERMARK);
	DWORD dwMemory = dwMemoryWaterMark;

	StrageCheck( g_conf.GetValueI(CONFIG_HOMEDIR), &dwStrage );
	MemoryCheck( &dwMemory );

	sprintf( szBuf, " StrageAvail:%u(KB), W-Mark:%u(KB)\r\n"
					" MemoryAvali:%u(KB), W-Mark:%u(KB)\r\n"
			, dwStrage/1024, dwStrageWaterMark/1024
			, dwMemory/1024, dwMemoryWaterMark/1024 );
	
	tmpFile.Append( szBuf );
	// rcpthost
	tmpFile.Append( "RCPTHOSTS------------------------------\r\n" );
	
	sk = g_rcpthosts.SeekToFirst();
	if( NULL == sk )
		tmpFile.Append( 
		" ======================\r\n"
		" WARNING! NO RECIPIENT.\r\n"
		" ======================\r\n"
		);
	// else
	ii = 0;
	while( NULL != sk )
	{
		const char* pszValue = g_conf.GetValueBySeekKey( sk );
		if( NULL == pszValue )
		{
			pszValue = "(null)";
		}

		sprintf( szBuf, " %02d:%s: %s\r\n", ii++, sk, g_rcpthosts.GetValueBySeekKey( sk ) );
		tmpFile.Append( szBuf );
		sk = g_rcpthosts.SeekNext( sk );
	}

	char szTmpFileName[MAX_PATH] = {0};
	
	tmpFile.GetFileName( szTmpFileName, MAX_PATH );
	Usr usr;
	MyHashEx hsRcptLocal;
	MyHashFile hsRcptRemote;
	if( 0 == hsRcptLocal.Set("postmaster", "") )
	{
		g_log.Warn( "failed create mail to postmaster. you must check memory." );
		tmpFile.Delete();
		return;
	}

	if( 0 == usr.DeliveryFile( szTmpFileName, &hsRcptLocal, &hsRcptRemote, &g_log ) )
	{
		g_log.Warn( "failed create mail to postmaster. you must check postmaster account." );
	}

	tmpFile.Delete();
}